@@ -1,20 +1,27 @@
+/*
 
+    Copyright (C) 2001 The Kompany
+		  2002-2003	Ilya Konstantinov <kde-devel@future.shiny.co.il>
+		  2002-2003	Marcus Meissner <marcus@jet.franken.de>
+		  2003		Nadeem Hasan <nhasan@nadmm.com>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+*/
 #include <qlabel.h>
-#include <qlineedit.h>
-#include <qcombobox.h>
 #include <qlayout.h>
-#include <qradiobutton.h>
-#include <qpushbutton.h>
-#include <qvbox.h>
-#include <qvgroupbox.h>
-#include <qvbuttongroup.h>
-#include <qgrid.h>
-#include <qwidgetstack.h>
-#include <qcheckbox.h>
-#include <qdir.h>
-#include <qwhatsthis.h>
-#include <qregexp.h>
-#include <qpopupmenu.h>
 
 #include <kgenericfactory.h>
 #include <ksimpleconfig.h>
@@ -24,9 +31,8 @@
 #include <kiconview.h>
 #include <kdialog.h>
 #include <klocale.h>
-#include <kglobal.h>
 #include <ktoolbar.h>
-
+#include <kpopupmenu.h>
 #include <kprotocolinfo.h>
 #include <kdebug.h>
 
@@ -42,7 +48,7 @@
 #define tocstr(x) ((char *)((x).latin1()))
 
 typedef KGenericFactory<KKameraConfig, QWidget> KKameraConfigFactory;
-K_EXPORT_COMPONENT_FACTORY( kcm_kamera, KKameraConfigFactory( "kcmkamera" ) );
+K_EXPORT_COMPONENT_FACTORY( kcm_kamera, KKameraConfigFactory( "kcmkamera" ) )
 
 // --------------- Camera control center module widget ---
 
@@ -51,7 +57,7 @@ KKameraConfig *KKameraConfig::m_instance = NULL;
 KKameraConfig::KKameraConfig(QWidget *parent, const char *name, const QStringList &)
 	: KCModule(KKameraConfigFactory::instance(), parent, name)
 {
-	m_devicePopup = new QPopupMenu(this);
+	m_devicePopup = new KPopupMenu(this);
 	m_actions = new KActionCollection(this);
 	m_config = new KSimpleConfig(KProtocolInfo::config("camera"));
 	
@@ -64,9 +70,6 @@ KKameraConfig::KKameraConfig(QWidget *parent, const char *name, const QStringLis
 
 		displayGPSuccessDialogue();
 
-		// autodetect new cameras
-		autoDetect();
-
 		// load existing configuration
 		load();
 
@@ -106,8 +109,14 @@ void KKameraConfig::displayGPSuccessDialogue(void)
 	
 	// create list of devices
 	m_deviceSel = new KIconView(this);
-	connect(m_deviceSel, SIGNAL(rightButtonClicked(QIconViewItem *, const QPoint &)), SLOT(slot_deviceMenu(QIconViewItem *, const QPoint &)));
-	connect(m_deviceSel, SIGNAL(selectionChanged(QIconViewItem *)), SLOT(slot_deviceSelected(QIconViewItem *)));
+
+	connect(m_deviceSel, SIGNAL(rightButtonClicked(QIconViewItem *, const QPoint &)),
+		SLOT(slot_deviceMenu(QIconViewItem *, const QPoint &)));
+	connect(m_deviceSel, SIGNAL(doubleClicked(QIconViewItem *)),
+		SLOT(slot_configureCamera()));
+	connect(m_deviceSel, SIGNAL(selectionChanged(QIconViewItem *)),
+		SLOT(slot_deviceSelected(QIconViewItem *)));
+
 	m_deviceSel->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
 	
 	// create actions
@@ -142,7 +151,7 @@ void KKameraConfig::populateDeviceListView(void)
 	CameraDevicesMap::Iterator it;
 	for (it = m_devices.begin(); it != m_devices.end(); it++) {
 		if (it.data()) {
-			new QIconViewItem(m_deviceSel, it.key(), KGlobal::iconLoader()->loadIcon("camera", KIcon::Desktop));
+			new QIconViewItem(m_deviceSel, it.key(), DesktopIcon("camera"));
 		}
 	}
 	slot_deviceSelected(m_deviceSel->currentItem());
@@ -159,17 +168,32 @@ void KKameraConfig::save(void)
 	m_config->sync();
 }
 
-void KKameraConfig::autoDetect(void)
+void KKameraConfig::load(void)
 {
-	m_cancelPending = false;
-
 	QStringList groupList = m_config->groupList();
-
+	QStringList::Iterator it;
         int i, count;
         CameraList list;
         CameraAbilitiesList *al;
         GPPortInfoList *il;
         const char *model, *value;
+	KCamera *kcamera;
+	
+	for (it = groupList.begin(); it != groupList.end(); it++) {
+		if (*it != "<default>")	{
+			m_config->setGroup(*it);
+			if (m_config->readEntry("Path").contains("usb:"))
+				continue;
+
+			kcamera = new KCamera(*it,m_config->readEntry("Path"));
+			connect(kcamera, SIGNAL(error(const QString &)), SLOT(slot_error(const QString &)));
+			connect(kcamera, SIGNAL(error(const QString &, const QString &)), SLOT(slot_error(const QString &, const QString &)));
+			kcamera->load(m_config);
+			m_devices[*it] = kcamera;
+		}
+	}
+	m_cancelPending = false;
+
 
         gp_abilities_list_new (&al);
         gp_abilities_list_load (al, m_context);
@@ -180,35 +204,30 @@ void KKameraConfig::autoDetect(void)
         gp_port_info_list_free (il);
 
         count = gp_list_count (&list);
+
+	QMap<QString,QString>	ports, names;
 	
 	for (i = 0 ; i<count ; i++) {
 		gp_list_get_name  (&list, i, &model);
 		gp_list_get_value (&list, i, &value);
 
-		if (groupList.contains(model))
-			continue;
-		kdDebug() << "Adding " << model << " at " << value << endl;
-		m_config->setGroup(model);
-		m_config->writeEntry("Model",model);
-		m_config->writeEntry("Path",value);
+		ports[value] = model;
+		if (!strcmp(value,"usb:"))
+			names[model] = value;
 	}
-}
+	if (ports.contains("usb:") && names[ports["usb:"]]!="usb:")
+		ports.remove("usb:");
 
-void KKameraConfig::load(void)
-{
-	QStringList groupList = m_config->groupList();
-	QStringList::Iterator it;
-	
-	for (it = groupList.begin(); it != groupList.end(); it++) {
-		if (*it != "<default>")	{
-			KCamera *kcamera = new KCamera(*it);
-			connect(kcamera, SIGNAL(error(const QString &)), SLOT(slot_error(const QString &)));
-			connect(kcamera, SIGNAL(error(const QString &, const QString &)), SLOT(slot_error(const QString &, const QString &)));
-			kcamera->load(m_config);
-			m_devices[*it] = kcamera;
-		}
+	QMap<QString,QString>::iterator portit;
+
+	for (portit = ports.begin() ; portit != ports.end(); portit++) {
+		/* kdDebug() << "Adding USB camera: " << portit.data() << " at " << portit.key() << endl; */
+
+		kcamera = new KCamera(portit.data(),portit.key());
+		connect(kcamera, SIGNAL(error(const QString &)), SLOT(slot_error(const QString &)));
+		connect(kcamera, SIGNAL(error(const QString &, const QString &)), SLOT(slot_error(const QString &, const QString &)));
+		m_devices[portit.data()] = kcamera;
 	}
-	
 	populateDeviceListView();
 }
 
@@ -242,7 +261,7 @@ void KKameraConfig::afterCameraOperation(void)
 QString KKameraConfig::suggestName(const QString &name)
 {
 	QString new_name = name;
-	new_name.replace(QRegExp("/"), ""); // we cannot have a slash in a URI's host
+	new_name.replace("/", ""); // we cannot have a slash in a URI's host
 
 	if (!m_devices.contains(new_name)) return new_name;
 	
@@ -258,7 +277,7 @@ QString KKameraConfig::suggestName(const QString &name)
 
 void KKameraConfig::slot_addCamera()
 {
-	KCamera *m_device = new KCamera(QString::null);
+	KCamera *m_device = new KCamera(QString::null,QString::null);
 	connect(m_device, SIGNAL(error(const QString &)), SLOT(slot_error(const QString &)));
 	connect(m_device, SIGNAL(error(const QString &, const QString &)), SLOT(slot_error(const QString &, const QString &)));
 	KameraDeviceSelectDialog dialog(this, m_device);
@@ -316,7 +335,7 @@ void KKameraConfig::slot_cameraSummary()
 	if (m_devices.contains(name)) {
 		KCamera *m_device = m_devices[name];
 		summary = m_device->summary();
-		if (QString::null != summary) {
+		if (!summary.isNull()) {
 			KMessageBox::information(this, summary);
 		}
 	}
@@ -334,7 +353,6 @@ void KKameraConfig::slot_cancelOperation()
 void KKameraConfig::slot_deviceMenu(QIconViewItem *item, const QPoint &point)
 {
 	if (item) {
-		QString name = item->text();
 		m_devicePopup->clear();
 		m_actions->action("camera_test")->plug(m_devicePopup);
 		m_actions->action("camera_remove")->plug(m_devicePopup);
@@ -352,14 +370,14 @@ void KKameraConfig::slot_deviceSelected(QIconViewItem *item)
 	m_actions->action("camera_summary")->setEnabled(item);
 }
 
-void KKameraConfig::cbGPIdle(GPContext *context, void *data)
+void KKameraConfig::cbGPIdle(GPContext * /*context*/, void * /*data*/)
 {
-	KKameraConfig *self( reinterpret_cast<KKameraConfig*>(data) );
+	/*KKameraConfig *self( reinterpret_cast<KKameraConfig*>(data) );*/
 
 	qApp->processEvents();
 }
 
-GPContextFeedback KKameraConfig::cbGPCancel(GPContext *context, void *data)
+GPContextFeedback KKameraConfig::cbGPCancel(GPContext * /*context*/, void *data)
 {
 	KKameraConfig *self( reinterpret_cast<KKameraConfig*>(data) );
 
@@ -376,7 +394,7 @@ GPContextFeedback KKameraConfig::cbGPCancel(GPContext *context, void *data)
 
 QString KKameraConfig::quickHelp() const
 {
-	return i18n("<h1>Kamera Configuration</h1>\n"
+	return i18n("<h1>Digital Camera</h1>\n"
 	  "This module allows you to configure support for your digital camera.\n"
 	  "You would need to select the camera's model and the port it is connected\n"
 	  "to on your computer (e.g. USB, Serial, Firewire). If your camera doesn't\n"
