@@ -1,4 +1,25 @@
-#include <qobject.h>
+/*
+
+    Copyright (C) 2001 The Kompany
+		  2002-2003	Ilya Konstantinov <kde-devel@future.shiny.co.il>
+		  2002-2003	Marcus Meissner <marcus@jet.franken.de>
+		  2003		Nadeem Hasan <nhasan@nadmm.com>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+*/
 #include <qlayout.h>
 #include <qwidgetstack.h>
 #include <qvbuttongroup.h>
@@ -16,7 +37,10 @@
 #include <kmessagebox.h>
 #include <kdebug.h>
 
-#include <gphoto2.h>
+extern "C" {
+	#include <gphoto2.h>
+}
+
 #include "kamera.h"
 #include "kameraconfigdialog.h"
 #include "kameradevice.moc"
@@ -30,9 +54,11 @@ static const int INDEX_SERIAL = 1;
 static const int INDEX_USB= 3;
 static GPContext *glob_context = 0;
 
-KCamera::KCamera(const QString &name)
+KCamera::KCamera(const QString &name, const QString &path)
 {
-	m_name = name;
+	m_name	= name;
+	m_model	= name;
+	m_path	= path;
 	m_camera = NULL;
 }
 
@@ -167,8 +193,10 @@ bool KCamera::test()
 void KCamera::load(KConfig *config)
 {
 	config->setGroup(m_name);
-	m_model = config->readEntry("Model");
-	m_path = config->readEntry("Path");
+	if (m_model == QString::null)
+		m_model = config->readEntry("Model");
+	if (m_path == QString::null)
+		m_path = config->readEntry("Path");
 	invalidateCamera();
 }
 
@@ -213,7 +241,7 @@ void KCamera::invalidateCamera()
 	}
 }
 
-bool KCamera::isTestable()
+bool KCamera::isTestable() const
 {
 	return true;
 }
@@ -243,46 +271,38 @@ CameraAbilities KCamera::abilities()
 // ---------- KameraSelectCamera ------------
 
 KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *device)
-	: KDialogBase(parent, "kkameradeviceselect", true, i18n("Select Camera Device"), Ok | Cancel)
+	: KDialogBase(parent, "kkameradeviceselect", true, i18n("Select Camera Device"), Ok | Cancel, Ok, true)
 {
 	m_device = device;
-	connect(m_device, SIGNAL(error(const QString &)), SLOT(slot_error(const QString &)));
-	connect(m_device, SIGNAL(error(const QString &, const QString &)), SLOT(slot_error(const QString &, const QString &)));
+	connect(m_device, SIGNAL(error(const QString &)),
+		SLOT(slot_error(const QString &)));
+	connect(m_device, SIGNAL(error(const QString &, const QString &)),
+		SLOT(slot_error(const QString &, const QString &)));
 
 	QWidget *page = new QWidget( this );
 	setMainWidget(page);
 
 	// a layout with vertical boxes
-	QHBoxLayout *topLayout = new QHBoxLayout(page, KDialog::marginHint(), KDialog::spacingHint());
-	topLayout->setAutoAdd(true);
+	QHBoxLayout *topLayout = new QHBoxLayout(page, 0, KDialog::spacingHint());
 
 	// the models list
 	m_modelSel = new KListView(page);
+	topLayout->addWidget( m_modelSel );
 	m_modelSel->addColumn(i18n("Supported Cameras"));
 	m_modelSel->setColumnWidthMode(0, QListView::Maximum);
-	connect(m_modelSel, SIGNAL(selectionChanged(QListViewItem *)), SLOT(slot_setModel(QListViewItem *)));
+	connect(m_modelSel, SIGNAL(selectionChanged(QListViewItem *)),
+        SLOT(slot_setModel(QListViewItem *)));
 	// make sure listview only as wide as it needs to be
-	m_modelSel->setSizePolicy(QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred));
+	m_modelSel->setSizePolicy(QSizePolicy(QSizePolicy::Maximum,
+		QSizePolicy::Preferred));
 
-	QWidget *right = new QWidget(page);
-	QVBoxLayout *rightLayout = new QVBoxLayout(right);
-	rightLayout->setSpacing(10);
+	QVBoxLayout *rightLayout = new QVBoxLayout(0L, 0, KDialog::spacingHint());
+	topLayout->addLayout( rightLayout );
 
-	/*
-	QGrid *grid = new QGrid(2, right);
-	new QLabel(i18n("Name:"), grid);
-	m_nameEdit = new QLineEdit(m_device->name(), grid);
-	rightLayout->addWidget(grid);
-	*/
-
-	m_portSelectGroup = new QVButtonGroup(i18n("Port"), right);
+	m_portSelectGroup = new QVButtonGroup(i18n("Port"), page);
 	rightLayout->addWidget(m_portSelectGroup);
-	QVGroupBox *portSettingsGroup = new QVGroupBox(i18n("Port Settings"), right);
-	rightLayout->addWidget(portSettingsGroup);
-
-	QGrid *grid = new QGrid(2, right);
-	rightLayout->addWidget(grid);
-	grid->setSpacing(5);
+	m_portSettingsGroup = new QVGroupBox(i18n("Port Settings"), page);
+	rightLayout->addWidget(m_portSettingsGroup);
 
 	// Create port type selection radiobuttons.
 	m_serialRB = new QRadioButton(i18n("Serial"), m_portSelectGroup);
@@ -292,24 +312,24 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 	m_portSelectGroup->insert(m_USBRB, INDEX_USB);
 	QWhatsThis::add(m_USBRB, i18n("If this option is checked, the camera would have to be connected to one of the USB slots in your computer or USB hub."));
 	// Create port settings widget stack
-	m_settingsStack = new QWidgetStack(portSettingsGroup);
-	connect(m_portSelectGroup, SIGNAL(clicked(int)), m_settingsStack, SLOT(raiseWidget(int)));
+	m_settingsStack = new QWidgetStack(m_portSettingsGroup);
+	connect(m_portSelectGroup, SIGNAL(clicked(int)),
+		m_settingsStack, SLOT(raiseWidget(int)));
 
 	// none tab
-	m_settingsStack->addWidget(new
-		QLabel(i18n("No port type selected."),
+	m_settingsStack->addWidget(new QLabel(i18n("No port type selected."),
 		m_settingsStack), INDEX_NONE);
 
 	// serial tab
-	grid = new QGrid(2, m_settingsStack);
-	grid->setSpacing(5);
+	QGrid *grid = new QGrid(2, m_settingsStack);
+	grid->setSpacing(KDialog::spacingHint());
 	new QLabel(i18n("Port:"), grid);
 	m_serialPortCombo = new QComboBox(TRUE, grid);
 	QWhatsThis::add(m_serialPortCombo, i18n("Here you should choose the serial port you connect the camera to."));
 	m_settingsStack->addWidget(grid, INDEX_SERIAL);
 
 	grid = new QGrid(2, m_settingsStack);
-	grid->setSpacing(5);
+	grid->setSpacing(KDialog::spacingHint());
 	new QLabel(i18n("Port"), grid);
 
 	m_settingsStack->addWidget(new
@@ -319,7 +339,7 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 	// query gphoto2 for existing serial ports
 	GPPortInfoList *list;
 	GPPortInfo info;
-	int gphoto_ports;
+	int gphoto_ports=0;
 	gp_port_info_list_new(&list);
 	if(gp_port_info_list_load(list) >= 0) {
 		gphoto_ports = gp_port_info_list_count(list);
@@ -333,11 +353,14 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 	gp_port_info_list_free(list);
 
 	// add a spacer
-	rightLayout->addItem( new QSpacerItem(0, 0, QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding) );
+	rightLayout->addStretch();
 
 	populateCameraListView();
 	load();
-        enableButtonOK(false );
+
+	enableButtonOK(false );
+    m_portSelectGroup->setEnabled( false );
+    m_portSettingsGroup->setEnabled( false );
 }
 
 bool KameraDeviceSelectDialog::populateCameraListView()
@@ -391,12 +414,15 @@ void KameraDeviceSelectDialog::load()
 			m_modelSel->setSelected(modelItem, true);
 			m_modelSel->ensureItemVisible(modelItem);
 		}
-	} while ( modelItem = modelItem->nextSibling());
+	} while ( ( modelItem = modelItem->nextSibling() ) );
 }
 
 void KameraDeviceSelectDialog::slot_setModel(QListViewItem *item)
 {
-    enableButtonOK( item );
+    enableButtonOK(true);
+    m_portSelectGroup->setEnabled(true);
+    m_portSettingsGroup->setEnabled(true);
+
     QString model = item->text(0);
 
 	CameraAbilities abilities;
