@@ -21,16 +21,19 @@
 #include <kviewpart.h>
 
 #include "bigEndianByteReader.h"
+#include "documentPage.h"
 #include "dviFile.h"
 #include "fontpool.h"
+#include "infodialog.h"
 #include "psgs.h"
-#include "selection.h"
 
+
+class documentWidget;
 class dviWindow;
 class fontProgressDialog;
 class infoDialog;
 class KAction;
-class KEdFind;
+class KDVIMultiPage;
 class KPrinter;
 class KProcess;
 class KProgressDialog;
@@ -39,16 +42,6 @@ class TeXFontDefinition;
 
 extern const int MFResolutions[];
 
-class DVI_Hyperlink {
- public:
-  DVI_Hyperlink() {}
-  DVI_Hyperlink(Q_UINT32 bl, QRect re, QString lT): baseline(bl), box(re), linkText(lT) {}
-
-  Q_UINT32 baseline;
-  QRect    box;
-  QString  linkText;
-};
-
 class DVI_Anchor {
  public:
   DVI_Anchor() {}
@@ -99,58 +92,41 @@ struct drawinf {
 };
 
 
-// This class contains everything one needs dviwin needs to know about
-// a certain page. 
 
-class pageData
-{
- public:
-  Q_UINT16   pageNumber;
-  
-  QPixmap    *pixmap;
-  
-  // List of source-hyperlinks in the current page. This vector is
-  // generated when the current page is drawn.
-  QValueVector<DVI_Hyperlink> sourceHyperLinkList;
-
-  QValueVector<DVI_Hyperlink> textLinkList; // List of text in the window
-  QValueVector<DVI_Hyperlink> hyperLinkList; // List of ordinary hyperlinks
-};
-
-
-class dviWindow : public QWidget, bigEndianByteReader
+class dviWindow : public QObject, bigEndianByteReader
 {
   Q_OBJECT
 
 public:
-  dviWindow( double zoom, QWidget *parent=0, const char *name=0 );
+  dviWindow(QWidget *parent);
   ~dviWindow();
 
+  documentPage  *currentlyDrawnPage;
+
+  QSize         sizeOfPage(Q_UINT16 page) {return currentlyDrawnPixmap.size();};
+
   class dvifile *dviFile;
 
-  void          showInfo();
+  void          setPrefs(bool flag_showPS, bool flag_showHyperLinks, const QString &editorCommand, 
+			 unsigned int MetaFontMode, bool makePK, bool useType1Fonts, bool useFontHints );
+
+  bool          supportsTextSearch(void) {return true;};
+
   void          exportPS(QString fname = QString::null, QString options = QString::null, KPrinter *printer = 0);
   void          exportPDF();
-  void          exportText();
 
   void          changePageSize(void);
   int		totalPages(void);
-  void		setShowPS( bool flag );
   bool		showPS(void) { return _postscript; };
   int		curr_page(void) { return current_page+1; };
-  void		setShowHyperLinks( bool flag );
   bool		showHyperLinks(void) { return _showHyperLinks; };
-  void		setEditorCommand( const QString &command )  { editorCommand = command; };
   void		setPaper(double w, double h);
   static bool   correctDVI(const QString &filename);
   
   // for the preview
-  QPixmap      *pix() { return currentlyDrawnPage.pixmap; };
+  QPixmap      *pix() { if (currentlyDrawnPage) return currentlyDrawnPage->getPixmap(); return 0; };
 
   // These should not be public... only for the moment
-  void          mousePressEvent ( QMouseEvent * e );
-  void          mouseMoveEvent ( QMouseEvent * e );
-  void          mouseReleaseEvent ( QMouseEvent * e );
   void          read_postamble(void);
   void          draw_part(double current_dimconv, bool is_vfmacro);
   void          set_vf_char(unsigned int cmd, unsigned int ch);
@@ -166,31 +142,21 @@ public:
   void          html_anchor_end(void);
   void          draw_page(void);
 
-  fontPool  *font_pool;
+  double        xres;         // horizontal resolution of the display device in dots per inch.
+  double        paper_width_in_cm;  // paper width in centimeters
+  double        paper_height_in_cm; // paper height in centimeters
 
-  double       xres;         // horizontal resolution of the display device in dots per inch.
-  double       paper_width_in_cm;  // paper width in centimeters
-  double       paper_height_in_cm; // paper height in centimeters
-
-  selection    DVIselection;
 
  /** Reference part of the URL which describes the filename. */
  QString        reference;
 
- QString        searchText;
- KAction        *findNextAction;
- KAction        *findPrevAction;
 
 public slots:
+  void          showInfo(void);
+  void          handleLocalLink(const QString &linkText);
+  void          handleSRCLink(const QString &linkText, QMouseEvent *e, documentWidget *widget);
+
   void          embedPostScript(void);
-
-  void          selectAll(void);
-  void          copyText(void);
-  void          showFindTextDialog(void);
-  void          findText(void);
-  void          findNextText(void);
-  void          findPrevText(void);
-
   void          abortExternalProgramm(void);
   bool		setFile(const QString &fname, const QString &ref = QString::null, bool sourceMarker=true);
 
@@ -198,17 +164,9 @@ public slots:
       in dviWindow::mouseMoveEvent(), see the explanation there. */
   void          clearStatusBar(void);
 
-  /** Displays the page of the first argument */
-  void		gotoPage(unsigned int page);
-
-  /** Displays the page of the first argument, and blinks the display
-      at the vertical offset vflashOffset. This is used when the user
-      clicks on a hyperlink: the target of the jump flashes so that
-      the user can locate it more easily. */
-  void		gotoPage(int page, int vflashOffset);
   double	setZoom(double zoom);
   double        zoom() { return _zoom; };
-  void		drawPage();
+  void		drawPage(documentPage *page);
  
   /** Slots used in conjunction with external programs */
   void          dvips_output_receiver(KProcess *, char *buffer, int buflen);
@@ -225,6 +183,11 @@ public slots:
   void          all_fonts_loaded(fontPool *);
 
 signals:
+  /** Emitted if the status of this class changed internally so that
+      all associated widgets should be repainted, e.g. if a lengthy
+      network download finished. */
+  void          needsRepainting();
+
   /** Emitted to indicate that the prescan phase has ended */
   void          prescanDone();
 
@@ -233,23 +196,24 @@ signals:
       page and the coordinates specified. */
   void          request_goto_page(int page, int y);
 
-  /** Emitted to indicate the the contents of the widget has changed
-      and that the tumbnail image should be updated. */
-  void          contents_changed(void);
-
   /** Passed through to the top-level kpart. */
   void setStatusBarText( const QString& );
 
   /** To be passed through to the kmultipage */
   void documentSpecifiedPageSize(const pageSize &size);
 
-protected:
- void          paintEvent(QPaintEvent *ev);
 
 private:
+  fontPool      font_pool;
+  infoDialog    *info;
+
+  QWidget       *parentWidget;
+
+
   // @@@ explanation
   void          prescan(parseSpecials specialParser);
   void          prescan_embedPS(char *cp, Q_UINT8 *);
+  void          prescan_removePageSizeInfo(char *cp, Q_UINT8 *);
   void          prescan_parseSpecials(char *cp, Q_UINT8 *);
   void          prescan_ParsePapersizeSpecial(QString cp);
   void          prescan_ParseBackgroundSpecial(QString cp);
@@ -288,8 +252,7 @@ private:
      *this. */
   QTimer        clearStatusBarTimer;
   
-  pageData      currentlyDrawnPage;
-  
+  QPixmap       currentlyDrawnPixmap;
   
   // List of source-hyperlinks on all pages. This vector is generated
   // when the DVI-file is first loaded, i.e. when draw_part is called
@@ -317,22 +280,7 @@ private:
   
   /** The global color is to be used when the color stack is empty */
   QColor              globalColor;
-  
-  /** Methods and counters used for the animation to mark the target of
-      an hyperlink. */
-  int               timerIdent;
-  void              timerEvent( QTimerEvent *e );
-  int               animationCounter;
-  int               flashOffset;
-  
-  /** Methods and classes concerned with the find functionality and
-      with selecting text */
-  class KEdFind    *findDialog;
-  QPoint            firstSelectedPoint;
-  QRect             selectedRectangle;
-  
-  infoDialog       *info;
-  
+    
   /** If PostScriptOutPutFile is non-zero, then no rendering takes
       place. Instead, the PostScript code which is generated by the
       \special-commands is written to the PostScriptString */
@@ -356,15 +304,8 @@ private:
   /** List of anchors in a document */
   QMap<QString, DVI_Anchor> anchorList;
   
-  double            fontPixelPerDVIunit() {return dviFile->getCmPerDVIunit() * MFResolutions[font_pool->getMetafontMode()]/2.54;};
-  
-  int		   ChangesPossible;
   unsigned int	   current_page;
   
-  /** Indicates if the current page is already drawn (=1) or not
-      (=0). */
-  char              is_current_page_drawn;
-  
   // Zoom factor. 1.0 means "100%"
   double            _zoom;
 
