@@ -1,6 +1,6 @@
    /*
 
-    $Id: kfax.cpp,v 1.90 2003/11/11 01:55:33 bmeyer Exp $
+    $Id: kfax.cpp,v 1.97.2.2 2004/11/09 19:55:50 deller Exp $
 
     Copyright (C) 1997 Bernd Johannes Wuebben
                        wuebben@math.cornell.edu
@@ -34,12 +34,17 @@
 #include <qfile.h>
 #include <qstrlist.h>
 #include <qtimer.h>
+#include <qpopupmenu.h>
+#include <qpainter.h>
+#include <qpaintdevicemetrics.h>
+#include <qbitmap.h>
 
 #include <klocale.h>
 #include <kaboutdata.h>
 #include <kstandarddirs.h>
 #include <kiconloader.h>
 #include <kfiledialog.h>
+#include <kfilemetainfo.h>
 #include <kstdaccel.h>
 #include <kconfig.h>
 #include <kmenubar.h>
@@ -51,12 +56,35 @@
 #include <kstdaction.h>
 #include <kdebug.h>
 #include <kurldrag.h>
-#include <kkeydialog.h>
+#include <kstatusbar.h>
+#include <kaction.h>
+#include <kprocess.h>
+#include <kprinter.h>
+#include <kio/job.h>
+#include <kdebug.h>
 
+#include "faxexpand.h"
 #include "kfax.h"
-#include "kfax.moc"
 #include "version.h"
 #include "viewfax.h"
+#include "options.h"
+
+#include <X11/X.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/Xatom.h>
+#include <X11/keysym.h>
+#include <X11/keysymdef.h>
+#include <X11/cursorfont.h>
+
+// StatusBar field IDs
+#define ID_LINE_COLUMN 1
+#define ID_INS_OVR 2
+#define ID_GENERAL 3
+#define ID_FNAME 4
+#define ID_TYPE 5
+#define ID_PAGE_NO 6
+
 
 TopLevel *toplevel;
 
@@ -64,19 +92,12 @@ extern  int GetImage(struct pagenode *pn);
 
 void 	TurnFollowing(int How, struct pagenode *pn);
 void 	handle_X_event(XEvent event);
-int 	notetiff(const char *name);
 void 	ShowLoop(void);
 void 	SetupDisplay();
 void 	mysighandler(int sig);
 void    setFaxDefaults();
 
 void 	parse(char* buf, char** args);
-int     copyfile(const char* to,char* from);
-
-extern  "C"{
-int    fax2psmain(const char* faxtiff_file,FILE* psoutput, float width,float height,int scale);
-int    fax2tiffmain(const char* inputfile, const char* outputfile,int bitorder,int stretchit,int type);
-}
 
 extern void g31expand(struct pagenode *pn, drawfunc df);
 extern void g32expand(struct pagenode *pn, drawfunc df);
@@ -84,11 +105,7 @@ extern void g4expand(struct pagenode *pn, drawfunc df);
 
 #define PATIENCE 100000
 
-
-#undef 	min
-#undef 	max
-#define min(a,b)	((a)<(b)?(a):(b))
-#define max(a,b)	((a)>(b)?(a):(b))
+static char KFAX_FILETYPES[] = "image/fax-g3 image/tiff";
 
 int 	ExpectConfNotify = 1;
 
@@ -111,14 +128,12 @@ extern 	void FreeImage(XImage *Image);
 
 extern 	XImage *Image, *Images[MAXZOOM];
 
-int 	xpos= 0, ox= 0;	/* x, old x, offset x, new xpos*/
-int 	ypos= 0, oy = 0;	/* y, old y, offset y, new y */
-int 	offx = 0;
-int 	offy = 0;
-int 	nx = 0;
-int 	ny = 0;
+int 	xpos, ox;   /* x, old x, offset x, new xpos*/
+int 	ypos, oy;  /* y, old y, offset y, new y */
+int 	offx, offy;
+int 	nx, ny;
 
-int oz, Resize = 0, Refresh = 0;	/* old zoom, window size changed,
+int oz, Resize, Refresh;		/* old zoom, window size changed,
 					   needs updating */
 int PaneWidth, PaneHeight;		/* current size of our window */
 int AbsX, AbsY;				/* absolute position of centre of window */
@@ -176,16 +191,14 @@ TopLevel::TopLevel (QWidget *, const char *name)
 {
   setMinimumSize (100, 100);
 
-  printdialog = NULL;
   buttondown = false;
 
   setupMenuBar();
   setupStatusBar();
-  setStandardToolBarMenuEnabled( true );
-
   updateActions();
 
-  createGUI();
+  resize(550,400);
+  setupGUI();
 
   readSettings();
 
@@ -206,12 +219,10 @@ TopLevel::TopLevel (QWidget *, const char *name)
   connect( hsb, SIGNAL(valueChanged(int)), SLOT(scrollHorz(int)) );
   hsb->hide();
 
-
   setCentralWidget(faxqtwin);
 
   setAcceptDrops(true);
 
-  resize(editor_width,editor_height);
   show();
 }
 
@@ -231,7 +242,6 @@ void TopLevel::setupMenuBar()
           actionCollection() );
   actPrint = KStdAction::print( this, SLOT( print() ), actionCollection() );
   KStdAction::quit( this, SLOT( close() ), actionCollection() );
-  KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
   actAdd = new KAction( i18n( "&Add..." ), "filenew", KShortcut(), this,
       SLOT( faxAdd() ), actionCollection(), "file_add_fax" );
 
@@ -260,19 +270,9 @@ void TopLevel::setupMenuBar()
       actionCollection() );
 
   // Settings menu
-  createStandardStatusBarAction();
-  setStandardToolBarMenuEnabled(true);
-
-
-  createStandardStatusBarAction();
   KStdAction::preferences( this, SLOT( faxoptions() ), actionCollection() );
 }
 
-void TopLevel::slotConfigureKeys()
-{
-  KKeyDialog::configure( actionCollection(), this );
-}
-
 void TopLevel::setupStatusBar()
 {
   statusbar = statusBar();
@@ -281,14 +281,14 @@ void TopLevel::setupStatusBar()
   statusbar->insertFixedItem(i18n("Res: XXXXX"), ID_GENERAL);
   statusbar->insertFixedItem(i18n("Type: XXXXXXX"), ID_TYPE);
   statusbar->insertFixedItem(i18n("Page: XX of XX"), ID_PAGE_NO);
-  statusbar->insertItem("", ID_FNAME, 1);
+  statusbar->insertItem(QString::null, ID_FNAME, 1);
   statusbar->setItemAlignment( ID_FNAME, AlignLeft );
 
-  statusbar->changeItem("", ID_INS_OVR);
-  statusbar->changeItem("", ID_GENERAL);
-  statusbar->changeItem("", ID_TYPE);
-  statusbar->changeItem("", ID_PAGE_NO);
-  statusbar->changeItem("", ID_FNAME);
+  statusbar->changeItem(QString::null, ID_INS_OVR);
+  statusbar->changeItem(QString::null, ID_GENERAL);
+  statusbar->changeItem(QString::null, ID_TYPE);
+  statusbar->changeItem(QString::null, ID_PAGE_NO);
+  statusbar->changeItem(QString::null, ID_FNAME);
 }
 
 void TopLevel::readSettings()
@@ -301,10 +301,6 @@ void TopLevel::readSettings()
 
   config->setGroup("General Options");
 
-  lastDir = config->readPathEntry("DefaultPath", QDir::currentDirPath());
-  editor_width = config->readNumEntry("Width", 550);
-  editor_height = config->readNumEntry("Height", 400);
-
   config->setGroup("Fax Options");
 
   fop.width = config->readNumEntry("width", 1728);
@@ -318,17 +314,6 @@ void TopLevel::readSettings()
   fop.raw = config->readNumEntry("raw", 3);
 
   setFaxDefaults();
-
-  config->setGroup("Printing");
-
-  pi.cmd = config->readPathEntry("PrintCommand", "lpr");
-  pi.file = config->readPathEntry("PrintFile");
-  pi.lpr = config->readNumEntry("print", 1);
-  pi.scale = config->readNumEntry("scale", 0);
-  pi.margins = config->readNumEntry("margins", 0);
-  pi.xmargin = config->readDoubleNumEntry("xmargin", 0.0);
-  pi.ymargin = config->readDoubleNumEntry("ymargin", 0.0);
-  pi.pagesize = config->readEntry("PageSize",US_LETTER);
 }
 
 void TopLevel::updateActions()
@@ -375,10 +360,6 @@ void TopLevel::writeSettings()
 
   config->setGroup("General Options");
 
-  config->writePathEntry("DefaultPath", lastDir);
-  config->writeEntry("Width", this->width());
-  config->writeEntry("Height", this->height());
-
   config->setGroup("Fax Options");
 
   config->writeEntry("resauto",fop.resauto);
@@ -391,22 +372,12 @@ void TopLevel::writeSettings()
   config->writeEntry("lsb",fop.lsbfirst);
   config->writeEntry("raw",fop.raw);
 
-  config->setGroup("Printing");
-
-  config->writePathEntry("PrintCommand",pi.cmd);
-  config->writePathEntry("PrintFile",pi.file);
-  config->writeEntry("print",pi.lpr);
-  config->writeEntry("scale",pi.scale);
-  config->writeEntry("PageSize",pi.pagesize);
-  config->writeEntry("margins",pi.margins);
-  config->writeEntry("xmargin", pi.xmargin);
-  config->writeEntry("ymargin", pi.ymargin);
   config->sync();
 }
 
 void TopLevel::faxOpen()
 {
-  KURL url = KFileDialog::getOpenURL(lastDir, "*");
+  KURL url = KFileDialog::getOpenURL(QString::null, KFAX_FILETYPES);
 
   faxOpen( url );
 
@@ -428,7 +399,7 @@ void TopLevel::faxOpen( const KURL & url )
 
 void TopLevel::faxAdd()
 {
-  KURL url = KFileDialog::getOpenURL(lastDir, "*");
+  KURL url = KFileDialog::getOpenURL(QString::null, KFAX_FILETYPES);
 
   faxAdd( url );
 
@@ -440,11 +411,6 @@ void TopLevel::faxAdd( const KURL & url )
   if (!url.isValid())
     return;
 
-  if( url.isLocalFile() )
-  {
-    lastDir = url.directory();
-  }
-
   openNetFile(url);
 
   updateGoActions();
@@ -457,16 +423,11 @@ void TopLevel::faxSave()
 
 void TopLevel::faxSaveAs()
 {
-  fileURL = KFileDialog::getOpenURL(lastDir, "*");
+  fileURL = KFileDialog::getSaveURL(QString::null, KFAX_FILETYPES);
 
   if (fileURL.isEmpty())
     return;
 
-  if( fileURL.isLocalFile() )
-  {
-    lastDir =  fileURL.directory();
-  }
-
   faxSave();
 
   actRecent->addURL( fileURL );
@@ -531,11 +492,33 @@ void TopLevel::actualSize()
   zoom( oz );
 }
 
+void loadfile(QString filename)
+{
+  QSize dpi(200,200);
+
+  KFileMetaInfo metaInfo(filename);
+  if (metaInfo.isValid() && metaInfo.item("Resolution").isValid())
+  {
+    QSize s = metaInfo.item("Resolution").value().toSize();
+    if (s.width() >= 100 && s.height() >= 100)
+	dpi = s;
+  }
+
+  (void) notetiff(QFile::encodeName(filename));
+
+  struct pagenode *pn;
+  for(pn = firstpage; pn; pn = pn->next) 
+   if (!pn->dpiX) {
+	pn->dpiX = dpi.width();
+	pn->dpiY = dpi.height();
+   }
+}
+
 void TopLevel::openadd(QString filename)
 {
   auxpage = lastpage;
 
-  (void) notetiff(QFile::encodeName(filename));
+  loadfile(QFile::encodeName(filename));
 
   if( firstpage != lastpage )
   {
@@ -649,13 +632,13 @@ void TopLevel::resizeView()
   }
 
   if(Image){
-    hsb->setRange(0,max(0,Image->width - qwindow_width));
+    hsb->setRange(0,QMAX(0,Image->width - qwindow_width));
     hsb->setSteps(5,qwindow_width/2);
-    //    printf("hsb range: %d\n",max(0,Image->width - qwindow_width));
-    vsb->setRange(0,max(0,Image->height - qwindow_height));
+    //    printf("hsb range: %d\n",QMAX(0,Image->width - qwindow_width));
+    vsb->setRange(0,QMAX(0,Image->height - qwindow_height));
     vsb->setSteps(5,qwindow_height/2);
-    //    printf("vsb range: %d\n",max(0,Image->height - qwindow_height));
-    //    printf("vsb min %d vdb max %d\n",vsb->minValue(),vsb->maxValue());
+    //    printf("vsb range: %d\n",QMAX(0,Image->height - qwindow_height));
+    //    printf("vsb QMIN %d vdb QMAX %d\n",vsb->QMINValue(),vsb->QMAXValue());
   }
 
 
@@ -664,251 +647,151 @@ void TopLevel::resizeView()
 
 }
 
-void TopLevel::print(){
-  if(!thispage){
-    KMessageBox::sorry(this, i18n("There is no document active."));
-    return;
+bool TopLevel::loadAllPages( int &numpages, int &currentpage )
+{
+  struct pagenode *pn;
+
+  numpages = 0;
+  currentpage = 1;
+
+  for(pn = firstpage; pn; pn = pn->next) {
+    ++numpages;
+    if (pn == thispage)
+	currentpage = numpages;
+    if (!Pimage(pn)) {
+	int k = -1;
+	while((k != 0) && (k != 3) && (k != 1))
+		k = GetImage(pn); // fetch image if it is not available yet.
+    }
   }
-
-  if(!printdialog){
-    printdialog = new PrintDialog(NULL,"print",true);
-    connect(printdialog,SIGNAL(print()),this,SLOT(printIt()));
-
-  }
-
-  struct printinfo newpi;
-
-  newpi.file = pi.file;
-  newpi.cmd = pi.cmd;
-  newpi.lpr = pi.lpr;
-  newpi.scale = pi.scale;
-  newpi.pagesize = pi.pagesize;
-  newpi.margins = pi.margins;
-  newpi.xmargin = pi.xmargin;
-  newpi.ymargin = pi.ymargin;
-
-  printdialog->setWidgets(&newpi);
-
-  QPoint point = this->mapToGlobal (QPoint (0,0));
-
-  QRect pos = this->geometry();
-  printdialog->setGeometry(point.x() + pos.width()/2  - printdialog->width()/2,
-			   point.y() + pos.height()/2 - printdialog->height()/2,
-			   printdialog->width(),
-			   printdialog->height()
-			   );
-
-  printdialog->show();
-
+  return (numpages != 0);
 }
 
-void TopLevel::printIt(){
-
-  if(!firstpage)
-    return;
-
-  if(!printdialog)
-    return;
-
-  kapp->processEvents();
-
-  struct printinfo *newpi;
-
-  newpi = printdialog->getInfo();
-
-  pi.file = newpi->file;
-  pi.cmd = newpi->cmd;
-  pi.lpr = newpi->lpr;
-  pi.scale = newpi->scale;
-  pi.pagesize = newpi->pagesize;
-  pi.margins = newpi->margins;
-  pi.xmargin = newpi->xmargin;
-  pi.ymargin = newpi->ymargin;
-
-
-  float width = 8.5;
-  float height = 11;
-
-  if(pi.pagesize ==  US_LETTER){
-    width = 8.5;
-    height = 11;
+void TopLevel::print(){
+  if(!thispage || !firstpage) {
+    return KMessageBox::sorry(this, i18n("There is no document active."));
   }
 
-  if(pi.pagesize == US_LEGAL){
-    width = 8.5;
-    height = 14;
-  }
+  int pages, currentpage;
+  loadAllPages(pages, currentpage);
 
-  if(pi.pagesize == US_LEDGER){
-    width = 11;
-    height = 17;
-  }
+  KPrinter printer(true, QPrinter::PrinterResolution);
+  printer.setFullPage( true );
+  printer.setUsePrinterResolution( true );
+  printer.setDocName( firstpage->name );
+  printer.setPageSelection( KPrinter::ApplicationSide );
+  printer.setMinMax( 1, pages ); 
+  printer.setCurrentPage( currentpage );
+  if ( !printer.setup( this ) )
+	return;
 
-  if(pi.pagesize == US_EXECUTIVE){
-    width = 7.25;
-    height = 10.5;
-  }
-
-  if(pi.pagesize == DIN_A3){
-    width = 12.69;
-    height = 16.53;
-  }
-
-  if(pi.pagesize == DIN_A4){
-    width = 8.267;
-    height = 11.692;
-  }
-
-  if(pi.pagesize == DIN_A5){
-    width = 5.944;
-    height = 8.2675;
-  }
-
-  if(pi.pagesize == DIN_A6){
-    width = 4.2125;
-    height = 5.8258;
-  }
-
-  if(pi.pagesize == DIN_B4){
-    width = 10.04;
-    height = 14.33;
-  }
-
-  if(pi.pagesize == JAP_LETTER){
-    width = 7.165;
-    height = 10.1175;
-  }
-
-  if(pi.pagesize == JAP_LEGAL){
-    width = 10.1175;
-    height = 14.33;
-  }
+  QPainter painter;
+  painter.begin( &printer );
+  printIt(printer, painter);
+  painter.end();
+}
 
 
-  if(pi.margins == 1){
-    width  = width - pi.xmargin/2.54*2.0; /* 2.0 since we want this margin both at the
-					     top as well as on the bottom */
-    height = height - pi.ymargin/2.54*2.0;
-  }
-
-  if(width <= 0.0 || height <=0.0 ){
-
-    QString str = i18n("Invalid page dimensions:\nWidth %1 Height %2\n")
-		.arg(width, 2).arg(height, 2);
-    KMessageBox::sorry(this, str);
-
-    return;
-  }
-
-  FILE* psfile;
-
-  if(pi.lpr){
-
-    psfile = NULL;
-    psfile = popen(QFile::encodeName(pi.cmd),"w");
-
-    if(psfile == NULL){
-      QString str = i18n("Cannot print to \"%1\"\n").arg(pi.cmd);
-      KMessageBox::sorry(this, str);
-      return;
-    }
-
-  }
-  else{
-
-    psfile = fopen(QFile::encodeName(pi.file),"w");
-    if(psfile == NULL){
-      QString str = i18n("Could not create %1\n").arg(pi.file);
-      KMessageBox::error(this, str);
-      return;
-    }
-  }
+void TopLevel::printIt( KPrinter &printer, QPainter &painter )
+{
+  QPaintDeviceMetrics dm(painter.device());
 
   QApplication::setOverrideCursor( waitCursor );
-  XDefineCursor(qtdisplay, Win, WorkCursor);
   kapp->processEvents();
 
+  int currentpage = 0;
+  bool first_page_printed = false;
   struct pagenode *pn;
-  QStrList filelist;
+  for(pn = firstpage; pn; pn = pn->next) {
 
-  for(pn = firstpage; pn; pn = pn->next){
+    ++currentpage;
+    // should this page be printed ?
+    if (printer.pageList().findIndex(currentpage) < 0)
+	continue;
 
-    if(filelist.find(pn->pathname) != -1) // we have printed this fax file already
-      continue;
-
-    filelist.append(pn->pathname);
-
-    if(pn->type == FAX_TIFF){
-
-      fax2psmain(pn->pathname,psfile,width,height,pi.scale);
+    XImage *Image = Pimage(pn);
+    if (!Image)
+	continue;
 
+    // byte-swapping the image
+    QByteArray bytes( Image->height*Image->bytes_per_line );
+    for (int y=Image->height-1; y>=0; --y) {
+      Q_UINT32 offset  = y*Image->bytes_per_line;
+      Q_UINT32 *source = (Q_UINT32 *) (Image->data + offset);
+      Q_UINT32 *dest   = (Q_UINT32 *) (bytes.data() + offset);
+      for (int x=(Image->bytes_per_line/4)-1; x>=0; --x) {
+ 	Q_UINT32 dv = 0, sv = *source;
+	for (int bit=32; bit>0; --bit) {
+		dv <<= 1;
+		dv |= sv&1;
+		sv >>= 1;
+	}
+        *dest = dv;
+	++dest;
+	++source;
+      }
     }
 
-    if(pn->type == FAX_RAW){
+    QImage qi( (uchar *)bytes.data(), Image->bytes_per_line*8, Image->height, 1, NULL, 2, QImage::LittleEndian);
 
-      int faxtype = 31;
+    if (first_page_printed)
+	printer.newPage();
+    first_page_printed = true;
 
-      if(defaultpage.expander ==  g32expand)
-	faxtype = 32;
+    QSize printersize( dm.width(), dm.height() );
+    kdDebug() << "Printersize = " << printersize << endl;
+    // print Image in original size if possible, else scale it.
 
-      if(defaultpage.expander ==  g4expand)
-	faxtype = 4;
+    QSize size; // logical size of the image
+    size.setWidth ( Image->width  * dm.logicalDpiX() / pn->dpiX );
+    size.setHeight( Image->height * dm.logicalDpiY() / pn->dpiY );
 
-      if(defaultpage.expander ==  g31expand)
-	faxtype = 31;
+    kdDebug() << "Org image size = " << Image->width << "x" << Image->height << endl;
+    kdDebug() << "New image size = " << size << " logical printer res =" << dm.logicalDpiX() << "x" << dm.logicalDpiY() << endl;
 
-      KTempFile tmpFile;
-      tmpFile.setAutoDelete(true);
+    uint top, left, bottom, right;
+    printer.margins( &top, &left, &bottom, &right );
+    kdDebug() << "Margins = " << top << " " << left << " " << bottom << " " << right << endl;
 
-      fax2tiffmain(pn->pathname,QFile::encodeName(tmpFile.name()),pn->lsbfirst,pn->vres?0:1,faxtype);
-      fax2psmain(QFile::encodeName(tmpFile.name()),psfile,width,height,pi.scale);
+    QSize maxSize( printersize.width()-left-right, printersize.height()-top-bottom );
+    QImage scaledImage;
+    if (size.width() > maxSize.width() || size.height() > maxSize.height() ) {
+	// Image does not fit - scale it and print centered
+	scaledImage = qi.scale( maxSize, QImage::ScaleMin );
+	kdDebug() << "Image does not fit - scaling to " << maxSize << endl;
+    } else {
+	// Image does fit - print it in original size, but centered
+	scaledImage = qi.scale( size, QImage::ScaleMin );
+	kdDebug() << "Image does fit - scaling to " << size << endl;
     }
+    kdDebug() << "Final image size " << scaledImage.size() << endl;
+    maxSize -= scaledImage.size();
+    maxSize /= 2;
+    painter.drawImage( left+maxSize.width(), top+maxSize.height(), scaledImage );
 
   }
 
-  if(pi.lpr)
-    pclose(psfile);
-  else
-    fclose(psfile);
-
   QApplication::restoreOverrideCursor();
-  XDefineCursor(qtdisplay, Win, ReadyCursor);
-
 }
 
-void TopLevel::saveNetFile( const KURL& u)
+void TopLevel::saveNetFile( const KURL& dest)
 {
-    if ( !u.isValid() )
+    if ( !dest.isValid() )
     {
 	KMessageBox::sorry(this, i18n("Malformed URL"));
 	return;
     }
 
-    // Just a usual file ?
-    if ( u.isLocalFile() )
-    {
-      statusbar->message( i18n( "Saving..." ) );
-      int res = copyfile(QFile::encodeName(u.path()),thispage->pathname);
-      statusbar->clear();
-      if (res==0) {
+    statusbar->message( i18n( "Saving..." ) );
+   
+    KURL source = KURL::fromPathOrURL(thispage->pathname);
+    bool ok = KIO::NetAccess::file_copy( source, dest, -1, true, false, this);
+
+    statusbar->clear();
+
+    if (!ok)
 	  KMessageBox::error(this, i18n("Failure in 'copy file()'\n"
 				"Could not save file!"));
-	  return;
-	}
-	return;
-      }
-
-    KTempFile tmpFile;
-    tmpFile.setAutoDelete(true);
-
-    int res = copyfile(QFile::encodeName(tmpFile.name()),thispage->pathname );
-    if (res==0) {
-      KMessageBox::error(this, i18n("Failure in 'copy file()'\n"
-			    "Could not save file!"));
-      return;
-    }
-
-    KIO::NetAccess::upload( tmpFile.name(), u );
 }
 
 void TopLevel::openNetFile( const KURL &u)
@@ -930,7 +813,7 @@ void TopLevel::openNetFile( const KURL &u)
   {
     statusbar->message(i18n("Downloading..."));
     QString tmpFile = QString::null;
-    if ( KIO::NetAccess::download( u, tmpFile ) )
+    if ( KIO::NetAccess::download( u, tmpFile, this ) )
     {
       openadd( tmpFile );
       setCaption( u.prettyURL() );
@@ -1099,6 +982,7 @@ void TopLevel::handle_X_event(XEvent Event)
 	putImage();
 	break;
       case XK_l:
+      case XK_r:
 	rotatePage();
 	if (Event.xkey.state & ShiftMask)
 	  TurnFollowing(TURN_L, thispage->next);
@@ -1288,7 +1172,7 @@ void TopLevel::flipPage()
     }
   }
 
-  Image = Images[min(i-1,oz)];
+  Image = Images[QMIN(i-1,oz)];
 
   Refresh = Resize = 1;
   putImage();
@@ -1321,7 +1205,7 @@ void TopLevel::mirrorPage()
     if (Images[i] == NULL) // out of memory
       break;
   }
-  Image = Images[min(oz,i-1)];
+  Image = Images[QMIN(oz,i-1)];
 
   Refresh = Resize = 1;
   putImage();
@@ -1329,7 +1213,6 @@ void TopLevel::mirrorPage()
 
 void TopLevel::scrollHorz(int){
 
-
   if(!Image)
     return;
 
@@ -1338,7 +1221,6 @@ void TopLevel::scrollHorz(int){
 
   Refresh = 1;
   putImage();
-
 }
 
 void TopLevel::scrollVert(int ){
@@ -1507,14 +1389,13 @@ void TopLevel::faxClose()
   vsb->hide();
   hsb->hide();
 
-  fileURL = "";
+  fileURL = QString::null;
 
   updateActions();
 }
 
-void TopLevel::FreeFax(){
-
-
+void TopLevel::FreeFax()
+{
   if(display_is_setup)
     XClearWindow(qtdisplay, Win);
 
@@ -1554,7 +1435,6 @@ void TopLevel::FreeFax(){
   firstpage = lastpage = viewpage = thispage = auxpage = NULL;
 
   uiUpdate();
-
 }
 
 void TopLevel::uiUpdate(){
@@ -1657,16 +1537,16 @@ void TopLevel::putImage()
        has a sunken frame and I need to offset by 1 pixel to the right and
        one pixel down so that I don't paint my fax into the border of the frame*/
 
-    XResizeWindow(qtdisplay,Win,min(qwindow_width -1,Image->width ),
-		  min(qwindow_height -1,Image->height ));
+    XResizeWindow(qtdisplay,Win,QMIN(qwindow_width -1,Image->width ),
+		  QMIN(qwindow_height -1,Image->height ));
 
     XPutImage(qtdisplay, Win, PaintGC, Image,
-	      max(xpos - qwindow_width/2,0), max(ypos - qwindow_height/2,0),
-	      0, 0, min(qwindow_width -1,Image->width)  ,
-	      min(qwindow_height -1,Image->height) );
+	      QMAX(xpos - qwindow_width/2,0), QMAX(ypos - qwindow_height/2,0),
+	      0, 0, QMIN(qwindow_width -1,Image->width)  ,
+	      QMIN(qwindow_height -1,Image->height) );
 
-    vsb->setValue(max(ypos - qwindow_height/2,0));
-    hsb->setValue(max(xpos - qwindow_width/2,0));
+    vsb->setValue(QMAX(ypos - qwindow_height/2,0));
+    hsb->setValue(QMAX(xpos - qwindow_width/2,0));
 
     XFlush(qtdisplay);
   }
@@ -1715,41 +1595,39 @@ void setFaxDefaults(){
   if(have_cmd_opt ) // we have commad line options all kfaxrc defaults are
     return;         // overridden
 
-  if(fop.resauto == 1){
+  if(fop.resauto == 1)
     defaultpage.vres = -1;
-  }
-  else{
+  else
     defaultpage.vres = fop.fine;
-  }
 
   if(fop.geomauto == 1){
-    defaultpage.width = 0;
-    defaultpage.height = 0;
+    defaultpage.width = defaultpage.height = 0;
   }
   else{
-    defaultpage.width = fop.width;
+    defaultpage.width  = fop.width;
     defaultpage.height = fop.height;
   }
 
-  if( fop.landscape)
+  if(fop.landscape)
     defaultpage.orient |= TURN_L;
 
   if(fop.flip)
      defaultpage.orient |= TURN_U;
 
-  defaultpage.inverse =    fop.invert;
+  defaultpage.inverse  = fop.invert;
   defaultpage.lsbfirst = fop.lsbfirst;
 
-  if(fop.raw == 2)
-    defaultpage.expander = g32expand;
-  if(fop.raw == 4)
-    defaultpage.expander = g4expand;
-  if((fop.raw != 4) && (fop.raw != 2) )
-    defaultpage.expander = g31expand;
+  switch (fop.raw) {
+	case 2:	defaultpage.expander = g32expand;
+		break;
+	case 4: defaultpage.expander = g4expand;
+		break;
+	default:defaultpage.expander = g31expand;
+  }
 
 }
 
-static const char *description =
+static const char description[] =
         I18N_NOOP("KDE G3/G4 Fax Viewer");
 
 static KCmdLineOptions options[] =
@@ -1766,13 +1644,13 @@ static KCmdLineOptions options[] =
    {"u", 0, 0 },
    {"upsidedown",   I18N_NOOP( "Turn image upside down" ), 0 },
    {"i", 0, 0 },
-   {"invert",       I18N_NOOP( "Invert black and white." ), 0 },
+   {"invert",       I18N_NOOP( "Invert black and white" ), 0 },
    {"m", 0, 0 },
-   {"mem <bytes>",  I18N_NOOP( "Limit memory use to 'bytes'." ), "8M" },
+   {"mem <bytes>",  I18N_NOOP( "Limit memory use to 'bytes'" ), "8M" },
    {"r", 0, 0 },
    {"reverse",      I18N_NOOP( "Fax data is packed lsb first" ), 0 },
-   {"2" ,       I18N_NOOP( "Raw files are g3-2d" ), 0 },
-   {"4",        I18N_NOOP( "Raw files are g4" ), 0 },
+   {"2" ,           I18N_NOOP( "Raw files are g3-2d" ), 0 },
+   {"4",            I18N_NOOP( "Raw files are g4" ), 0 },
    {"+file(s)",     I18N_NOOP( "Fax file(s) to show" ), 0 },
    KCmdLineLastOption
 };
@@ -1796,7 +1674,7 @@ int main (int argc, char **argv)
   viewfaxmain();
 
   toplevel = new TopLevel();
-  toplevel->show ();
+  toplevel->show();
 
   startingup = 1;
   a.processEvents();
@@ -1820,42 +1698,5 @@ int main (int argc, char **argv)
   return a.exec ();
 }
 
-int copyfile(const char* toname,char* fromname)
-{
-  char buffer[4*1028];
-  int count = 0;
-  int count2;
 
-  FILE*  fromfile;
-  FILE*  tofile;
-
-  if (QFile::exists(toname)) {
-    if(KMessageBox::questionYesNo( 0,
-			      i18n("A file with this name already exists.\n"
-			      "Do you want to overwrite it?")))
-      return 1;
-  }
-
-  if((fromfile = fopen(fromname,"r")) == NULL)
-    return 0;
-
-  if((tofile =  fopen(toname,"w")) == NULL){
-    fclose(fromfile);
-    return 0;
-  }
-
-
-  while((count = fread(buffer,sizeof(char),4*1028,fromfile))){
-    count2 = fwrite(buffer,sizeof(char),count,tofile);
-    if (count2 != count){
-      fclose(fromfile);
-      fclose(tofile);
-      return 0;
-    }
-
-  }
-  fclose(fromfile);
-  fclose(tofile);
-
-  return 1;
-}
+#include "kfax.moc"
