@@ -1,6 +1,6 @@
     /*
 
-    $Id: kfax.h,v 1.40 2003/11/11 01:55:33 bmeyer Exp $
+    $Id: kfax.h,v 1.43.2.1 2004/11/09 19:55:50 deller Exp $
 
     Requires the Qt widget libraries, available at no cost at
     http://www.troll.no
@@ -28,54 +28,21 @@
 #ifndef _KFAX_H_
 #define _KFAX_H_
 
-#include <qptrlist.h>
-#include <qstring.h>
-
-#include <qpopupmenu.h>
-#include <qmenubar.h>
-#include <qapplication.h>
-#include <qkeycode.h>
-#include <qaccel.h>
-#include <qobject.h>
-#include <qmultilineedit.h>
-#include <qradiobutton.h>
-#include <qfiledialog.h>
-#include <qcheckbox.h>
-#include <qgroupbox.h>
-#include <qtooltip.h>
-#include <qregexp.h>
-#include <qprinter.h>
-
 #include <kapplication.h>
-#include <kconfig.h>
-#include <ktoolbar.h>
-#include <kstatusbar.h>
 #include <kmainwindow.h>
+#include <qptrlist.h>
 #include <kurl.h>
-#include <kaction.h>
-
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xatom.h>
-#include <X11/keysym.h>
-#include <X11/keysymdef.h>
-#include <X11/cursorfont.h>
-
-#include "faxexpand.h"
-
-#include "print.h"
-#include "options.h"
 
 #define Pimage(p)	((XImage *)(p)->extra)
 
-// StatusBar field IDs
-#define ID_LINE_COLUMN 1
-#define ID_INS_OVR 2
-#define ID_GENERAL 3
-#define ID_FNAME 4
-#define ID_TYPE 5
-#define ID_PAGE_NO 6
+class QPopupMenu;
+class KConfig;
+class KStatusBar;
+class KAction;
+class KRecentFilesAction;
+class KPrinter;
+class QPainter;
+class QScrollBar;
 
 typedef KToolBar::BarPosition BarPosition;
 
@@ -105,7 +72,7 @@ public:
 
 protected:
 
-    void resizeEvent(QResizeEvent *);
+    void resizeEvent( QResizeEvent * );
     void wheelEvent( QWheelEvent * );
     void dragEnterEvent( QDragEnterEvent * event );
     void dropEvent( QDropEvent * event );
@@ -143,18 +110,15 @@ private:
     KAction *actFlip, *actNext, *actPrev, *actFirst, *actLast;
 
     int open_mode;
-    int editor_width;
-    int editor_height;
 
     KConfig *config;
-    QString lastDir;
 
-    QScrollBar *	hsb;
-    QScrollBar *	vsb;
+    QScrollBar *hsb;
+    QScrollBar *vsb;
     QFrame* 	mainpane;
 
-    PrintDialog * printdialog;
-    struct printinfo pi;
+    void printIt( KPrinter &printer, QPainter &painter );
+    bool loadAllPages( int &numpages, int &currentpage );
 
 public slots:
 
@@ -183,13 +147,13 @@ public slots:
     void uiUpdate();
 
     void openadd(QString filename);
-    void printIt();
     void FreeFax();
     void scrollHorz(int);
     void scrollVert(int);
-    void slotConfigureKeys();
 };
 
 void kfaxerror(const QString&, const QString&);
+void loadfile(QString filename);
+
+#endif // _KFAX_H_
 
-#endif
