@@ -36,6 +36,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 
+#include <qglobal.h>
 #include <qdir.h>
 #include <qfile.h>
 #include <qevent.h>
@@ -45,6 +46,11 @@
 #include <kcmdlineargs.h>
 #include <klocale.h>
 
+#include "kfax.h"
+#include "faxexpand.h"
+#include "version.h"
+#include "viewfax.h"
+
 /* NewImage() needs to fiddle with the Display structure */
 #define XLIB_ILLEGAL_ACCESS
 #include <X11/Xlib.h>
@@ -54,10 +60,6 @@
 #include <X11/keysymdef.h>
 #include <X11/cursorfont.h>
 
-#include <qglobal.h>
-#include "faxexpand.h"
-#include "version.h"
-#include "viewfax.h"
 
 #define VIEWFAXVERSION "2.3"
 
@@ -85,7 +87,6 @@ XImage *ZoomImage(XImage *Big);
 void FreeImage(XImage *Image);
 
 static int release(int quit);
-/*static char *suffix(char *opt, const char *str);*/
 
 
 /* X variables */
@@ -103,8 +104,6 @@ extern Window Win;
 extern int qwindow_width;
 extern int qwindow_height;
 
-extern void kfaxerror(const QString& title, const QString& error);
-
 struct pagenode *firstpage, *lastpage, *thispage, *helppage, *auxpage;
 struct pagenode defaultpage;
 
@@ -241,7 +240,7 @@ int viewfaxmain()
     firstpage = lastpage = thispage = helppage = auxpage =  0;
 
     for (int i = 0; i < args->count(); i++){
-	(void) notetiff(args->arg(i));
+	loadfile(args->arg(i));
     }
     args->clear();
 
@@ -250,12 +249,7 @@ int viewfaxmain()
       exit(1);
     }
 
-    if ( firstpage == 0) {
-      have_no_fax = TRUE;
-    }
-    else{
-      have_no_fax = FALSE;
-    }
+    have_no_fax = (firstpage == 0);
 
     Disp = qtdisplay;
     Default_Screen = XDefaultScreen(qtdisplay);
@@ -264,17 +258,6 @@ int viewfaxmain()
 }
 
 
-
-/* return mismatching suffix of option name */
-/*static char *suffix(char *opt, const char *prefix){
-
-    while (*opt && *opt == *prefix) {
-	opt++; prefix++;
-    }
-    return opt;
-}
-*/
-
 /* Change orientation of all following pages */
 void TurnFollowing(int How, struct pagenode *pn)
 {
