@@ -20,7 +20,8 @@
 #ifndef _VIEWFAX_H_
 #define _VIEWFAX_H_
 
+#define MAXZOOM 4
+
 int viewfaxmain();
-void viewfax_addCmdLineOptions();
 
 #endif
