@@ -64,13 +64,13 @@ KJpegPlugin::KJpegPlugin(QObject *parent, const char *name,
                  KFileMimeTypeInfo::Addable |
                  KFileMimeTypeInfo::MultiLine );
 
-  item = addItemInfo( exifGroup, "Manufacturer", i18n("Camera manufacturer"),
+  item = addItemInfo( exifGroup, "Manufacturer", i18n("Camera Manufacturer"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "Model", i18n("Camera model"),
+  item = addItemInfo( exifGroup, "Model", i18n("Camera Model"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "Date/time", i18n("Date/time"),
+  item = addItemInfo( exifGroup, "Date/time", i18n("Date/Time"),
                       QVariant::DateTime );
 
   item = addItemInfo( exifGroup, "CreationDate", i18n("Creation Date"),
@@ -83,61 +83,60 @@ KJpegPlugin::KJpegPlugin(QObject *parent, const char *name,
                       QVariant::Size );
   setHint( item, KFileMimeTypeInfo::Size );
   setUnit( item, KFileMimeTypeInfo::Pixels );
-  setSuffix( item, i18n(" Pixels") ); // no unit supported for QSize atm.
 
   item = addItemInfo( exifGroup, "Orientation", i18n("Orientation"),
                       QVariant::Int );
 
-  item = addItemInfo( exifGroup, "ColorMode", i18n("Color mode"),
+  item = addItemInfo( exifGroup, "ColorMode", i18n("Color Mode"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "Flash used", i18n("Flash used"),
+  item = addItemInfo( exifGroup, "Flash used", i18n("Flash Used"),
                       QVariant::Bool );
-  item = addItemInfo( exifGroup, "Focal length", i18n("Focal length"),
+  item = addItemInfo( exifGroup, "Focal length", i18n("Focal Length"),
                       QVariant::String );
-  setSuffix( item, i18n("Millimeters", " mm") );
+  setUnit( item, KFileMimeTypeInfo::Millimeters );
 
-  item = addItemInfo( exifGroup, "35mm equivalent", i18n("35mm equivalent"),
+  item = addItemInfo( exifGroup, "35mm equivalent", i18n("35mm Equivalent"),
                       QVariant::Int );
-  setSuffix( item, i18n("Millimeters", " mm") );
+  setUnit( item, KFileMimeTypeInfo::Millimeters );
 
-  item = addItemInfo( exifGroup, "CCD width", i18n("CCD width"),
+  item = addItemInfo( exifGroup, "CCD width", i18n("CCD Width"),
                       QVariant::String );
-  setSuffix( item, i18n("Millimeters", " mm") );
+  setUnit( item, KFileMimeTypeInfo::Millimeters );
 
-  item = addItemInfo( exifGroup, "Exposure time", i18n("Exposure time"),
+  item = addItemInfo( exifGroup, "Exposure time", i18n("Exposure Time"),
                       QVariant::String );
   setHint( item, KFileMimeTypeInfo::Seconds );
 
   item = addItemInfo( exifGroup, "Aperture", i18n("Aperture"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "Focus dist.", i18n("Focus dist."),
+  item = addItemInfo( exifGroup, "Focus dist.", i18n("Focus Dist."),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "Exposure bias", i18n("Exposure bias"),
+  item = addItemInfo( exifGroup, "Exposure bias", i18n("Exposure Bias"),
                       QVariant::String );
 
   item = addItemInfo( exifGroup, "Whitebalance", i18n("Whitebalance"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "Metering mode", i18n("Metering mode"),
+  item = addItemInfo( exifGroup, "Metering mode", i18n("Metering Mode"),
                       QVariant::String );
 
   item = addItemInfo( exifGroup, "Exposure", i18n("Exposure"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "ISO equiv.", i18n("ISO equiv."),
+  item = addItemInfo( exifGroup, "ISO equiv.", i18n("ISO Equiv."),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "JPEG quality", i18n("JPEG quality"),
+  item = addItemInfo( exifGroup, "JPEG quality", i18n("JPEG Quality"),
                       QVariant::String );
 
-  item = addItemInfo( exifGroup, "User comment", i18n("User comment"),
+  item = addItemInfo( exifGroup, "User comment", i18n("User Comment"),
                       QVariant::String );
   setHint(item,  KFileMimeTypeInfo::Description);
 
-  item = addItemInfo( exifGroup, "JPEG process", i18n("JPEG process"),
+  item = addItemInfo( exifGroup, "JPEG process", i18n("JPEG Process"),
                       QVariant::String );
 
   item = addItemInfo( exifGroup, "Thumbnail", i18n("Thumbnail"),
@@ -243,7 +242,7 @@ bool KJpegPlugin::readInfo( KFileMetaInfo& info, uint what )
 
     appendItem( exifGroup, "ColorMode", ImageInfo.getIsColor() ?
                 i18n("Color") : i18n("Black and white") );
-    
+
     int flashUsed = ImageInfo.getFlashUsed(); // -1, 0, 1
     if ( flashUsed >= 0 ) {
         appendItem( exifGroup, "Flash used",
@@ -464,7 +463,7 @@ QDateTime KJpegPlugin::parseDateTime( const QString& string )
     QString day     = string.mid( 8, 2 );
     QString hour    = string.mid( 11, 2 );
     QString minute  = string.mid( 14, 2 );
-    QString seconds = string.mid( 18, 2 );
+    QString seconds = string.mid( 17, 2 );
 
     bool ok;
     bool allOk = true;
