@@ -69,7 +69,7 @@ public:
     KGVPageDecorator* pageDecorator() const { return _pageDecorator; }
     KGVDocument*      document()      const { return _document;      }
     const KGVConfigDialog* configDialog()   { return _configDialog;  }
-    
+
 
     /**
      * Reimplemented from ReadOnlyPart in order to delete the file from
@@ -107,7 +107,10 @@ public slots:
     void slotNextPage();
     void slotGotoStart();
     void slotGotoEnd();
-    // void slotFitWidth();
+
+    void slotFitToPage();
+    void slotFitToScreen();
+    void slotDoFitToScreen();
 
     void showScrollBars( bool );
     void slotCancelWatch();
@@ -146,7 +149,6 @@ protected slots:
     void slotWatchFile();
 
     void slotOpenFileCompleted();
-    void slotConfigureKeys();
 
 protected:
     virtual void guiActivateEvent( KParts::GUIActivateEvent* );
@@ -182,12 +184,15 @@ private:
     KGVConfigDialog* _configDialog;
     LogWindow*       _logWindow;
 
+    QTimer*		_fitTimer;
+
     KSelectAction* _selectOrientation;
     KSelectAction* _selectMedia;
     KAction*       _zoomIn;
     KAction*       _zoomOut;
     KSelectAction* _zoomTo;
-    // KAction* _fitWidth;
+    KAction *      _fitWidth;
+    KAction *      _fitScreen;
     KAction*       _prevPage;
     KAction*       _nextPage;
     KAction*       _firstPage;
@@ -213,6 +218,9 @@ private:
     bool _isFileDirty      : 1;
     bool _showLogWindow    : 1;
     bool _stickyOptions    : 1;
+    bool _embeddedInKGhostView : 1;
+
+    int _customZoomIndex;
 
     DisplayOptions _options;
 };
