@@ -174,16 +174,18 @@ bool KGVDocument::uncompressFile()
 
     auto_ptr<QIODevice> filterDev( KFilterDev::deviceForFile( _fileName, _mimetype, true ) );
     if ( !filterDev.get() ) {
-	if ( _fileName.right( 3 ) == ".gz" ) {
+        KMimeType::Ptr mt = KMimeType::mimeType(_mimetype);
+	if ( (_fileName.right( 3 ) == ".gz") || mt->is("application/x-gzip") ) {
 	    kdDebug(4500) << "KGVDocument::uncompressFile(): manually guessing gzip" << endl;
 	    filterDev.reset( KFilterDev::deviceForFile( _fileName, "application/x-gzip", true ) );
-	} else if ( _fileName.right( 4 ) == ".bz2" ) {
+	} else if ( (_fileName.right( 4 ) == ".bz2") || mt->is("application/x-bzip2") ) {
 	    kdDebug(4500) << "KGVDocument::uncompressFile(): manually guessing bzip2" << endl;
 	    filterDev.reset( KFilterDev::deviceForFile( _fileName, "application/x-bzip2", true ) );
 	} else {
 	    kdDebug( 4500 ) << "KGVDocument::uncompressFile(): Unable to guess " << _fileName << endl;
 	}
-	if ( !filterDev.get() ) return false;
+	if ( !filterDev.get() )
+	    return false;
     }
     if( !filterDev->open( IO_ReadOnly ) ) 
     {
@@ -306,7 +308,7 @@ void KGVDocument::scanDSC()
     QTime clock;
     clock.start();
     */
-    while( ( count = fread( buf, sizeof(char), 4096, _psFile ) ) != 0 )
+    while( ( count = fread( buf, sizeof(char), sizeof( buf ), _psFile ) ) != 0 )
     {
 	_dsc->scanData( buf, count );
 	/*
@@ -364,7 +366,7 @@ bool KGVDocument::convertFromPDF( const QString& saveFileName,
     /*QValueList<QCString> args = process.args();
     QValueList<QCString>::Iterator it = args.begin();
     for ( ; it != args.end() ; ++it )
-        kdDebug() << ( *it ) << endl;*/
+        kdDebug(4500) << ( *it ) << endl;*/
 
     if( !process.start( KProcess::Block ) )
     {
@@ -525,8 +527,16 @@ void KGVDocument::print()
 	    KTempFile tf( QString::null, ".ps" );
 	    if( tf.status() == 0 ) 
 	    {
-	        savePages( tf.name(), printer.pageList() );
-	        printer.printFiles( QStringList( tf.name() ), true );
+		if ( printer.pageList().empty() ) { 
+		    KMessageBox::sorry( 0,
+			    i18n( "Printing failed because the list of "
+				"pages to be printed was empty." ),
+			    i18n( "Error Printing" ) );
+		} else if ( savePages( tf.name(), printer.pageList() ) ) {
+		    printer.printFiles( QStringList( tf.name() ), true );
+		} else {
+		    KMessageBox::error( 0, i18n( "<qt><strong>Printing failure:</strong><br>Could not convert to PostScript</qt>" ) );
+		}
 	    }
 	    else 
 	    {
@@ -668,7 +678,7 @@ bool KGVDocument::psCopyDoc( const QString& inputFile,
 	char buf[256];
 	int count;
 	dsc = dsc_init( 0 );
-	while( ( count = fread( buf, 1, 1, fp ) ) != 0 )
+	while( ( count = fread( buf, 1, sizeof( buf ), fp ) ) != 0 )
 	    dsc_scan_data( dsc, buf, count );
 	dsc_fixup( dsc );
 	fclose( fp );
