@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include <qdatetime.h>
@@ -26,7 +26,7 @@ KGVPageView::KGVPageView( QWidget* parent, const char* name )
   : QScrollView( parent, name )
 {
     _page = 0;
-    
+
     setFocusPolicy( QWidget::StrongFocus );
     viewport()->setFocusPolicy( QWidget::WheelFocus );
 }
@@ -52,60 +52,74 @@ bool KGVPageView::atBottom() const
 
 bool KGVPageView::eventFilter( QObject* o, QEvent* e )
 {
-    if( o == _page ) {
-	QMouseEvent* me = static_cast< QMouseEvent* >( e );
-	
-	if( e->type() == QEvent::MouseButtonPress ) {
-	    if( me->button() & LeftButton ) {
-		_dragGrabPos = me->globalPos();
-		setCursor( sizeAllCursor );
-	    }
-	    else if (me->button() & MidButton ) {
-		emit ReadDown();
-	    }
-	}
-	else if( e->type() == QEvent::MouseButtonRelease ) {
-	    if( me->button() & LeftButton ) {
-		setCursor( arrowCursor );
-	    }
-	}
-	else if( e->type() == QEvent::MouseMove ) {
-	    if( me->state() & LeftButton ) {
-		QPoint delta = _dragGrabPos - me->globalPos();
-		scrollBy( delta.x(), delta.y() );
-		_dragGrabPos = me->globalPos();
-	    }
-	}
-	else if( e->type() == QEvent::Resize ) {
-	    // We need to call QScrollView::eventFilter before centerContents,
-	    // otherwise a loop will be introduced.
-	    bool result = QScrollView::eventFilter( o, e );
-	    centerContents();
-	    emit pageSizeChanged( _page->size() );
-	    return result;
-	}
+    if ( o == _page &&  e->type() == QEvent::Resize ) {
+	// We need to call QScrollView::eventFilter before centerContents,
+	// otherwise a loop will be introduced.
+	bool result = QScrollView::eventFilter( o, e );
+	centerContents();
+	emit pageSizeChanged( _page->size() );
+	return result;
     }
-
     return QScrollView::eventFilter( o, e );
 }
 
+
 void KGVPageView::wheelEvent( QWheelEvent *e )
 {
     int delta = e->delta();
     e->accept();
-    if ((e->state() & ShiftButton) == ShiftButton) {
-        if ( e->delta() < 0 )
-                       emit zoomOut();
-               else
-                       emit zoomIn();
+    if ((e->state() & ControlButton) == ControlButton) {
+	if ( e->delta() < 0 )
+	    emit zoomOut();
+	else
+	    emit zoomIn();
     }
     else if ( delta <= -120 && atBottom() )
-        emit ReadDown();
-    else if ( delta >= 120 && atTop() )
-        emit ReadUp();
+    {
+	emit ReadDown();
+    }
+    else if ( delta >= 120 && atTop())
+    {
+	emit ReadUp();
+    }
+
     else
         QScrollView::wheelEvent( e );
 }
+void KGVPageView::mousePressEvent( QMouseEvent * e )
+{
+    if ( e->button() & LeftButton )
+    {
+	_dragGrabPos = e -> globalPos();
+	setCursor( sizeAllCursor );
+    }
+    else if ( e->button() & MidButton )
+    {
+	emit ReadDown();
+    }
+    else if ( e -> button() & RightButton )
+    {
+	emit rightClick();
+    }
+}
+
+void KGVPageView::mouseReleaseEvent( QMouseEvent *e )
+{
+    if ( e -> button() & LeftButton )
+    {
+	setCursor( arrowCursor );
+    }
+}
+
+void KGVPageView::mouseMoveEvent( QMouseEvent * e )
+{
+    if ( e->state() & LeftButton )
+    {
+	QPoint delta = _dragGrabPos - e->globalPos();
+	scrollBy( delta.x(), delta.y() );
+	_dragGrabPos = e->globalPos();
+    }
+}
 
 bool KGVPageView::readUp()
 {
@@ -188,7 +202,7 @@ void KGVPageView::enableScrollBars( bool b )
 }
 
 void KGVPageView::keyPressEvent( QKeyEvent* e )
-{   
+{
     switch ( e->key() ) {
     case Key_Up:
 	scrollUp();
@@ -220,7 +234,7 @@ void KGVPageView::centerContents()
 {
     if( !_page ) 
 	return;
-    
+
     int newX = 0;
     int newY = 0;
 
@@ -236,3 +250,5 @@ void KGVPageView::centerContents()
 }
 
 #include "kgvpageview.moc"
+
+// vim:sw=4:sts=4:ts=8:noet
