@@ -32,6 +32,7 @@
 #include <kstdaccel.h>
 #include <ktempfile.h>
 #include <kmenubar.h>
+#include <kedittoolbar.h>
 #include <kdebug.h>
 
 #include <kicontheme.h>
@@ -78,16 +79,10 @@ KGVShell::KGVShell() :
 		    KStdAccel::shortcut( KStdAccel::Reload ),
 		    m_gvpart, SLOT(  reloadFile() ),
 		    actionCollection(), "reload" );
-	    new KAction( i18n( "&Fit to Page Width" ), 0, this,
-			 SLOT( slotFitToPage() ), actionCollection(),
-			 "fit_to_page");
-	    new KAction( i18n( "&Fit to Screen" ), Key_S, this,
-			 SLOT( slotFitToScreen() ), actionCollection(),
-			 "fit_to_screen");
 	    new KAction( i18n( "&Maximize" ), Key_M, this,
 			 SLOT( slotMaximize() ), actionCollection(),
 			 "maximize");
-    _showMenuBarAction = KStdAction::showMenubar( this, SLOT( slotShowMenubar() ), actionCollection(), "showmenubar" );
+    _showMenuBarAction = KStdAction::showMenubar( this, SLOT( slotShowMenubar() ), actionCollection() );
 
     /*---- Settings -------------------------------------------------------*/
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,1,90)
@@ -97,14 +92,18 @@ KGVShell::KGVShell() :
     setStandardToolBarMenuEnabled(true);
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,1,90)
     m_fullScreenAction = KStdAction::fullScreen( this, SLOT( slotUpdateFullScreen() ), actionCollection(), this );
-#else 
+#else
     m_fullScreenAction = new KToggleAction( this, SLOT( slotUpdateFullScreen() ) );
 #endif
+    KStdAction::configureToolbars( this, SLOT( slotConfigureToolbars() ), actionCollection() );
+    KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()),
+actionCollection());
 
     //_popup = new KPopupMenu( i18n( "Full Screen Options" ), this, "rmb popup" );
     _popup = new KPopupMenu( this, "rmb popup" );
     _popup->insertTitle( i18n( "Full Screen Options" ) );
     m_fullScreenAction->plug( _popup );
+    _showMenuBarAction->plug( _popup );
 
     m_fsFilter = new FullScreenFilter( *this );
 
@@ -124,7 +123,6 @@ KGVShell::KGVShell() :
     connect( m_gvpart->pageView(), SIGNAL( rightClick() ),SLOT( slotRMBClick() ) );
     connect( m_gvpart, SIGNAL( canceled(const QString&) ),SLOT( slotReset() ) );
     connect( m_gvpart, SIGNAL( completed() ), SLOT( slotDocumentState() ) );
-    mFitTimer = new QTimer( this );
 
     if (!initialGeometrySet())
         resize(640,400);
@@ -177,7 +175,7 @@ void KGVShell::slotReset()
 void
 KGVShell::readProperties( KConfig *config )
 {
-    KURL url = config->readPathEntry( "URL" );
+    KURL url = KURL::fromPathOrURL( config->readPathEntry( "URL" ) );
     if ( url.isValid() ) {
         openURL( url );
 	DisplayOptions options;
@@ -201,6 +199,7 @@ KGVShell::readSettings()
     KGlobal::config()->setDesktopGroup();
     bool fullScreen = KGlobal::config()->readBoolEntry( "FullScreen", false );
     setFullScreen( fullScreen );
+    _showMenuBarAction->setChecked( menuBar()->isVisible() );
 }
 
 void
@@ -210,7 +209,6 @@ KGVShell::writeSettings()
 
     recent->saveEntries( KGlobal::config() );
 
-
     KGlobal::config()->setDesktopGroup();
     KGlobal::config()->writeEntry( "FullScreen", m_fullScreenAction->isChecked());
 
@@ -261,7 +259,7 @@ KGVShell::openStdin()
 
     _tmpFile->close();
 
-    if( m_gvpart->openURL( _tmpFile->name() ) ) setCaption( "stdin" );
+    if( m_gvpart->openURL( KURL::fromPathOrURL( _tmpFile->name() ) ) ) setCaption( "stdin" );
 }
 
 void KGVShell::slotFileOpen()
@@ -283,32 +281,6 @@ void KGVShell::slotDocumentState()
     stateChanged( "documentState" );
 }
 
-void KGVShell::slotFitToPage()
-{
-    kdDebug(4500) << "KGVShell::slotFitToPage()" << endl;
-    if( m_gvpart->pageView()->page() )
-	m_gvpart->miniWidget()->fitWidth( m_gvpart->pageView()->viewport()->width() - 16 );
-    // We subtract 16 pixels because of the page decoration.
-}
-
-
-void KGVShell::slotFitToScreen()
-{
-    kdDebug(4500) << "KGVShell::slotFitToScreen()" << endl;
-    if ( mFitTimer->isActive() ) {
-	disconnect( mFitTimer, SIGNAL( timeout() ), this, 0 );
-	connect( mFitTimer, SIGNAL( timeout() ), SLOT( slotDoFitToScreen() ) );
-    }
-    else slotDoFitToScreen();
-}
-
-void KGVShell::slotDoFitToScreen()
-{
-    kdDebug(4500) << "KGVShell::slotDoFitToScreen()" << endl;
-    if( m_gvpart->pageView()->page() )
-	m_gvpart->miniWidget()->fitWidthHeight( m_gvpart->pageView()->viewport()->width() - 16,
-						m_gvpart->pageView()->viewport()->height() - 16 );
-}
 
 void KGVShell::slotMaximize()
 {
@@ -316,9 +288,7 @@ void KGVShell::slotMaximize()
     KWin::setState( winId(), NET::MaxHoriz | NET::MaxVert );
     // If we do it now, it comes to nothing since it would work
     // on the current (non-maximized) size
-    disconnect( mFitTimer, SIGNAL( timeout() ), this, 0 );
-    connect( mFitTimer, SIGNAL( timeout() ), SLOT( slotFitToPage() ) );
-    mFitTimer->start( 800, true );
+    QTimer::singleShot( 800, m_gvpart, SLOT( slotFitToPage() ) );
 }
 
 void KGVShell::slotResize()
@@ -344,8 +314,8 @@ void KGVShell::slotUpdateFullScreen()
 	m_gvpart->updateFullScreen( true );
 	showFullScreen();
 	kapp->installEventFilter( m_fsFilter );
-	if ( m_gvpart->document()->isOpen() ) 
-		slotFitToPage();
+	if ( m_gvpart->document()->isOpen() )
+		m_gvpart->slotFitToPage();
     }
     else
     {
@@ -356,12 +326,25 @@ void KGVShell::slotUpdateFullScreen()
 	KToggleAction *statusbarAction = dynamic_cast<KToggleAction *>(actionCollection()->action(KStdAction::name(KStdAction::ShowStatusbar)));
 	assert( statusbarAction );
 	if (statusbarAction->isChecked()) statusBar()->show();
-#endif 
+#endif
 	toolBar()->show();
 	showNormal();
     }
 }
 
+void KGVShell::slotConfigureToolbars()
+{
+    saveMainWindowSettings( KGlobal::config(), "MainWindow" );
+    KEditToolbar dlg( factory() );
+    connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(slotNewToolbarConfig()));
+    dlg.exec();
+}
+
+void KGVShell::slotNewToolbarConfig()
+{
+    applyMainWindowSettings( KGlobal::config(), "MainWindow" );
+}
+
 void KGVShell::slotRMBClick()
 {
     _popup->exec( QCursor::pos() );
