@@ -114,6 +114,8 @@ void KIconEdit::init()
            SLOT( slotUpdateStatusMessage(const QString &)));
   connect(grid, SIGNAL(clipboarddata(bool)),
            SLOT( slotUpdatePaste(bool)));
+  connect(grid, SIGNAL(colorSelected(uint)),
+           m_paletteToolBar, SLOT(currentColorChanged(uint)));
   connect(grid, SIGNAL(modifiedchanged(bool)),
            SLOT( slotUpdateStatusModified(bool)));
   gridview->checkClipboard(); //Not very elegant, but fixes a buglet
@@ -205,7 +207,7 @@ void KIconEdit::readProperties(KConfig *config)
 
     QString entry = config->readPathEntry("Name"); // no default
     if (entry.isEmpty())  return;
-    icon->open(&grid->image(), entry);
+    icon->open(&grid->image(), KURL::fromPathOrURL( entry ));
 }
 
 /*
@@ -269,11 +271,11 @@ void KIconEdit::setupActions()
 
   m_actCut = KStdAction::cut(this, SLOT(slotCut()), actionCollection());
   m_actCut->setWhatsThis(i18n("Cut\n\nCut the current selection out of the"
-      " icon\n\n(Tip: You can make both rectangular and circular selections)"));
+      " icon.\n\n(Tip: You can make both rectangular and circular selections)"));
 
   m_actCopy = KStdAction::copy(this, SLOT(slotCopy()), actionCollection());
   m_actCopy->setWhatsThis(i18n("Copy\n\nCopy the current selection out of the"
-      " icon\n\n(Tip: You can make both rectangular and circular selections)"));
+      " icon.\n\n(Tip: You can make both rectangular and circular selections)"));
 
   m_actPaste = KStdAction::paste(this, SLOT(slotPaste()), actionCollection());
   m_actPaste->setWhatsThis(i18n("Paste\n\n"
@@ -342,6 +344,7 @@ void KIconEdit::setupActions()
   toggle = new KToggleAction( i18n( "Show &Grid" ), "grid",
       cut, this, SLOT( slotShowGrid() ), actionCollection(),
       "options_show_grid" );
+  toggle->setCheckedState(i18n("Hide &Grid"));
   toggle->setWhatsThis( i18n( "Show grid\n\nToggles the grid in the icon"
       " edit grid on/off" ) );
   toggle->setChecked( KIconEditProperties::self()->showGrid() );
@@ -489,7 +492,7 @@ bool KIconEdit::setupStatusBar()
 
 void KIconEdit::addRecent(const QString & path)
 {
-  m_actRecent->addURL(path);
+  m_actRecent->addURL(KURL( path ));
 }
 
 #include "kiconedit.moc"
