@@ -24,6 +24,8 @@
 
 #include <qwidget.h>
 
+#include "kicongrid.h"
+
 class KIconEditProperties : public QObject
 {
 public:
@@ -37,6 +39,12 @@ public:
   bool pasteTransparent() { return m_pasteTransparent; }
   bool showRulers() { return m_showRulers; }
   int gridScale() { return m_gridScale; }
+  
+  KIconEditGrid::TransparencyDisplayType transparencyDisplayType() const { return m_transparencyDisplayType; }
+  QColor checkerboardColor1() const { return m_checkerboardColor1; }
+  QColor checkerboardColor2() const { return m_checkerboardColor2; }
+  KIconEditGrid::CheckerboardSize checkerboardSize() const { return m_checkerboardSize; }
+  QColor transparencySolidColor() const { return m_transparencySolidColor; }
 
   void setBgPixmap( const QString &p ) { m_bgPixmap = p; }
   void setBgColor( const QColor &c ) { m_bgColor = c; }
@@ -45,6 +53,11 @@ public:
   void setPasteTransparent( bool b ) { m_pasteTransparent = b; }
   void setShowRulers( bool b ) { m_showRulers = b; }
   void setGridScale( int s ) { m_gridScale = s; }
+  void setTransparencyDisplayType(KIconEditGrid::TransparencyDisplayType t) { m_transparencyDisplayType = t; }
+  void setCheckerboardColor1(const QColor& c) { m_checkerboardColor1 = c; }
+  void setCheckerboardColor2(const QColor& c) { m_checkerboardColor2 = c; }
+  void setCheckerboardSize(KIconEditGrid::CheckerboardSize size) { m_checkerboardSize = size; }
+  void setTransparencySolidColor(const QColor& c) { m_transparencySolidColor = c; }
 
   void save();
 
@@ -57,6 +70,11 @@ protected:
   bool m_pasteTransparent;
   bool m_showRulers;
   int m_gridScale;
+  KIconEditGrid::TransparencyDisplayType m_transparencyDisplayType;
+  QColor m_checkerboardColor1;
+  QColor m_checkerboardColor2;
+  KIconEditGrid::CheckerboardSize m_checkerboardSize;
+  QColor m_transparencySolidColor;
 
   static KIconEditProperties* m_self;
 };
