@@ -1,6 +1,3 @@
-#ifndef STREAM_CCITT_H
-#define STREAM_CCITT_H
-
 //========================================================================
 //
 // Stream-CCITT.h
@@ -11,6 +8,9 @@
 //
 //========================================================================
 
+#ifndef __STREAM_CCITT_H
+#define __STREAM_CCITT_H
+
 struct CCITTCode {
   short bits;
   short n;
@@ -461,4 +461,4 @@ static CCITTCode blackTab3[64] = {
     {2, 2}, {2, 2}, {2, 2}, {2, 2}
 };
 
-#endif
+#endif // __STREAM_CCITT_H
