@@ -124,6 +124,23 @@ public:
     */
    PMVector controlPoint( int i ) const;
 
+   /**
+    * Sets the uv vectors flag
+    */
+   void enableUV( bool yes );
+   /**
+    * Returns the uv vectors flag
+    */
+   bool isUVEnabled( ) const { return m_uvEnabled; }
+   /**
+    * Sets the ith uv vector
+    */
+   void setUVVector( int i, const PMVector& v );
+   /**
+    * Returns the ith uv vector
+    */
+   PMVector uvVector( int i ) const;
+
 protected:
    /** */
    virtual bool isDefault( ) { return false; }
@@ -141,7 +158,9 @@ private:
                                   PMCP0ID, PMCP1ID, PMCP2ID, PMCP3ID,
                                   PMCP4ID, PMCP5ID, PMCP6ID, PMCP7ID,
                                   PMCP8ID, PMCP9ID, PMCP10ID, PMCP11ID,
-                                  PMCP12ID, PMCP13ID, PMCP14ID, PMCP15ID };
+                                  PMCP12ID, PMCP13ID, PMCP14ID, PMCP15ID,
+                                  PMUVEnabledID,
+                                  PMUV0ID, PMUV1ID, PMUV2ID, PMUV3ID };
 
    /**
     * The patch type
@@ -160,6 +179,8 @@ private:
     */
    PMVector m_point[16];
    int m_vsUSteps, m_vsVSteps;
+   bool m_uvEnabled;
+   PMVector m_uvVectors[4];
 
    static PMMetaObject* s_pMetaObject;
 };
