@@ -26,7 +26,7 @@
 
 #include <klocale.h>
 
-PMDefinePropertyClass( PMBlobSphere, PMProperty );
+PMDefinePropertyClass( PMBlobSphere, PMBlobSphereProperty );
 
 PMMetaObject* PMBlobSphere::s_pMetaObject = 0;
 PMObject* createNewBlobSphere( PMPart* part )
@@ -79,13 +79,13 @@ PMMetaObject* PMBlobSphere::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "BlobSphere", Base::metaObject( ),
                                         createNewBlobSphere );
       s_pMetaObject->addProperty(
-         new PMProperty( "center", &PMBlobSphere::setCentre,
+         new PMBlobSphereProperty( "center", &PMBlobSphere::setCentre,
                          &PMBlobSphere::centre ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "radius", &PMBlobSphere::setRadius,
+         new PMBlobSphereProperty( "radius", &PMBlobSphere::setRadius,
                          &PMBlobSphere::radius ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "strength", &PMBlobSphere::setStrength,
+         new PMBlobSphereProperty( "strength", &PMBlobSphere::setStrength,
                          &PMBlobSphere::strength ) );
    }
    return s_pMetaObject;
