@@ -25,7 +25,7 @@
 #include <qtextstream.h>
 #include <klocale.h>
 
-PMDefinePropertyClass( PMComment, PMProperty );
+PMDefinePropertyClass( PMComment, PMCommentProperty );
 
 PMMetaObject* PMComment::s_pMetaObject = 0;
 PMObject* createNewComment( PMPart* part )
@@ -115,7 +115,7 @@ PMMetaObject* PMComment::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Comment", Base::metaObject( ),
                                         createNewComment );
       s_pMetaObject->addProperty(
-         new PMProperty( "text", &PMComment::setText, &PMComment::text ) );
+         new PMCommentProperty( "text", &PMComment::setText, &PMComment::text ) );
    }
    return s_pMetaObject;
 }
