@@ -43,7 +43,7 @@ PMViewStructure* PMCone::s_pDefaultViewStructure = 0;
 int PMCone::s_numSteps = c_defaultConeSteps;
 int PMCone::s_parameterKey = 0;
 
-PMDefinePropertyClass( PMCone, PMProperty );
+PMDefinePropertyClass( PMCone, PMConeProperty );
 
 PMMetaObject* PMCone::s_pMetaObject = 0;
 PMObject* createNewCone( PMPart* part )
@@ -107,15 +107,15 @@ PMMetaObject* PMCone::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Cone", Base::metaObject( ),
                                         createNewCone );
       s_pMetaObject->addProperty(
-         new PMProperty( "end1", &PMCone::setEnd1, &PMCone::end1 ) );
+         new PMConeProperty( "end1", &PMCone::setEnd1, &PMCone::end1 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "end2", &PMCone::setEnd2, &PMCone::end2 ) );
+         new PMConeProperty( "end2", &PMCone::setEnd2, &PMCone::end2 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "radius1", &PMCone::setRadius1, &PMCone::radius1 ) );
+         new PMConeProperty( "radius1", &PMCone::setRadius1, &PMCone::radius1 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "radius2", &PMCone::setRadius2, &PMCone::radius2 ) );
+         new PMConeProperty( "radius2", &PMCone::setRadius2, &PMCone::radius2 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "open", &PMCone::setOpen, &PMCone::open ) );
+         new PMConeProperty( "open", &PMCone::setOpen, &PMCone::open ) );
    }
    return s_pMetaObject;
 }
