@@ -42,7 +42,7 @@ PMViewStructure* PMDisc::s_pDefaultViewStructure = 0;
 int PMDisc::s_numSteps = c_defaultDiscSteps;
 int PMDisc::s_parameterKey = 0;
 
-PMDefinePropertyClass( PMDisc, PMProperty );
+PMDefinePropertyClass( PMDisc, PMDiscProperty );
 
 PMMetaObject* PMDisc::s_pMetaObject = 0;
 PMObject* createNewDisc( PMPart* part )
@@ -103,13 +103,13 @@ PMMetaObject* PMDisc::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Disc", Base::metaObject( ),
                                         createNewDisc );
       s_pMetaObject->addProperty(
-         new PMProperty( "center", &PMDisc::setCenter, &PMDisc::center ) );
+         new PMDiscProperty( "center", &PMDisc::setCenter, &PMDisc::center ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "normal", &PMDisc::setNormal, &PMDisc::normal ) );
+         new PMDiscProperty( "normal", &PMDisc::setNormal, &PMDisc::normal ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "radius", &PMDisc::setRadius, &PMDisc::radius ) );
+         new PMDiscProperty( "radius", &PMDisc::setRadius, &PMDisc::radius ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "holeRadius", &PMDisc::setHoleRadius, &PMDisc::holeRadius ) );
+         new PMDiscProperty( "holeRadius", &PMDisc::setHoleRadius, &PMDisc::holeRadius ) );
    }
    return s_pMetaObject;
 }
