@@ -41,7 +41,7 @@ const double radiusGatherMultiDefault = 1.0;
 const double radiusMediaDefault = 0.0;
 const double radiusMediaMultiDefault = 1.0;
 
-PMDefinePropertyClass( PMGlobalPhotons, PMProperty );
+PMDefinePropertyClass( PMGlobalPhotons, PMGlobalPhotonsProperty );
 PMDefineEnumPropertyClass( PMGlobalPhotons, PMGlobalPhotons::PMNumberType, PMNumberProperty );
 
 PMMetaObject* PMGlobalPhotons::s_pMetaObject = 0;
@@ -114,41 +114,41 @@ PMMetaObject* PMGlobalPhotons::metaObject( ) const
       p1->addEnumValue( "Count", Count );
       s_pMetaObject->addProperty( p1 );
 
-      s_pMetaObject->addProperty( new PMProperty( "spacing",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "spacing",
          &PMGlobalPhotons::setSpacing, &PMGlobalPhotons::spacing ) );
-      s_pMetaObject->addProperty( new PMProperty( "count",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "count",
          &PMGlobalPhotons::setCount, &PMGlobalPhotons::count ) );
-      s_pMetaObject->addProperty( new PMProperty( "gatherMin",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "gatherMin",
          &PMGlobalPhotons::setGatherMin, &PMGlobalPhotons::gatherMin ) );
-      s_pMetaObject->addProperty( new PMProperty( "gatherMax",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "gatherMax",
          &PMGlobalPhotons::setGatherMax, &PMGlobalPhotons::gatherMax ) );
-      s_pMetaObject->addProperty( new PMProperty( "mediaMaxSteps",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "mediaMaxSteps",
          &PMGlobalPhotons::setMediaMaxSteps, &PMGlobalPhotons::mediaMaxSteps ) );
-      s_pMetaObject->addProperty( new PMProperty( "mediaFactor",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "mediaFactor",
          &PMGlobalPhotons::setMediaFactor, &PMGlobalPhotons::mediaFactor ) );
-      s_pMetaObject->addProperty( new PMProperty( "jitter",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "jitter",
          &PMGlobalPhotons::setJitter, &PMGlobalPhotons::jitter ) );
-      s_pMetaObject->addProperty( new PMProperty( "maxTraceLevelGlobal",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "maxTraceLevelGlobal",
          &PMGlobalPhotons::setMaxTraceLevelGlobal, &PMGlobalPhotons::maxTraceLevelGlobal ) );
-      s_pMetaObject->addProperty( new PMProperty( "maxTraceLevel",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "maxTraceLevel",
          &PMGlobalPhotons::setMaxTraceLevel, &PMGlobalPhotons::maxTraceLevel ) );
-      s_pMetaObject->addProperty( new PMProperty( "adcBailoutGlobal",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "adcBailoutGlobal",
          &PMGlobalPhotons::setAdcBailoutGlobal, &PMGlobalPhotons::adcBailoutGlobal ) );
-      s_pMetaObject->addProperty( new PMProperty( "adcBailout",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "adcBailout",
          &PMGlobalPhotons::setAdcBailout, &PMGlobalPhotons::adcBailout ) );
-      s_pMetaObject->addProperty( new PMProperty( "autostop",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "autostop",
          &PMGlobalPhotons::setAutostop, &PMGlobalPhotons::autostop ) );
-      s_pMetaObject->addProperty( new PMProperty( "expandIncrease",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "expandIncrease",
          &PMGlobalPhotons::setExpandIncrease, &PMGlobalPhotons::expandIncrease ) );
-      s_pMetaObject->addProperty( new PMProperty( "expandMin",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "expandMin",
          &PMGlobalPhotons::setExpandMin, &PMGlobalPhotons::expandMin ) );
-      s_pMetaObject->addProperty( new PMProperty( "radiusGather",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "radiusGather",
          &PMGlobalPhotons::setRadiusGather, &PMGlobalPhotons::radiusGather ) );
-      s_pMetaObject->addProperty( new PMProperty( "radiusGatherMulti",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "radiusGatherMulti",
          &PMGlobalPhotons::setRadiusGatherMulti, &PMGlobalPhotons::radiusGatherMulti ) );
-      s_pMetaObject->addProperty( new PMProperty( "radiusMedia",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "radiusMedia",
          &PMGlobalPhotons::setRadiusMedia, &PMGlobalPhotons::radiusMedia ) );
-      s_pMetaObject->addProperty( new PMProperty( "radiusMediaMulti",
+      s_pMetaObject->addProperty( new PMGlobalPhotonsProperty( "radiusMediaMulti",
          &PMGlobalPhotons::setRadiusMediaMulti, &PMGlobalPhotons::radiusMediaMulti ) );
    }
    return s_pMetaObject;
