@@ -43,7 +43,7 @@ const PMVector defaultPoint[defaultNumberOfPoints] =
 const bool defaultSturm = false;
 const PMLathe::SplineType defaultSplineType = PMLathe::LinearSpline;
 
-PMDefinePropertyClass( PMLathe, PMProperty );
+PMDefinePropertyClass( PMLathe, PMLatheProperty );
 PMDefineEnumPropertyClass( PMLathe, PMLathe::SplineType, PMSplineTypeProperty );
 
 PMMetaObject* PMLathe::s_pMetaObject = 0;
@@ -205,7 +205,7 @@ PMMetaObject* PMLathe::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Lathe", Base::metaObject( ),
                                         createNewLathe );
       s_pMetaObject->addProperty(
-         new PMProperty( "sturm", &PMLathe::setSturm, &PMLathe::sturm ) );
+         new PMLatheProperty( "sturm", &PMLathe::setSturm, &PMLathe::sturm ) );
       PMSplineTypeProperty* p = new PMSplineTypeProperty(
          "splineType", &PMLathe::setSplineType, &PMLathe::splineType );
       p->addEnumValue( "LinearSpline", LinearSpline );
