@@ -24,12 +24,15 @@
 
 #include <klocale.h>
 
+const int methodDefault = 1;
 const int intervalsDefault = 10;
 const int samplesMinDefault = 1;
 const int samplesMaxDefault = 1;
 const double confidenceDefault = 0.9;
 const double varianceDefault = 0.0078125;
 const double ratioDefault = 0.9;
+const int aaLevelDefault = 4;
+const double aaThresholdDefault = 0.1;
 const PMColor absorptionDefault = PMColor( 0.0, 0.0, 0.0 );
 const PMColor emissionDefault = PMColor( 0.0, 0.0, 0.0 );
 const int scatteringTypeDefault = 0;
@@ -37,7 +40,7 @@ const PMColor scatteringColorDefault = PMColor( 0.0, 0.0, 0.0 );
 const double scatteringEccentricityDefault = 0;
 const double scatteringExtinctionDefault = 1.0;
 
-PMDefinePropertyClass( PMMedia, PMProperty );
+PMDefinePropertyClass( PMMedia, PMMediaProperty );
 PMMetaObject* PMMedia::s_pMetaObject = 0;
 PMObject* createNewMedia( PMPart* part )
 {
@@ -47,23 +50,21 @@ PMObject* createNewMedia( PMPart* part )
 PMMedia::PMMedia( PMPart* part )
       : Base( part )
 {
+   m_method = methodDefault;
    m_intervals = intervalsDefault;
    m_samplesMin = samplesMinDefault;
    m_samplesMax = samplesMaxDefault;
    m_confidence = confidenceDefault;
    m_variance = varianceDefault;
    m_ratio = ratioDefault;
+   m_aaLevel = aaLevelDefault;
+   m_aaThreshold = aaThresholdDefault;
    m_absorption = absorptionDefault;
    m_emission = emissionDefault;
    m_scatteringType = scatteringTypeDefault;
    m_scatteringColor = scatteringColorDefault;
    m_scatteringEccentricity = scatteringEccentricityDefault;
    m_scatteringExtinction = scatteringExtinctionDefault;
-   m_enableIntervals = false;
-   m_enableSamples = false;
-   m_enableConfidence = false;
-   m_enableVariance = false;
-   m_enableRatio = false;
    m_enableAbsorption = false;
    m_enableEmission = false;
    m_enableScattering = false;
@@ -72,23 +73,21 @@ PMMedia::PMMedia( PMPart* part )
 PMMedia::PMMedia( const PMMedia& m )
       : Base( m )
 {
+   m_method = m.m_method;
    m_intervals = m.m_intervals;
    m_samplesMin = m.m_samplesMin;
    m_samplesMax = m.m_samplesMax;
+   m_aaLevel = m.m_aaLevel;
    m_confidence = m.m_confidence;
    m_variance = m.m_variance;
    m_ratio = m.m_ratio;
+   m_aaThreshold = m.m_aaThreshold;
    m_absorption = m.m_absorption;
    m_emission = m.m_emission;
    m_scatteringType = m.m_scatteringType;
    m_scatteringColor = m.m_scatteringColor;
    m_scatteringEccentricity = m.m_scatteringEccentricity;
    m_scatteringExtinction = m.m_scatteringExtinction;
-   m_enableIntervals = m.m_enableIntervals;
-   m_enableSamples = m.m_enableSamples;
-   m_enableConfidence = m.m_enableConfidence;
-   m_enableVariance = m.m_enableVariance;
-   m_enableRatio = m.m_enableRatio;
    m_enableAbsorption = m.m_enableAbsorption;
    m_enableEmission = m.m_enableEmission;
    m_enableScattering = m.m_enableScattering;
@@ -105,44 +104,42 @@ PMMetaObject* PMMedia::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Media", Base::metaObject( ),
                                         createNewMedia );
       s_pMetaObject->addProperty(
-         new PMProperty( "intervals", &PMMedia::setIntervals, &PMMedia::intervals ) );
+         new PMMediaProperty( "method", &PMMedia::setMethod, &PMMedia::method ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "samplesMin", &PMMedia::setSamplesMin, &PMMedia::samplesMin ) );
+         new PMMediaProperty( "intervals", &PMMedia::setIntervals, &PMMedia::intervals ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "confidence", &PMMedia::setConfidence, &PMMedia::confidence ) );
+         new PMMediaProperty( "samplesMin", &PMMedia::setSamplesMin, &PMMedia::samplesMin ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "variance", &PMMedia::setVariance, &PMMedia::variance ) );
+         new PMMediaProperty( "samplesMax", &PMMedia::setSamplesMax, &PMMedia::samplesMax ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "ratio", &PMMedia::setRatio, &PMMedia::ratio ) );
+         new PMMediaProperty( "aaLevel", &PMMedia::setAALevel, &PMMedia::aaLevel ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "absorption", &PMMedia::setAbsorption, &PMMedia::absorption ) );
+         new PMMediaProperty( "confidence", &PMMedia::setConfidence, &PMMedia::confidence ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "emission", &PMMedia::setEmission, &PMMedia::emission ) );
+         new PMMediaProperty( "variance", &PMMedia::setVariance, &PMMedia::variance ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "scatteringType", &PMMedia::setScatteringType, &PMMedia::scatteringType ) );
+         new PMMediaProperty( "ratio", &PMMedia::setRatio, &PMMedia::ratio ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "scatteringColor", &PMMedia::setScatteringColor, &PMMedia::scatteringColor ) );
+         new PMMediaProperty( "aaThreshold", &PMMedia::setAAThreshold, &PMMedia::aaThreshold ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "scatteringExtinction", &PMMedia::setScatteringExtinction, &PMMedia::scatteringExtinction ) );
+         new PMMediaProperty( "absorption", &PMMedia::setAbsorption, &PMMedia::absorption ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "scatteringEccentricity", &PMMedia::setScatteringEccentricity, &PMMedia::scatteringEccentricity ) );
+         new PMMediaProperty( "emission", &PMMedia::setEmission, &PMMedia::emission ) );
+      s_pMetaObject->addProperty(
+         new PMMediaProperty( "scatteringType", &PMMedia::setScatteringType, &PMMedia::scatteringType ) );
+      s_pMetaObject->addProperty(
+         new PMMediaProperty( "scatteringColor", &PMMedia::setScatteringColor, &PMMedia::scatteringColor ) );
+      s_pMetaObject->addProperty(
+         new PMMediaProperty( "scatteringExtinction", &PMMedia::setScatteringExtinction, &PMMedia::scatteringExtinction ) );
+      s_pMetaObject->addProperty(
+         new PMMediaProperty( "scatteringEccentricity", &PMMedia::setScatteringEccentricity, &PMMedia::scatteringEccentricity ) );
 
       s_pMetaObject->addProperty(
-         new PMProperty( "intervalsEnabled", &PMMedia::enableIntervals, &PMMedia::isIntervalsEnabled ) );
+         new PMMediaProperty( "absorptionEnabled", &PMMedia::enableAbsorption, &PMMedia::isAbsorptionEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "samplesEnabled", &PMMedia::enableSamples, &PMMedia::isSamplesEnabled ) );
+         new PMMediaProperty( "emissionEnabled", &PMMedia::enableEmission, &PMMedia::isEmissionEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "confidenceEnabled", &PMMedia::enableConfidence, &PMMedia::isConfidenceEnabled ) );
-      s_pMetaObject->addProperty(
-         new PMProperty( "varianceEnabled", &PMMedia::enableVariance, &PMMedia::isVarianceEnabled ) );
-      s_pMetaObject->addProperty(
-         new PMProperty( "ratioEnabled", &PMMedia::enableRatio, &PMMedia::isRatioEnabled ) );
-      s_pMetaObject->addProperty(
-         new PMProperty( "absorptionEnabled", &PMMedia::enableAbsorption, &PMMedia::isAbsorptionEnabled ) );
-      s_pMetaObject->addProperty(
-         new PMProperty( "emissionEnabled", &PMMedia::enableEmission, &PMMedia::isEmissionEnabled ) );
-      s_pMetaObject->addProperty(
-         new PMProperty( "scatteringEnabled", &PMMedia::enableScattering, &PMMedia::isScatteringEnabled ) );
+         new PMMediaProperty( "scatteringEnabled", &PMMedia::enableScattering, &PMMedia::isScatteringEnabled ) );
    }
    return s_pMetaObject;
 }
@@ -165,20 +162,18 @@ QString PMMedia::description( ) const
 void PMMedia::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    Base::serialize( e, doc );
-   e.setAttribute( "enable_intervals", m_enableIntervals );
-   e.setAttribute( "enable_samples", m_enableSamples );
-   e.setAttribute( "enable_confidence", m_enableConfidence );
-   e.setAttribute( "enable_variance", m_enableVariance );
-   e.setAttribute( "enable_ratio", m_enableRatio );
    e.setAttribute( "enable_absorption", m_enableAbsorption );
    e.setAttribute( "enable_emission", m_enableEmission );
    e.setAttribute( "enable_scattering", m_enableScattering );
+   e.setAttribute( "method", m_method );
    e.setAttribute( "intervals", m_intervals );
    e.setAttribute( "samples_min", m_samplesMin );
    e.setAttribute( "samples_max", m_samplesMax );
+   e.setAttribute( "aa_level", m_aaLevel );
    e.setAttribute( "confidence", m_confidence );
    e.setAttribute( "variance", m_variance );
    e.setAttribute( "ratio", m_ratio );
+   e.setAttribute( "aa_threshold", m_aaThreshold );
    e.setAttribute( "absorption", m_absorption.serializeXML( ) );
    e.setAttribute( "emission", m_emission.serializeXML( ) );
    e.setAttribute( "scattering_type", m_scatteringType );
@@ -190,20 +185,18 @@ void PMMedia::serialize( QDomElement& e, QDomDocument& doc ) const
 void PMMedia::readAttributes( const PMXMLHelper& h )
 {
    Base::readAttributes( h );
-   m_enableIntervals = h.boolAttribute( "enable_intervals", false );
-   m_enableSamples = h.boolAttribute( "enable_samples", false );
-   m_enableConfidence = h.boolAttribute( "enable_confidence", false );
-   m_enableVariance = h.boolAttribute( "enable_variance", false );
-   m_enableRatio = h.boolAttribute( "enable_ratio", false );
    m_enableAbsorption = h.boolAttribute( "enable_absorption", false );
    m_enableEmission = h.boolAttribute( "enable_emission", false );
    m_enableScattering = h.boolAttribute( "enable_scattering", false );
+   m_method = h.intAttribute( "method", methodDefault );
    m_intervals = h.intAttribute( "intervals", intervalsDefault );
    m_samplesMin = h.intAttribute( "samples_min", samplesMinDefault );
    m_samplesMax = h.intAttribute( "samples_max", samplesMaxDefault );
+   m_aaLevel = h.intAttribute( "aa_level", aaLevelDefault );
    m_confidence = h.doubleAttribute( "confidence", confidenceDefault );
    m_variance = h.doubleAttribute( "variance", varianceDefault );
    m_ratio = h.doubleAttribute( "ratio", ratioDefault );
+   m_aaThreshold = h.doubleAttribute( "aa_threshold", aaThresholdDefault );
    m_absorption = h.colorAttribute( "absorption", absorptionDefault );
    m_emission = h.colorAttribute( "emission", emissionDefault );
    m_scatteringType = h.intAttribute( "scattering_type", scatteringTypeDefault );
@@ -214,6 +207,28 @@ void PMMedia::readAttributes( const PMXMLHelper& h )
                                                        scatteringExtinctionDefault );
 }
 
+void PMMedia::setMethod( int c )
+{
+   if ( c < 1 )
+   {
+      kdError( PMArea ) << "method is < 1 in PMMedia::setMethod\n";
+      c = 1;
+   }
+
+   if ( c > 3 )
+   {
+      kdError( PMArea ) << "method is > 3 in PMMedia::setMethod\n";
+      c = 3;
+   }
+
+   if ( c != m_method )
+   {
+      if ( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMMethodID, m_method );
+      m_method = c;
+   }
+}
+
 void PMMedia::setIntervals( int c )
 {
    if( c != m_intervals )
@@ -244,6 +259,16 @@ void PMMedia::setSamplesMax( int c )
    }
 }
 
+void PMMedia::setAALevel( int c )
+{
+   if ( c != m_aaLevel )
+   {
+      if ( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMAALevelID, m_aaLevel );
+      m_aaLevel = c;
+   }
+}
+
 void PMMedia::setConfidence( double c )
 {
    if( c != m_confidence )
@@ -274,6 +299,16 @@ void PMMedia::setRatio( double c )
    }
 }
 
+void PMMedia::setAAThreshold( double c )
+{
+   if ( c != m_aaThreshold )
+   {
+      if ( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMAAThresholdID, m_aaThreshold );
+      m_aaThreshold = c;
+   }
+}
+
 void PMMedia::setAbsorption( const PMColor& c )
 {
    if( c != m_absorption )
@@ -334,56 +369,6 @@ void PMMedia::setScatteringExtinction( double c )
    }
 }
 
-void PMMedia::enableIntervals( bool c )
-{
-   if( c != m_enableIntervals )
-   {
-      if( m_pMemento )
-         m_pMemento->addData( s_pMetaObject, PMEnableIntervalsID, m_enableIntervals );
-      m_enableIntervals = c;
-   }
-}
-
-void PMMedia::enableSamples( bool c )
-{
-   if( c != m_enableSamples )
-   {
-      if( m_pMemento )
-         m_pMemento->addData( s_pMetaObject, PMEnableSamplesID, m_enableSamples );
-      m_enableSamples = c;
-   }
-}
-
-void PMMedia::enableConfidence( bool c )
-{
-   if( c != m_enableConfidence )
-   {
-      if( m_pMemento )
-         m_pMemento->addData( s_pMetaObject, PMEnableConfidenceID, m_enableConfidence );
-      m_enableConfidence = c;
-   }
-}
-
-void PMMedia::enableVariance( bool c )
-{
-   if( c != m_enableVariance )
-   {
-      if( m_pMemento )
-         m_pMemento->addData( s_pMetaObject, PMEnableVarianceID, m_enableVariance );
-      m_enableVariance = c;
-   }
-}
-
-void PMMedia::enableRatio( bool c )
-{
-   if( c != m_enableRatio )
-   {
-      if( m_pMemento )
-         m_pMemento->addData( s_pMetaObject, PMEnableRatioID, m_enableRatio );
-      m_enableRatio = c;
-   }
-}
-
 void PMMedia::enableAbsorption( bool c )
 {
    if( c != m_enableAbsorption )
@@ -431,21 +416,6 @@ void PMMedia::restoreMemento( PMMemento* s )
       {
          switch( data->valueID( ) )
          {
-            case PMEnableIntervalsID:
-               enableIntervals( data->boolData( ) );
-               break;
-            case PMEnableSamplesID:
-               enableSamples( data->boolData( ) );
-               break;
-            case PMEnableConfidenceID:
-               enableConfidence( data->boolData( ) );
-               break;
-            case PMEnableVarianceID:
-               enableVariance( data->boolData( ) );
-               break;
-            case PMEnableRatioID:
-               enableRatio( data->boolData( ) );
-               break;
             case PMEnableAbsorptionID:
                enableAbsorption( data->boolData( ) );
                break;
@@ -455,6 +425,9 @@ void PMMedia::restoreMemento( PMMemento* s )
             case PMEnableScatteringID:
                enableScattering( data->boolData( ) );
                break;
+            case PMMethodID:
+               setMethod( data->intData( ) );
+               break;
             case PMIntervalsID:
                setIntervals( data->intData( ) );
                break;
@@ -464,6 +437,9 @@ void PMMedia::restoreMemento( PMMemento* s )
             case PMSamplesMaxID:
                setSamplesMax( data->intData( ) );
                break;
+            case PMAALevelID:
+               setAALevel( data->intData( ) );
+               break;
             case PMConfidenceID:
                setConfidence( data->doubleData( ) );
                break;
@@ -473,6 +449,9 @@ void PMMedia::restoreMemento( PMMemento* s )
             case PMRatioID:
                setRatio( data->doubleData( ) );
                break;
+            case PMAAThresholdID:
+               setAAThreshold( data->doubleData( ) );
+               break;
             case PMAbsorptionID:
                setAbsorption( data->colorData( ) );
                break;
