@@ -75,7 +75,6 @@
 #include "pmdialogeditbase.h"
 #include "pmdocumentationmap.h"
 #include "pmlibrarymanager.h"
-#include "pmlibraryobjectpreview.h"
 #include "pmlibraryhandleedit.h"
 #include "pmlibraryobject.h"
 #include "pmlibrarybrowser.h"
@@ -98,10 +97,14 @@
 QTime PMDebugTime;
 #endif
 
+//#define KPM_WITH_OBJECT_LIBRARY
+
 PMPart::PMPart( QWidget* parentWidget, const char* widgetName,
                 QObject* parent, const char* name, bool readwrite,
                 PMShell* shell )
-      : KParts::ReadWritePart( parent, name ), m_commandManager( this )
+      : DCOPObject( "PMPartIface" ),
+        KParts::ReadWritePart( parent, name ),
+        m_commandManager( this )
 {
    setPluginLoadingMode( LoadPlugins );
    setInstance( PMFactory::instance( ), false );
@@ -158,10 +161,12 @@ PMPart::PMPart( QWidget* parentWidget, const char* widgetName,
    slotObjectChanged( m_pScene, PMCNewSelection, this );
 }
 
-PMPart::PMPart( QWidget* parentWidget, const char* widgetName,
+PMPart::PMPart( QWidget* /*parentWidget*/, const char* /*widgetName*/,
                 QObject* parent, const char* name, bool readwrite,
-                bool onlyCutPaste, PMShell* shell )
-      : KParts::ReadWritePart( parent, name ), m_commandManager( this )
+                bool /*onlyCutPaste*/, PMShell* shell )
+      : DCOPObject( "LibraryBrowserIface" ),
+        KParts::ReadWritePart( parent, name ),
+        m_commandManager( this )
 {
    setPluginLoadingMode( LoadPlugins );
    setInstance( PMFactory::instance( ), false );
@@ -193,12 +198,13 @@ PMPart::PMPart( QWidget* parentWidget, const char* widgetName,
    m_pInsertRuleSystem = new PMInsertRuleSystem( this );
    m_pIOManager = new PMIOManager( this );
    m_pInsertRuleSystem->loadRules( "baseinsertrules.xml" );
+   m_pSymbolTable = new PMSymbolTable( );
+
+   initCopyPasteActions( );
 
    connect( &m_commandManager, SIGNAL( objectChanged( PMObject*, const int, QObject* ) ),
             SLOT( slotObjectChanged( PMObject*, const int, QObject* ) ) );
 
-   initCopyPasteActions( );
-
    PMPluginManager::theManager( )->registerPart( this );
 
    emit refresh( );
@@ -486,7 +492,7 @@ void PMPart::initActions( )
                                      actionCollection( ), "new_rotate" );
       m_readWriteActions.append( m_pNewRotateAction );
       m_pNewMatrixAction = new KAction( i18n( "Matrix" ), "pmmatrix", 0, this, SLOT( slotNewMatrix( ) ),
-                                        actionCollection( ), "new_matrix" );
+                                        actionCollection( ), "new_povraymatrix" );
       m_readWriteActions.append( m_pNewMatrixAction );
 
       m_pNewCommentAction = new KAction( i18n( "Comment" ), "pmcomment", 0, this, SLOT( slotNewComment( ) ),
@@ -523,9 +529,6 @@ void PMPart::initActions( )
       m_readWriteActions.append( m_pNewMeshAction );
 
 #ifdef KPM_WITH_OBJECT_LIBRARY
-      m_pImportLibraryObjectAction = new KAction( i18n( "Library Object" ), "pmdeclare", 0, this, SLOT( slotImportLibraryObject( ) ),
-                                         actionCollection( ), "import_library_object" );
-      m_readWriteActions.append( m_pImportLibraryObjectAction );
       m_pSearchLibraryObjectAction = new KAction( i18n( "Search Object" ), "pmsearchlibrary", 0, this, SLOT( slotSearchLibraryObject( ) ),
                                          actionCollection( ), "search_library_object" );
       m_readWriteActions.append( m_pSearchLibraryObjectAction );
@@ -632,8 +635,6 @@ void PMPart::initActions( )
 
       // POV-Ray
 
-      m_pImportLibraryObjectAction = 0;
-      m_pExportLibraryObjectAction = 0;
       m_pUndoAction = 0;
       m_pRedoAction = 0;
    }
@@ -747,7 +748,6 @@ void PMPart::saveConfig( KConfig* cfg )
    PMLibraryHandleEdit::saveConfig( cfg );
    PMDocumentationMap::theMap( )->saveConfig( cfg );
    PMLibraryManager::theManager( )->saveConfig(cfg );
-   PMLibraryBrowser::saveConfig( cfg );
 
    cfg->setGroup( "Rendering" );
    cfg->writeEntry( "SphereUSteps", PMSphere::uSteps( ) );
@@ -790,7 +790,6 @@ void PMPart::restoreConfig( KConfig* cfg )
    PMLibraryHandleEdit::restoreConfig( cfg );
    PMDocumentationMap::theMap( )->restoreConfig( cfg );
    PMLibraryManager::theManager( )->restoreConfig(cfg );
-   PMLibraryBrowser::restoreConfig( cfg );
 
    cfg->setGroup( "Rendering" );
    PMSphere::setUSteps( cfg->readNumEntry( "SphereUSteps", c_defaultSphereUSteps ) );
@@ -939,7 +938,7 @@ bool PMPart::exportPovray( const KURL& url )
       if( tempFile )
       {
          tempFile->close( );
-         ok = KIO::NetAccess::upload( tempFile->name( ), url );
+         ok = KIO::NetAccess::upload( tempFile->name( ), url, (QWidget*) 0 );
          tempFile->unlink( );
          file = 0;
       }
@@ -953,6 +952,186 @@ bool PMPart::exportPovray( const KURL& url )
    return ok;
 }
 
+QString PMPart::activeObjectName( )
+{
+   QString result = "";
+   PMObject* tmpObj;
+   PMObject* testSib;
+   int idx = 0;
+
+   tmpObj = activeObject( );
+   while( tmpObj != m_pScene && tmpObj )
+   {
+      // count previous siblings of the same type (for array like entries)
+      testSib = tmpObj;
+      while( ( testSib = testSib->prevSibling( ) ) )
+         if( testSib->type( ) == tmpObj->type( ) )
+            idx++;
+
+      // prepend to result
+      if( idx != 0 )
+         result = tmpObj->type( ) + "[" + QString::number( idx ) + "]/" + result;
+      else
+         result = tmpObj->type( ) + "/" + result;
+
+      // go up in the scene
+      tmpObj = tmpObj->parent( );
+      idx = 0;
+   }
+
+   // Make the object name an absolute name
+   result = "/" + result;
+
+   return result;
+}
+
+bool PMPart::setActiveObject( const QString& name )
+{
+   PMObject* tmpObj;
+   PMObject* tmpSibling;
+   int pos, siblingIndex, objIndex, firstBracket, lastBracket;
+   QString objPath;
+   QString pathElem;
+
+   // check if it's a absolute object name
+   if( name[0] = '/' )
+   {
+      tmpObj = m_pScene;
+      objPath = name.mid( 1 );  // clear that first character
+   }
+   else
+      tmpObj = activeObject( );
+
+   // get the first element
+   pos = objPath.find( '/' );
+   if( pos != -1 )
+   {
+      pathElem = objPath.mid( 0, pos );
+      objPath = objPath.mid( pos + 1 );
+   }
+   else
+   {
+      pathElem = objPath;
+      objPath = QString::null;
+   }
+
+   while( !pathElem.isNull( ) )
+   {
+      if( !pathElem.isEmpty( ) )
+      {
+         // Special treatment for brackets
+         firstBracket = pathElem.find( '[' );
+         if( firstBracket != -1 )
+         {
+            lastBracket = pathElem.findRev( ']' );
+            objIndex = pathElem.mid( firstBracket + 1, lastBracket - firstBracket - 1).toInt( );
+            pathElem = pathElem.left( firstBracket );
+         }
+         else
+            objIndex = 0;
+
+         // Iterate the children for this element. We stop when there are no more siblings
+         // or the object is of the correct type and it's index count is also correct
+         siblingIndex = 0;
+         tmpSibling = tmpObj->firstChild( );
+         for( ; tmpSibling && ( tmpSibling->type( ) != pathElem || siblingIndex != objIndex );
+                                                                tmpSibling = tmpSibling->nextSibling( ) )
+         {
+            // Found an object of the type we are looking for
+            if( tmpSibling->type( ) == pathElem )
+               siblingIndex++;
+         }
+         if( tmpSibling )
+            tmpObj = tmpSibling;
+         else
+            // No correct sibling
+            return false;
+
+      }
+
+      // Get the next element
+      pos = objPath.find( '/' );
+      if( pos != -1 )
+      {
+         pathElem = objPath.mid( 0, pos );
+         objPath = objPath.mid( pos + 1 );
+      }
+      else
+      {
+         pathElem = objPath;
+         objPath = QString::null;
+      }
+   }
+   if( tmpObj )
+   {
+      slotObjectChanged( tmpObj, PMCNewSelection, this );
+      return true;
+   }
+   else
+      return false;
+}
+
+QStringList PMPart::getProperties( )
+{
+   PMObject* curObj = activeObject( );
+
+   // Ensure that there is an active object
+   if( !curObj )
+      return QStringList( );
+
+   return curObj->properties( );
+}
+
+bool PMPart::setProperty( const QString& name, const PMVariant& value )
+{
+   PMObject* curObj = activeObject( );
+
+   // Ensure that there is an active object
+   if( !curObj )
+      return false;
+
+   curObj->setProperty( name, value );
+   slotObjectChanged( curObj, PMCNewSelection, this );
+   return true;
+}
+
+bool PMPart::setProperty( const QString& name, const QString& value )
+{
+   PMObject* curObj = activeObject( );
+   PMVariant variant;
+
+   // Ensure that there is an active object
+   if( !curObj )
+      return false;
+
+   variant.fromString( curObj->property( name ).dataType( ), value );
+   curObj->setProperty( name, variant );
+   slotObjectChanged( curObj, PMCNewSelection, this );
+   return true;
+}
+
+const PMVariant PMPart::getProperty( const QString& name )
+{
+   PMObject* curObj = activeObject( );
+
+   // Ensure that there is an active object
+   if( !curObj )
+      return PMVariant( );
+
+   return curObj->property( name );
+}
+
+const QString PMPart::getPropertyStr( const QString& name )
+{
+   PMObject* curObj = activeObject( );
+
+   // Ensure that there is an active object
+   if( !curObj )
+      return PMVariant( ).asString( );
+
+   return curObj->property( name ).asString( );
+}
+
 const PMObjectList& PMPart::selectedObjects( )
 {
    uint numObjects = m_selectedObjects.count( );
@@ -1801,18 +1980,21 @@ void PMPart::slotIDChanged( PMObject* obj, const QString& oldID )
    }
 }
 
-void PMPart::slotNewObject( PMObject* newObject )
+void PMPart::slotNewObject( PMObject* newObject, int insertAs )
 {
    PMObjectList list;
    list.append( newObject );
-   int insertAs;
    PMCommand* command = 0;
 
    if( m_pActiveObject )
    {
-      insertAs = whereToInsert( m_pActiveObject, list );
+      // If no position was specified ask the user
+      if( insertAs <= 0 )
+         insertAs = whereToInsert( m_pActiveObject, list );
+      // If either through a parameter or by user action a position was selected
       if( insertAs > 0 )
       {
+         // insert the object in the position indicated
          switch( insertAs )
          {
             case PMInsertPopup::PMIFirstChild:
@@ -1853,6 +2035,34 @@ void PMPart::slotNewObject( const QString& type )
       slotNewObject( newObject );
 }
 
+void PMPart::slotNewObject( const QString& type, const QString& pos )
+{
+   PMObject* newObject = m_pPrototypeManager->newObject( type );
+   if( newObject )
+   {
+      if( pos == "FirstChild" )
+         slotNewObject( newObject, PMInsertPopup::PMIFirstChild );
+      else if( pos == "LastChild" )
+         slotNewObject( newObject, PMInsertPopup::PMILastChild );
+      else if( pos == "Sibling" )
+         slotNewObject( newObject, PMInsertPopup::PMISibling );
+      else
+         slotNewObject( newObject );
+   }
+}
+
+QStringList PMPart::getObjectTypes( )
+{
+   QStringList result;
+   QPtrListIterator<PMMetaObject> it = m_pPrototypeManager->prototypeIterator( );
+
+   for( ; it.current( ); ++it )
+   {
+      result.append( it.current( )->className( ) );
+   }
+   return result;
+}
+
 void PMPart::slotNewTransformedObject( PMObject* o )
 {
    if( o )
@@ -2266,11 +2476,6 @@ void PMPart::slotNewMesh( )
    slotNewObject( new PMMesh( this ) );
 }
 
-void PMPart::slotImportLibraryObject( )
-{
-   PMLibraryBrowser::showBrowser( );
-}
-
 void PMPart::slotSearchLibraryObject( )
 {
    PMLibraryObjectSearch* aux = new PMLibraryObjectSearch( NULL );
