@@ -1174,6 +1174,12 @@ void PMPov31SerMaterialMap( const PMObject* object, const PMMetaObject* metaObje
    dev->objectEnd( );
 }
 
+const int c_defaultMediaIntervals = 10;
+const int c_defaultMediaSamplesMin = 1;
+const int c_defaultMediaSamplesMax = 1;
+const double c_defaultMediaConfidence = 0.9;
+const double c_defaultMediaVariance = 0.0078125;
+const double c_defaultMediaRatio = 0.9;
 const double c_defaultMediaScatteringEccentricity = 0;
 const double c_defaultMediaScatteringExtinction = 1.0;
 
@@ -1186,28 +1192,29 @@ void PMPov31SerMedia( const PMObject* object, const PMMetaObject* metaObject, PM
 
    dev->objectBegin( "media" );
    dev->callSerialization( object, metaObject->superClass( ) );
-   if( o->isIntervalsEnabled( ) )
+   if( o->intervals( ) != c_defaultMediaIntervals )
    {
       str1.setNum( o->intervals( ) );
       dev->writeLine( "intervals " + str1 );
    }
-   if( o->isSamplesEnabled( ) )
+   if( o->samplesMin( ) != c_defaultMediaSamplesMin ||
+       o->samplesMax( ) != c_defaultMediaSamplesMax )
    {
       str1.setNum( o->samplesMin( ) );
       str2.setNum( o->samplesMax( ) );
       dev->writeLine( "samples " + str1 + "," + str2 );
    }
-   if( o->isConfidenceEnabled( ) )
+   if( o->confidence( ) != c_defaultMediaConfidence )
    {
       str1.setNum( o->confidence( ) );
       dev->writeLine( "confidence " + str1 );
    }
-   if( o->isVarianceEnabled( ) )
+   if( o->variance( ) != c_defaultMediaVariance )
    {
       str1.setNum( o->variance( ) );
       dev->writeLine( "variance " + str1 );
    }
-   if( o->isRatioEnabled( ) )
+   if( o->ratio( ) != c_defaultMediaRatio )
    {
       str1.setNum( o->ratio( ) );
       dev->writeLine( "ratio " + str1 );
@@ -1393,6 +1400,8 @@ void PMPov31SerPattern( const PMObject* object, const PMMetaObject*, PMOutputDev
       case PMPattern::PatternWrinkles:
          dev->writeLine( "wrinkles" );
          break;
+      default:
+         break;
    }
    // depth
    if( o->parent( ) )
