@@ -18,7 +18,6 @@
 
 #include "pmprism.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmprismedit.h"
 #include "pmmemento.h"
@@ -28,9 +27,8 @@
 #include "pmprismmemento.h"
 #include "pmsplinesegment.h"
 #include "pmdefaults.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
+#include "pmobjectaction.h"
 
 #include <klocale.h>
 
@@ -54,9 +52,107 @@ const double defaultHeight2 = 1.0;
 
 int PMPrism::s_sSteps = c_defaultPrismSSteps;
 int PMPrism::s_parameterKey = 0;
+PMMetaObject* PMPrism::s_pMetaObject = 0;
+PMObject* createNewPrism( PMPart* part )
+{
+   return new PMPrism( part );
+}
 
-PMPrism::PMPrism( )
-      : Base( )
+PMDefinePropertyClass( PMPrism, PMPrismProperty );
+PMDefineEnumPropertyClass( PMPrism, PMPrism::SplineType, PMSplineTypeProperty );
+PMDefineEnumPropertyClass( PMPrism, PMPrism::SweepType, PMSweepTypeProperty );
+
+class PMPointProperty : public PMPropertyBase
+{
+public:
+   PMPointProperty( )
+         : PMPropertyBase( "splinePoints", PMVariant::Vector )
+   {
+      m_index[0] = 0;
+      m_index[1] = 0;
+   }
+   virtual int dimensions( ) const { return 2; }
+   virtual void setIndex( int dimension, int index )
+   {
+      if( dimension == 0 || dimension == 1 )
+         m_index[dimension] = index;
+   }
+   virtual int size( PMObject* object, int dimension ) const
+   {
+      PMPrism* prism = ( PMPrism* ) object;
+      QValueList< QValueList<PMVector> > points = prism->points( );
+      if( dimension == 0 )
+         return points.size( );
+      else
+      {
+         QValueList< QValueList<PMVector> >::ConstIterator it
+            = points.at( m_index[0] );
+         if( it != points.end( ) )
+            return ( *it ).size( );
+      }
+      return 0;
+   }
+protected:
+   virtual bool setProtected( PMObject* obj, const PMVariant& var )
+   {
+      PMPrism* p = ( PMPrism* ) obj;
+      QValueList< QValueList<PMVector> > list = p->points( );
+      QValueList< QValueList<PMVector> >::Iterator sit = list.begin( );
+      int i;
+      PMVector v = var.vectorData( );
+      v.resize( 2 );
+
+      for( i = 0; i < m_index[0] && sit != list.end( ); i++ )
+         ++sit;
+      // expand the list if necessary
+      for( ; i < m_index[0]; i++ )
+         list.insert( sit, QValueList< PMVector >( ) );
+      if( sit == list.end( ) )
+         sit = list.insert( sit, QValueList< PMVector >( ) );
+
+      QValueList<PMVector>::Iterator it = ( *sit ).begin( );
+
+      for( i = 0; i < m_index[1] && it != ( *sit ).end( ); i++ )
+         ++it;
+      // expand the list if necessary
+      for( ; i < m_index[1]; i++ )
+         ( *sit ).insert( it, v );
+      if( it == ( *sit ).end( ) )
+         it = ( *sit ).insert( it, v );
+      else
+         *it = v;
+
+      p->setPoints( list );
+      return true;
+   }
+   virtual PMVariant getProtected( const PMObject* obj )
+   {
+      PMPrism* p = ( PMPrism* ) obj;
+      QValueList< QValueList<PMVector> > list = p->points( );
+      QValueList< QValueList<PMVector> >::ConstIterator sit = list.at( m_index[0] );
+      if( sit == list.end( ) )
+      {
+         kdError( PMArea ) << "Range error in PMPrism::PointProperty::get" << endl;
+         return PMVariant( );
+      }
+
+      QValueList<PMVector>::ConstIterator it = ( *sit ).at( m_index[1] );
+
+      if( it == ( *sit ).end( ) )
+      {
+         kdError( PMArea ) << "Range error in PMPrism::PointProperty::get" << endl;
+         return PMVariant( );
+      }
+
+      return PMVariant( *it );
+   }
+
+private:
+   int m_index[2];
+};
+
+PMPrism::PMPrism( PMPart* part )
+      : Base( part )
 {
    int i;
    QValueList<PMVector> p;
@@ -72,6 +168,18 @@ PMPrism::PMPrism( )
    m_height2 = defaultHeight2;
 }
 
+PMPrism::PMPrism( const PMPrism& p )
+      : Base( p )
+{
+   m_splineType = p.m_splineType;
+   m_sweepType = p.m_sweepType;
+   m_points = p.m_points;
+   m_height1 = p.m_height1;
+   m_height2 = p.m_height2;
+   m_open = p.m_open;
+   m_sturm = p.m_sturm;
+}
+
 PMPrism::~PMPrism( )
 {
 }
@@ -81,80 +189,6 @@ QString PMPrism::description( ) const
    return i18n( "prism" );
 }
 
-void PMPrism::serialize( PMOutputDevice& dev ) const
-{
-   dev.objectBegin( "prism" );
-
-   serializeName( dev );
-
-   switch( m_splineType )
-   {
-      case LinearSpline:
-         dev.writeLine( "linear_spline" );
-         break;
-      case QuadraticSpline:
-         dev.writeLine( "quadratic_spline" );
-         break;
-      case CubicSpline:
-         dev.writeLine( "cubic_spline" );
-         break;
-      case BezierSpline:
-         dev.writeLine( "bezier_spline" );
-         break;
-   }
-   switch( m_sweepType )
-   {
-      case LinearSweep:
-         dev.writeLine( "linear_sweep" );
-         break;
-      case ConicSweep:
-         dev.writeLine( "conic_sweep" );
-         break;
-   }
-   dev.writeLine( QString( "%1, %2," ).arg( m_height1 ).arg( m_height2 ) );
-
-   // count number of points
-   QValueList< QValueList<PMVector> >::ConstIterator spit = m_points.begin( );
-   int lines = 0;
-   for( ; spit != m_points.end( ); ++spit )
-   {
-      if( m_splineType != BezierSpline )
-         lines += ( *spit ).count( ) + 1;
-      else
-         lines += ( *spit ).count( ) / 3 * 4;
-   }
-   dev.writeLine( QString( "%1," ).arg( lines ) );
-
-   for( spit = m_points.begin( ); spit != m_points.end( ); ++spit )
-   {
-      bool first = true;
-   
-      QValueList<PMVector> fullPoints = expandedPoints( *spit );
-      QValueList<PMVector>::ConstIterator it = fullPoints.begin( );
-      
-      for( ; it != fullPoints.end( ); ++it )
-      {
-         if( !first )
-            dev.write( ", " );
-         dev.write( ( *it ).serialize( ) );
-         first = false;
-      }
-      QValueList< QValueList<PMVector> >::ConstIterator spit2 = spit;
-      spit2++;
-      if( spit2 != m_points.end( ) )
-         dev.write( "," );
-      dev.writeLine( "" );
-   }
-
-   if( m_open )
-      dev.writeLine( "open" );
-   if( m_sturm )
-      dev.writeLine( "sturm" );
-   
-   Base::serialize( dev );
-   dev.objectEnd( );
-}
-
 void PMPrism::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    QDomElement data = doc.createElement( "extra_data" );
@@ -238,11 +272,48 @@ void PMPrism::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMPrism::isA( PMObjectType t ) const
+PMMetaObject* PMPrism::metaObject( ) const
 {
-   if( t == PMTPrism )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "Prism", Base::metaObject( ),
+                                        createNewPrism );
+      s_pMetaObject->addProperty(
+         new PMPrismProperty( "sturm", &PMPrism::setSturm, &PMPrism::sturm ) );
+      s_pMetaObject->addProperty(
+         new PMPrismProperty( "open", &PMPrism::setOpen, &PMPrism::open ) );
+      s_pMetaObject->addProperty(
+         new PMPrismProperty( "height1", &PMPrism::setHeight1, &PMPrism::height1 ) );
+      s_pMetaObject->addProperty(
+         new PMPrismProperty( "height2", &PMPrism::setHeight2, &PMPrism::height2 ) );
+
+      PMSplineTypeProperty* p = new PMSplineTypeProperty(
+         "splineType", &PMPrism::setSplineType, &PMPrism::splineType );
+      p->addEnumValue( "LinearSpline", LinearSpline );
+      p->addEnumValue( "QuadraticSpline", QuadraticSpline );
+      p->addEnumValue( "CubicSpline", CubicSpline );
+      p->addEnumValue( "BezierSpline", BezierSpline );
+      s_pMetaObject->addProperty( p );
+
+      PMSweepTypeProperty* sp = new PMSweepTypeProperty(
+         "sweepType", &PMPrism::setSweepType, &PMPrism::sweepType );
+      sp->addEnumValue( "LinearSweep", LinearSweep );
+      sp->addEnumValue( "ConicSweep", ConicSweep );
+      s_pMetaObject->addProperty( sp );
+
+      s_pMetaObject->addProperty( new PMPointProperty( ) );
+   }
+   return s_pMetaObject;
+}
+
+void PMPrism::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 void PMPrism::setSplineType( PMPrism::SplineType t )
@@ -250,7 +321,7 @@ void PMPrism::setSplineType( PMPrism::SplineType t )
    if( m_splineType != t )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPrism, PMSplineTypeID, ( int ) m_splineType );
+         m_pMemento->addData( s_pMetaObject, PMSplineTypeID, ( int ) m_splineType );
       setViewStructureChanged( );
       m_splineType = t;
    }
@@ -261,7 +332,7 @@ void PMPrism::setSweepType( PMPrism::SweepType t )
    if( m_sweepType != t )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPrism, PMSweepTypeID, ( int ) m_sweepType );
+         m_pMemento->addData( s_pMetaObject, PMSweepTypeID, ( int ) m_sweepType );
       setViewStructureChanged( );
       m_sweepType = t;
    }
@@ -272,7 +343,7 @@ void PMPrism::setSturm( bool s )
    if( m_sturm != s )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPrism, PMSturmID, m_sturm );
+         m_pMemento->addData( s_pMetaObject, PMSturmID, m_sturm );
       m_sturm = s;
    }
 }
@@ -282,7 +353,7 @@ void PMPrism::setOpen( bool o )
    if( m_open != o )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPrism, PMOpenID, m_open );
+         m_pMemento->addData( s_pMetaObject, PMOpenID, m_open );
       m_open = o;
    }
 }
@@ -292,7 +363,7 @@ void PMPrism::setHeight1( double h )
    if( m_height1 != h )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPrism, PMHeight1ID, m_height1 );
+         m_pMemento->addData( s_pMetaObject, PMHeight1ID, m_height1 );
       m_height1 = h;
       setViewStructureChanged( );
    }
@@ -303,7 +374,7 @@ void PMPrism::setHeight2( double h )
    if( m_height2 != h )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPrism, PMHeight2ID, m_height2 );
+         m_pMemento->addData( s_pMetaObject, PMHeight2ID, m_height2 );
       m_height2 = h;
       setViewStructureChanged( );
    }
@@ -315,7 +386,7 @@ void PMPrism::setPoints( const QValueList< QValueList<PMVector> >& points )
    {
       if( m_pMemento )
          ( ( PMPrismMemento* ) m_pMemento )->setPrismPoints( m_points );
-      
+
       setViewStructureChanged( );
       m_points = points;
    }
@@ -342,7 +413,7 @@ void PMPrism::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTPrism )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
@@ -372,7 +443,7 @@ void PMPrism::restoreMemento( PMMemento* s )
    }
    if( m->prismPointsSaved( ) )
       setPoints( m->prismPoints( ) );
-   
+
    Base::restoreMemento( s );
 }
 
@@ -381,7 +452,7 @@ void PMPrism::createViewStructure( )
 {
    if( s_sSteps == 0 )
       s_sSteps = c_defaultPrismSSteps;
-   
+
    // calculate number of points and lines of the view structure
    QValueList< QValueList<PMVector> >::ConstIterator spit = m_points.begin( );
    int np = 0;
@@ -404,17 +475,17 @@ void PMPrism::createViewStructure( )
       }
       np += snp;
    }
-   
+
    int nl = 0;
    nl = np * s_sSteps * 3;
    np *= s_sSteps * 2;
-      
+
    if( m_pViewStructure )
    {
       if( m_pViewStructure->points( ).size( ) != ( unsigned ) np )
          m_pViewStructure->points( ).resize( np );
       if( m_pViewStructure->lines( ).size( ) != ( unsigned ) nl )
-         m_pViewStructure->lines( ).resize( nl );      
+         m_pViewStructure->lines( ).resize( nl );
    }
    else
       m_pViewStructure = new PMViewStructure( np, nl );
@@ -423,7 +494,7 @@ void PMPrism::createViewStructure( )
    PMPointArray& points = m_pViewStructure->points( );
    int lb = 0;
    int pb = 0;
-   
+
    for( spit = m_points.begin( ); spit != m_points.end( ); ++spit )
    {
       QValueList<PMSplineSegment> segments;
@@ -431,7 +502,7 @@ void PMPrism::createViewStructure( )
 
       int ns = fullPoints.count( );
       int i, j;
-      
+
       switch( m_splineType )
       {
          case LinearSpline:
@@ -451,7 +522,7 @@ void PMPrism::createViewStructure( )
 
       // create the spline segments
       it1 = fullPoints.begin( );
-      it2 = it1; ++it2; 
+      it2 = it1; ++it2;
       it3 = it2; ++it3;
       it4 = it3; ++it4;
       PMSplineSegment s;
@@ -504,14 +575,14 @@ void PMPrism::createViewStructure( )
       lb += vp;
       for( i = 0; i < vp; i++ )
          lines[lb+i] = PMLine( pb + i, pb + vp + i );
-      lb += vp;         
-      
+      lb += vp;
+
       // calculate the points
       PMVector point2( 2 ), point3;
       QValueList<PMSplineSegment>::Iterator sit = segments.begin( );
       int pi = 0;
       double poffset = 1.0 / s_sSteps;
-      
+
       for( i = 0; i < ns; ++i, ++sit )
       {
          for( j = 0; j < s_sSteps; j++ )
@@ -532,7 +603,7 @@ void PMPrism::createViewStructure( )
                point3[1] = 1.0;
                point3[2] = point2[1];
                points[pb+pi] = PMPoint( point3 * m_height1 );
-               points[pb+pi+vp] = PMPoint( point3 * m_height2 );                  
+               points[pb+pi+vp] = PMPoint( point3 * m_height2 );
             }
             pi++;
          }
@@ -568,7 +639,7 @@ void PMPrism::controlPoints( PMControlPointList& list )
          it2 = ( *it1 ).begin( );
          PM2DControlPoint* firstPoint = 0;
          PM2DControlPoint* secondPoint = 0;
-         
+
          for( i2 = 0; it2 != ( *it1 ).end( ); ++it2, i2++ )
          {
             cp = new PM2DControlPoint( *it2, PM2DControlPoint::PM2DXZ, i2,
@@ -577,7 +648,7 @@ void PMPrism::controlPoints( PMControlPointList& list )
                firstPoint = cp;
             else if( i2 == 1 )
                secondPoint = cp;
-            
+
             cp->setThirdCoordinate( m_height2 );
             if( m_sweepType == ConicSweep )
                cp->setScale( m_height2 );
@@ -587,9 +658,9 @@ void PMPrism::controlPoints( PMControlPointList& list )
                firstPoint->setBasePoint( cp );
             if( ( m_splineType == CubicSpline ) && ( i2 == ( refb + 2 ) ) )
                cp->setBasePoint( secondPoint );
-            
+
             list.append( cp );
-            
+
             if( ( m_splineType != BezierSpline ) && ( i2 == refb ) )
                i2++;
          }
@@ -600,7 +671,7 @@ void PMPrism::controlPoints( PMControlPointList& list )
          PM2DControlPoint* firstPoint = 0;
          PM2DControlPoint* lastPoint = 0;
          PM2DControlPoint* startPoint = 0;
-         
+
          for( i2 = 0; it2 != ( *it1 ).end( ); ++it2, i2++ )
          {
             int i2mod4 = i2 % 4;
@@ -612,7 +683,7 @@ void PMPrism::controlPoints( PMControlPointList& list )
                lastPoint = cp;
             if( !startPoint )
                startPoint = cp;
-            
+
             cp->setThirdCoordinate( m_height2 );
             if( m_sweepType == ConicSweep )
                cp->setScale( m_height2 );
@@ -620,9 +691,9 @@ void PMPrism::controlPoints( PMControlPointList& list )
                cp->setBasePoint( firstPoint );
             if( ( i2mod4 == 0 ) && lastPoint )
                lastPoint->setBasePoint( cp );
-            
+
             list.append( cp );
-            
+
             if( i2mod4 == 2 )
                i2++;
          }
@@ -656,7 +727,7 @@ void PMPrism::controlPointsChanged( PMControlPointList& list )
       h2changed = true;
    }
    ++it;
-   
+
    for( ; it.current( ); ++it )
    {
       p1 = ( PM2DControlPoint* ) it.current( );
@@ -681,7 +752,7 @@ void PMPrism::controlPointsChanged( PMControlPointList& list )
          if( m_sweepType == ConicSweep )
             p1->setScale( m_height2 );
       }
-      
+
       ++pit;
       if( pit == ( *spit ).end( ) )
       {
@@ -696,16 +767,16 @@ void PMPrism::addObjectActions( const PMControlPointList& /*cp*/,
 {
    PMObjectAction* a;
 
-   a = new PMObjectAction( PMTPrism, PMSplitSegmentID,
-                           i18n( "Split Segment" ) );
+   a = new PMObjectAction( s_pMetaObject, PMSplitSegmentID,
+                           i18n( "Add Point" ) );
    actions.append( a );
-   
-   a = new PMObjectAction( PMTPrism, PMJoinSegmentsID,
-                           i18n( "Join Segments" ) );
+
+   a = new PMObjectAction( s_pMetaObject, PMJoinSegmentsID,
+                           i18n( "Remove Point" ) );
 
    bool enableJoin = false;
    QValueList< QValueList<PMVector> >::ConstIterator spit = m_points.begin( );
-   
+
    int minp = 4;
    switch( m_splineType )
    {
@@ -736,7 +807,7 @@ void PMPrism::objectActionCalled( const PMObjectAction* action,
                                   const QPtrList<PMVector>& cpViewPosition,
                                   const PMVector& clickPosition )
 {
-   if( action->objectType( ) == PMTPrism )
+   if( action->objectType( ) == s_pMetaObject )
    {
       switch( action->actionID( ) )
       {
@@ -795,7 +866,7 @@ void PMPrism::splitSegment( const PMControlPointList& /*cp*/,
                   skip = true;
                break;
          }
-         
+
          if( !skip )
          {
             if( first )
@@ -826,7 +897,7 @@ void PMPrism::splitSegment( const PMControlPointList& /*cp*/,
          ++it2;
       }
    }
-   
+
    // add a new segment
    QValueList< QValueList<PMVector> > newPoints = m_points;
    spit = newPoints.at( nsp );
@@ -920,7 +991,7 @@ void PMPrism::splitSegment( const PMControlPointList& /*cp*/,
       newSubPoints.insert( it, mid - grad );
       newSubPoints.insert( it, mid );
       newSubPoints.insert( it, mid + grad );
-      
+
       ++it;
       if( it == newSubPoints.end( ) )
          end = *newSubPoints.begin( );
@@ -989,14 +1060,14 @@ void PMPrism::joinSegments( const PMControlPointList& /*cp*/,
          }
          if( nump < minp )
             skip = true;
-            
+
          if( !skip )
          {
             dist[0] = (**it1)[0];
             dist[1] = (**it1)[1];
             dist -= clickPosition;
             abs = dist.abs( );
-               
+
             if( ( minabs > abs ) || ( ns < 0 ) )
             {
                minabs = abs;
@@ -1013,13 +1084,13 @@ void PMPrism::joinSegments( const PMControlPointList& /*cp*/,
       kdError( PMArea ) << "Not enough points in PMPrism::joinSegments\n";
       return;
    }
-   
+
    // remove the segment
    QValueList< QValueList<PMVector> > newPoints = m_points;
    spit = newPoints.at( nsp );
    QValueList<PMVector> newSubPoints = *spit;
    QValueList<PMVector>::Iterator it;
-   
+
    if( m_splineType != BezierSpline )
    {
       it = newSubPoints.at( ( unsigned ) ns );
@@ -1070,7 +1141,7 @@ QValueList<PMVector> PMPrism::expandedPoints( const QValueList<PMVector>& p ) co
    // add the missing points
    int refa = 0, refb = p.count( );
    QValueList<PMVector> result = p;
-      
+
    switch( m_splineType )
    {
       case LinearSpline:
