@@ -36,7 +36,7 @@ const PMVector c_defaultOffset = PMVector( 0.0, 0.0 );
 int PMText::s_parameterKey = 0;
 int PMText::s_steps = c_defaultTextSteps;
 
-PMDefinePropertyClass( PMText, PMProperty );
+PMDefinePropertyClass( PMText, PMTextProperty );
 
 PMMetaObject* PMText::s_pMetaObject = 0;
 PMObject* createNewText( PMPart* part )
@@ -96,13 +96,13 @@ PMMetaObject* PMText::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Text", Base::metaObject( ),
                                         createNewText );
       s_pMetaObject->addProperty(
-         new PMProperty( "font", &PMText::setFont, &PMText::font ) );
+         new PMTextProperty( "font", &PMText::setFont, &PMText::font ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "text", &PMText::setText, &PMText::text ) );
+         new PMTextProperty( "text", &PMText::setText, &PMText::text ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "thickness", &PMText::setThickness, &PMText::thickness ) );
+         new PMTextProperty( "thickness", &PMText::setThickness, &PMText::thickness ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "offset", &PMText::setOffset, &PMText::offset ) );
+         new PMTextProperty( "offset", &PMText::setOffset, &PMText::offset ) );
    }
    return s_pMetaObject;
 }
