@@ -40,7 +40,7 @@ int PMTorus::s_vStep = c_defaultTorusVSteps;
 int PMTorus::s_uStep = c_defaultTorusUSteps;
 int PMTorus::s_parameterKey = 0;
 
-PMDefinePropertyClass( PMTorus, PMProperty );
+PMDefinePropertyClass( PMTorus, PMTorusProperty );
 
 PMMetaObject* PMTorus::s_pMetaObject = 0;
 PMObject* createNewTorus( PMPart* part )
@@ -81,13 +81,13 @@ PMMetaObject* PMTorus::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Torus", Base::metaObject( ),
                                         createNewTorus );
       s_pMetaObject->addProperty(
-         new PMProperty( "minorRadius", &PMTorus::setMinorRadius,
+         new PMTorusProperty( "minorRadius", &PMTorus::setMinorRadius,
                          &PMTorus::minorRadius ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "majorRadius", &PMTorus::setMajorRadius,
+         new PMTorusProperty( "majorRadius", &PMTorus::setMajorRadius,
                          &PMTorus::majorRadius ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "sturm", &PMTorus::setSturm, &PMTorus::sturm ) );
+         new PMTorusProperty( "sturm", &PMTorus::setSturm, &PMTorus::sturm ) );
    }
    return s_pMetaObject;
 }
