@@ -88,7 +88,7 @@ void PMViewLayoutEntry::loadData( QDomElement& e )
    // Read the view type
    s = e.tagName( );
    m_viewType = s;
-   
+
    // Read dock position
    s = e.attribute( "position", "Right" );
    if( s == "Right" ) m_dockPosition = PMDockWidget::DockRight;
@@ -100,7 +100,7 @@ void PMViewLayoutEntry::loadData( QDomElement& e )
    s = e.attribute( "columnWidth", "33" );
    m_columnWidth = s.toInt( &ok );
    if( !ok ) m_columnWidth = 33;
-   
+
    // and view height
    s = e.attribute( "height", "50" );
    m_height = s.toInt( &ok );
@@ -204,7 +204,7 @@ const QString PMViewLayoutEntry::viewTypeAsString( )
       else
          return f->description( );
    }
-   
+
    kdError( PMArea ) << i18n( "Unknown view type." )<< endl;
    return i18n( "Unknown" );
 }
@@ -266,7 +266,7 @@ void PMViewLayout::loadData( QDomElement& e )
       if( m.isElement( ) )
       {
          QDomElement me = m.toElement( );
-         PMViewLayoutEntry vle; 
+         PMViewLayoutEntry vle;
 
          vle.loadData( me );
          m_entries.append( vle );
@@ -329,6 +329,7 @@ void PMViewLayout::displayLayout( PMShell* shell )
                            QPoint( ( *it ).floatingPositionX( ),
                                    ( *it ).floatingPositionY( ) ) );
          dock->resize( ( *it ).floatingWidth( ), ( *it ).floatingHeight( ) );
+			dock->show( );
       }
       else
       {
@@ -347,7 +348,7 @@ void PMViewLayout::displayLayout( PMShell* shell )
                case PMDockWidget::DockRight:
                   dock->manualDock( lastWidget, ( *it ).dockPosition( ),
                                     lastWidth * 100 / width );
-                  
+
                   width -= lastWidth;
                   if( width < 1 ) width = 1;
                   height = 100;
@@ -368,7 +369,7 @@ void PMViewLayout::displayLayout( PMShell* shell )
                   break;
             }
          }
-         
+
          switch( ( *it ).dockPosition( ) )
          {
             case PMDockWidget::DockRight:
@@ -411,7 +412,7 @@ void PMViewLayout::normalize( )
          sumColWidth += ( *it ).columnWidth( );
    if( sumColWidth == 0 )
       sumColWidth = 1;
-   
+
    for( it = m_entries.begin( ); it != m_entries.end( ); ++it )
    {
       if( ( *it ).dockPosition( ) == PMDockWidget::DockRight )
@@ -443,14 +444,14 @@ void PMViewLayout::normalize( )
 PMViewLayout PMViewLayout::extractViewLayout( PMShell* shell )
 {
    PMViewLayout layout;
-   
+
    QValueList< QValueList< PMViewLayoutEntry > > cols;
    cols.append( QValueList< PMViewLayoutEntry >( ) );
 
    // find docked widgets
    recursiveExtractColumns( cols, cols.begin( ), 100,
                             shell->centralWidget( ) );
-   
+
    QValueListIterator< QValueList< PMViewLayoutEntry > > cit;
    QValueListIterator< PMViewLayoutEntry > eit;
 
@@ -462,10 +463,10 @@ PMViewLayout PMViewLayout::extractViewLayout( PMShell* shell )
    QPtrList<PMDockWidget> list;
    shell->manager( )->findFloatingWidgets( list );
    QPtrListIterator<PMDockWidget> it( list );
-   
+
    for( ; it.current( ); ++it )
    {
-      kdDebug( PMArea ) << it.current( ) << endl;
+      kdDebug( PMArea ) << it.current( ) << " " << it.current( )->isVisible( ) << endl;
       QWidget* w = it.current( )->getWidget( );
       if( w )
       {
@@ -477,22 +478,22 @@ PMViewLayout PMViewLayout::extractViewLayout( PMShell* shell )
          e.setFloatingPositionY( p.y( ) );
          e.setFloatingWidth( it.current( )->width( ) );
          e.setFloatingHeight( it.current( )->height( ) );
-         
+
          if( w->inherits( "PMViewBase" ) )
          {
             PMViewBase* v = ( PMViewBase* ) w;
-            ( *eit ).setViewType( v->viewType( ) );
+            e.setViewType( v->viewType( ) );
             PMViewOptions* vo =
                PMViewFactory::theFactory( )->newOptionsInstance( v->viewType( ) );
             if( vo )
             {
                v->saveViewConfig( vo );
-               ( *eit ).setCustomOptions( vo );
+               e.setCustomOptions( vo );
             }
          }
          else
             invalid = true;
-         
+
          if( !invalid )
             layout.addEntry( e );
       }
@@ -531,8 +532,8 @@ void PMViewLayout::recursiveExtractColumns(
                QValueList< QValueList< PMViewLayoutEntry > >::iterator cit1
                   = cols.insert( cit, QValueList< PMViewLayoutEntry >( ) );
 
-               recursiveExtractColumns( cols, cit1, w1, sp->getFirst( ) ); 
-               recursiveExtractColumns( cols, cit, w2, sp->getLast( ) ); 
+               recursiveExtractColumns( cols, cit1, w1, sp->getFirst( ) );
+               recursiveExtractColumns( cols, cit, w2, sp->getLast( ) );
             }
          }
          if( colStart )
@@ -573,14 +574,14 @@ void PMViewLayout::recursiveExtractOneColumn(
 
             ( *eit ).setHeight( h1 );
             ( *eit ).setDockPosition( PMDockWidget::DockRight );
-            
+
             PMViewLayoutEntry e;
             e.setHeight( h2 );
             e.setDockPosition( PMDockWidget::DockBottom );
             QValueList< PMViewLayoutEntry >::iterator eit1 = eit;
             eit1 = entries.insert( ++eit1, e );
 
-            recursiveExtractOneColumn( entries, eit, h1, sp->getFirst( ) ); 
+            recursiveExtractOneColumn( entries, eit, h1, sp->getFirst( ) );
             recursiveExtractOneColumn( entries, eit1, h2, sp->getLast( ) );
          }
          else if( w->inherits( "PMDockTabGroup" ) )
@@ -599,7 +600,7 @@ void PMViewLayout::recursiveExtractOneColumn(
                   PMViewLayoutEntry e;
                   e.setHeight( height );
                   e.setDockPosition( PMDockWidget::DockCenter );
-                  
+
                   eit++;
                   eit = entries.insert( eit, e );
                   recursiveExtractOneColumn( entries, eit, height, tw );
@@ -609,8 +610,7 @@ void PMViewLayout::recursiveExtractOneColumn(
          else
          {
             // a kpovmodeler view???
-            QString cn( w->className( ) );
-            if( cn.left( 2 ) == "PM" && cn.right( 4 ) == "View" )
+				if( w->inherits( "PMViewBase" ) )
             {
                PMViewBase* v = ( PMViewBase* ) w;
                ( *eit ).setViewType( v->viewType( ) );
@@ -649,10 +649,10 @@ QStringList PMViewLayoutManager::availableLayouts( )
 {
    QStringList result;
    QValueListIterator<PMViewLayout> it;
-   
+
    for( it = m_layouts.begin( ); it != m_layouts.end( ); ++it )
       result.push_back( ( *it ).name( ) );
-   
+
    return result;
 }
 
@@ -662,7 +662,7 @@ void PMViewLayoutManager::loadData( )
       m_layouts.clear( );
 
    m_layoutsLoaded = true;
-   
+
    QString fileName = locate( "data", "kpovmodeler/viewlayouts.xml" );
    if( fileName.isEmpty( ) )
    {
@@ -700,13 +700,13 @@ void PMViewLayoutManager::loadData( )
       p.setDockPosition( PMDockWidget::DockBottom );
       p.setHeight( 50 );
       a.addEntry( p );
-      
+
       m_layouts.append( a );
       m_defaultLayout = a.name( );
-      
+
       return;
    }
-   
+
    QFile file( fileName );
    if( !file.open( IO_ReadOnly ) )
    {
@@ -758,7 +758,7 @@ void PMViewLayoutManager::saveData( )
    e.setAttribute( "default", m_defaultLayout );
 
    QValueListIterator<PMViewLayout> it;
-   
+
    for( it = m_layouts.begin( ); it != m_layouts.end( ); ++it )
    {
       QDomElement l;
@@ -772,7 +772,7 @@ void PMViewLayoutManager::saveData( )
    str.setEncoding( QTextStream::UnicodeUTF8 );
    str << doc.toString( );
    file.close( );
-}   
+}
 
 PMViewLayoutManager* PMViewLayoutManager::theManager( )
 {
@@ -794,7 +794,7 @@ void PMViewLayoutManager::displayLayout( const QString& name, PMShell* shell )
 
          if( shell->centralWidget( ) )
             shell->manager( )->findChildDockWidget( shell->centralWidget( ), lst );
-         while( lst.first( ) ) 
+         while( lst.first( ) )
          {
             ( ( PMDockWidget* )lst.first( ) )->undock( );
             ( ( PMDockWidget* )lst.first( ) )->close( );
@@ -803,7 +803,7 @@ void PMViewLayoutManager::displayLayout( const QString& name, PMShell* shell )
 
          QPtrList<PMDockWidget> flist;
          shell->manager( )->findFloatingWidgets( flist );
-         while( flist.first( ) ) 
+         while( flist.first( ) )
          {
             flist.first( )->undock( );
             flist.first( )->close( );
@@ -872,8 +872,8 @@ PMSaveViewLayoutDialog::PMSaveViewLayoutDialog( PMShell* parent,
                      KDialogBase::Ok | KDialogBase::Cancel )
 {
    m_pShell = parent;
-   
-   setButtonOKText( i18n( "Save" ) );
+
+   setButtonOK( KStdGuiItem::save() );
    enableButtonOK( false );
 
    QWidget* w = new QWidget( this );
@@ -881,19 +881,19 @@ PMSaveViewLayoutDialog::PMSaveViewLayoutDialog( PMShell* parent,
 
    QLabel* l = new QLabel( i18n( "Enter view layout name:" ), w );
    vl->addWidget( l );
-   
+
    m_pLayoutName = new QLineEdit( w );
    vl->addWidget( m_pLayoutName );
    connect( m_pLayoutName, SIGNAL( textChanged( const QString& ) ),
             SLOT( slotNameChanged( const QString& ) ) );
-   
+
    QListBox* lb = new QListBox( w );
    vl->addWidget( lb );
    connect( lb, SIGNAL( highlighted( const QString& ) ),
             SLOT( slotNameSelected( const QString& ) ) );
    lb->insertStringList( PMViewLayoutManager::theManager( )
                          ->availableLayouts( ) );
-   
+
    setMainWidget( w );
    setInitialSize( QSize( 300, 200 ) );
 }
@@ -905,20 +905,20 @@ PMSaveViewLayoutDialog::~PMSaveViewLayoutDialog( )
 void PMSaveViewLayoutDialog::slotOk( )
 {
    QString name = m_pLayoutName->text( );
-   
+
    PMViewLayoutManager* m = PMViewLayoutManager::theManager( );
    PMViewLayout* layout = m->findLayout( name );
 
    PMViewLayout newLayout = PMViewLayout::extractViewLayout( m_pShell );
    newLayout.setName( name );
-   
+
    if( layout )
       *layout = newLayout;
    else
       m->addLayout( newLayout );
 
    m->saveData( );
-   
+
    KDialogBase::slotOk( );
 }
 
