@@ -60,9 +60,9 @@
 
 #define kApp KApplication::kApplication()
 
-KSnapshot::KSnapshot(QWidget *parent, const char *name)
-  : DCOPObject("interface"),
-    KDialogBase(parent, name, true, QString::null, Help|User1, User1,
+KSnapshot::KSnapshot(QWidget *parent, const char *name, bool grabCurrent)
+  : DCOPObject("interface"), 
+    KDialogBase(parent, name, true, QString::null, Help|User1, User1, 
     true, KStdGuiItem::quit() )
 {
     grabber = new QWidget( 0, 0, WStyle_Customize | WX11BypassWM );
@@ -88,7 +88,15 @@ KSnapshot::KSnapshot(QWidget *parent, const char *name)
 
     grabber->show();
     grabber->grabMouse( waitCursor );
-    snapshot = QPixmap::grabWindow( qt_xrootwin() );
+
+    if ( !grabCurrent )
+	snapshot = QPixmap::grabWindow( qt_xrootwin() );
+    else {
+	mainWidget->setMode( WindowUnderCursor );
+	mainWidget->setIncludeDecorations( true );
+	performGrab();
+    }
+
     updatePreview();
     grabber->releaseMouse();
     grabber->hide();
@@ -125,6 +133,7 @@ KSnapshot::KSnapshot(QWidget *parent, const char *name)
 		   CTRL+Key_A, this, SLOT(slotSaveAs()));
     accel->insert(KStdAccel::Print, this, SLOT(slotPrint()));
     accel->insert(KStdAccel::New, this, SLOT(slotGrab()));
+    accel->insert(KStdAccel::Copy, this, SLOT(slotCopy()));
 
     accel->insert( "Quit2", Key_Q, this, SLOT(slotSave()));
     accel->insert( "Save2", Key_S, this, SLOT(slotSaveAs()));
@@ -152,7 +161,7 @@ bool KSnapshot::save( const KURL& url )
     if ( KIO::NetAccess::exists( url, false, this ) ) {
         const QString title = i18n( "File Exists" );
         const QString text = i18n( "<qt>Do you really want to overwrite <b>%1</b>?</qt>" ).arg(url.prettyURL());
-        if (KMessageBox::Yes != KMessageBox::warningYesNoCancel( this, text, title ) )
+        if (KMessageBox::Yes != KMessageBox::warningYesNoCancel( this, text, title ) ) 
         {
             return false;
         }
@@ -244,20 +253,19 @@ void KSnapshot::slotDragSnapshot()
 void KSnapshot::slotGrab()
 {
     hide();
-    if ( mainWidget->mode() == Region )
-    {
-        rgnGrab = new RegionGrabber();
-        connect( rgnGrab, SIGNAL( regionGrabbed( const QPixmap & ) ),
-            SLOT( slotRegionGrabbed( const QPixmap & ) ) );
-    }
-    else
-    {
-        if ( mainWidget->delay() )
-            grabTimer.start( mainWidget->delay() * 1000, true );
-        else {
-            grabber->show();
-            grabber->grabMouse( crossCursor );
-        }
+
+    if ( mainWidget->delay() )
+	grabTimer.start( mainWidget->delay() * 1000, true );
+    else {
+	if ( mainWidget->mode() == Region ) {
+	    rgnGrab = new RegionGrabber();
+	    connect( rgnGrab, SIGNAL( regionGrabbed( const QPixmap & ) ),
+		     SLOT( slotRegionGrabbed( const QPixmap & ) ) );
+	}
+	else {
+	    grabber->show();
+	    grabber->grabMouse( crossCursor );
+	}
     }
 }
 
@@ -369,7 +377,7 @@ void KSnapshot::autoincFilename()
     QRegExp numSearch("[0-9]+");
 
     // Does it have a number?
-    int start= numSearch.search(name, 0);
+    int start = numSearch.searchRev(name);
     if (start != -1) {
         // It has a number, increment it
         int len = numSearch.matchedLength();
@@ -389,7 +397,7 @@ void KSnapshot::autoincFilename()
         }
     }
 
-    //Rebuild the path
+    //Rebuild the path 
     KURL newURL = filename;
     newURL.setFileName( name );
     setURL( newURL.url() );
@@ -413,7 +421,14 @@ void KSnapshot::updatePreview()
 
 void KSnapshot::grabTimerDone()
 {
-    performGrab();
+    if ( mainWidget->mode() == Region ) {
+        rgnGrab = new RegionGrabber();
+        connect( rgnGrab, SIGNAL( regionGrabbed( const QPixmap & ) ),
+            SLOT( slotRegionGrabbed( const QPixmap & ) ) );
+    }
+    else {
+	performGrab();
+    }
     KNotifyClient::beep(i18n("The screen has been successfully grabbed."));
 }
 
@@ -519,7 +534,7 @@ void KSnapshot::performGrab()
 
 		//Create the bounding box.
 		QRegion bbox(0, 0, snapshot.width(), snapshot.height());
-
+                
                 if( border > 0 ) {
                     contents.translate( border, border );
                     contents += QRegion( 0, 0, border, h );
@@ -527,7 +542,7 @@ void KSnapshot::performGrab()
                     contents += QRegion( 0, h - border, w, border );
                     contents += QRegion( w - border, 0, border, h );
                 }
-
+                
 		//Get the masked away area.
 		QRegion maskedAway = bbox - contents;
 		QMemArray<QRect> maskedAwayRects = maskedAway.rects();
