@@ -34,6 +34,7 @@
 #include "SVGSVGElementImpl.h"
 #include "SVGImageElementImpl.h"
 #include "SVGScriptElementImpl.h"
+#include "SVGTitleElementImpl.h"
 #include "SVGAnimationElementImpl.h"
 
 #include "KSVGReader.h"
@@ -167,7 +168,14 @@ float SVGDocumentImpl::screenPixelsPerMillimeterY() const
 
 DOM::DOMString SVGDocumentImpl::title() const
 {
-	return m_title;
+	DOM::Node n;
+	for(n = rootElement()->firstChild(); !n.isNull(); n = n.nextSibling())
+	{
+		SVGElementImpl *elem = getElementFromHandle(n.handle());
+		if(dynamic_cast<SVGTitleElementImpl *>(elem))
+			return elem->collectText();
+	}
+	return "";
 }
 
 DOM::DOMString SVGDocumentImpl::referrer() const
@@ -177,12 +185,18 @@ DOM::DOMString SVGDocumentImpl::referrer() const
 
 DOM::DOMString SVGDocumentImpl::domain() const
 {
-	return m_domain;
+	return m_baseURL.host();
 }
 
 DOM::DOMString SVGDocumentImpl::URL() const
 {
-	return m_URL;
+	return m_baseURL.prettyURL();
+}
+
+void SVGDocumentImpl::setReferrer(const DOM::DOMString &referrer)
+{
+	// TODO : better may be to request for referrer instead of storing it
+	m_referrer = referrer;
 }
 
 void SVGDocumentImpl::setRootElement(SVGSVGElementImpl *elem)
@@ -519,7 +533,11 @@ SVGElementImpl *SVGDocumentImpl::recursiveSearch(DOM::Node start, const DOM::DOM
 }
 
 /*
-@begin	SVGDocumentImpl::s_hashTable 5
+@begin	SVGDocumentImpl::s_hashTable 9
+ title				SVGDocumentImpl::Title				DontDelete|ReadOnly
+ referrer			SVGDocumentImpl::Referrer			DontDelete|ReadOnly
+ domain				SVGDocumentImpl::Domain				DontDelete|ReadOnly
+ URL				SVGDocumentImpl::Url				DontDelete|ReadOnly
  doctype			SVGDocumentImpl::DocType			DontDelete|ReadOnly
  implementation		SVGDocumentImpl::Implementation		DontDelete|ReadOnly
  rootElement		SVGDocumentImpl::RootElement		DontDelete|ReadOnly
@@ -541,6 +559,14 @@ Value SVGDocumentImpl::getValueProperty(ExecState *exec, int token) const
 {
 	switch(token)
 	{
+		case Title:
+			return String(title().string());
+		case Referrer:
+			return String(referrer().string());
+		case Domain:
+			return String(domain().string());
+		case Url:
+			return String(URL().string());
 		case DocType:
 			return getDOMNode(exec, doctype());
 		case Implementation:
