@@ -838,6 +838,12 @@ void LibartLine::init(const SVGMatrixImpl *screenCTM)
 	vec[1].x = m_line->x2()->baseVal()->value();
 	vec[1].y = m_line->y2()->baseVal()->value();
 
+	// A subpath consisting of a moveto and lineto to the same exact location or a subpath consisting of a moveto
+	// and a closepath will be stroked only if the 'stroke-linecap' property is set to "round", producing a circle
+	// centered at the given point.
+	if(vec[1].x == vec[0].x && vec[1].y == vec[0].y && m_line->getCapStyle() == PATH_STROKE_CAP_ROUND)
+		vec[1].x += .5;
+
 	vec[2].code = ART_END;
 
 	if(m_context == NORMAL)
@@ -911,6 +917,13 @@ void LibartPolyline::init(const SVGMatrixImpl *screenCTM)
 		polyline[index].y = m_poly->points()->getItem(index)->y();
 	}
 
+	// A subpath consisting of a moveto and lineto to the same exact location or a subpath consisting of a moveto
+	// and a closepath will be stroked only if the 'stroke-linecap' property is set to "round", producing a circle
+	// centered at the given point.
+	if(numberOfPoints == 2 && polyline[1].x == polyline[0].x && polyline[1].y == polyline[0].y && m_poly->getCapStyle() == PATH_STROKE_CAP_ROUND)
+		polyline[1].x += .5;
+
+
 	if(m_poly->isFilled()) // if the polyline must be filled, inform libart that it should not be closed.
 	{
 		polyline[index].code = (ArtPathcode) ART_END2;
@@ -1093,6 +1106,12 @@ void LibartPath::init(const SVGMatrixImpl *screenCTM)
 
 		m_array[index].code = ART_END;
 
+		// A subpath consisting of a moveto and lineto to the same exact location or a subpath consisting of a moveto
+		// and a closepath will be stroked only if the 'stroke-linecap' property is set to "round", producing a circle
+		// centered at the given point.
+		if(index == 2 && m_array[1].code == ART_LINETO && m_array[1].x3 == m_array[0].x3 && m_array[1].y3 == m_array[0].y3 && m_path->getCapStyle() == PATH_STROKE_CAP_ROUND)
+			m_array[1].x3 += .5;
+
 		// There are pure-moveto paths which reference paint servers *bah*
 		// Do NOT render them
 		bool render = false;
