@@ -14,7 +14,7 @@
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
- */
+*/
 
 #include <qtooltip.h>
 
@@ -34,7 +34,7 @@ KURLWidget::KURLWidget(const QString& text, QWidget *parent, const char *name)
 void KURLWidget::run()
 {
     KURL ku( url() );
-    if ( !ku.isMalformed() ) {
+    if ( ku.isValid() ) {
 	(void) new KRun( ku );
     }
 }
