@@ -39,7 +39,6 @@
 #include <kstatusbar.h>
 #include <kdebug.h>
 #include <kedittoolbar.h>
-#include <kkeydialog.h>
 #include <kstdaccel.h>
 #include <kprogress.h>
 
@@ -262,9 +261,6 @@ void KView::imageSizeChanged( const QSize & /*size*/ )
 {
 	QSize size = m_pCanvas->imageSize();
 	statusBar()->changeItem( QString( "%1 x %2" ).arg( size.width() ).arg( size.height() ), STATUSBAR_SIZE_ID );
-	if( m_bImageSizeChangedBlocked )
-		return;
-
 	handleResize();
 }
 
@@ -385,11 +381,6 @@ void KView::slotPreferences()
 	dlg->show();
 }
 
-void KView::slotKeyBindings()
-{
-	KKeyDialog::configure( actionCollection() );
-}
-
 void KView::slotConfigureToolbars()
 {
 	saveMainWindowSettings( KGlobal::config(), "MainWindow" );
@@ -536,29 +527,55 @@ void KView::setupActions( QObject * partobject )
 	if( m_paShowStatusBar )
 		connect( m_paShowStatusBar, SIGNAL( toggled( bool ) ), SLOT( statusbarToggled( bool ) ) );
 	m_paShowFullScreen = KStdAction::fullScreen( 0, 0, actionCollection(), this );
-        connect( m_paShowFullScreen, SIGNAL( toggled( bool )), this, SLOT( slotUpdateFullScreen( bool )));
+	connect( m_paShowFullScreen, SIGNAL( toggled( bool )), this, SLOT( slotUpdateFullScreen( bool )));
 	KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );
-	KStdAction::keyBindings( this, SLOT( slotKeyBindings() ), actionCollection() );
+	KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()), 
+actionCollection());
 	KStdAction::configureToolbars( this, SLOT( slotConfigureToolbars() ), actionCollection() );
 }
 
 void KView::handleResize()
 {
+	if( m_bImageSizeChangedBlocked )
+		return;
+	m_bImageSizeChangedBlocked = true;
+	setUpdatesEnabled( false );
 	switch( m_nResizeMode )
 	{
 		case ResizeWindow:
-			setUpdatesEnabled( false );
 			fitWindowToImage();
-			//fitWindowToImage();
 			fitWindowToImage();
-			setUpdatesEnabled( true );
 			break;
 		case ResizeImage:
-			m_bImageSizeChangedBlocked = true;
 			m_pCanvas->boundImageTo( m_pViewer->widget()->size() );
-			m_bImageSizeChangedBlocked = false;
+			break;
+		case BestFit:
+			QSize imageSize = m_pCanvas->imageSize();
+			if( imageSize.isEmpty() )
+				return;
+
+			// Compare the image size and the maximum available space in the
+			// display canvas i.e. will the image fit without resizing ?
+			QSize maxCanvas = maxCanvasSize();
+			if( ( maxCanvas.height() >= imageSize.height() )
+					&& ( maxCanvas.width() >= imageSize.width() ) )
+			{
+				//Image can be displayed at full size
+				m_pCanvas->setZoom( 1.0 );
+			}
+			else
+			{
+				// Image is too big for the available canvas.
+				m_pCanvas->boundImageTo( maxCanvas );
+			}
+			// We assume the displayed image size has changed and we must resize
+			// the window around it (using the code for ResizeWindow).
+			fitWindowToImage();
+			fitWindowToImage();
 			break;
 	}
+	setUpdatesEnabled( true );
+	m_bImageSizeChangedBlocked = false;
 }
 
 void KView::fitWindowToImage()
