@@ -12,14 +12,14 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
-// $Id: kimagecanvas.cpp,v 1.22.2.4 2003/07/17 15:39:33 mueller Exp $
 
 #include "kimagecanvas.h"
 #include "kimageholder.h"
 #include "version.h"
+#include "config/defaults.h"
 
 #include <qcolor.h>
 #include <qimage.h>
@@ -29,8 +29,9 @@
 
 #include <kpixmap.h>
 #include <kdebug.h>
-#include <kdebugclasses.h>
 #include <kgenericfactory.h>
+#include <ksettings/dispatcher.h>
+#include <kconfig.h>
 
 #define KIMAGECANVAS_WIPESIZE 5
 
@@ -38,7 +39,9 @@ const int MOUSECURSORHIDETIME = 3000;
 
 //extern bool qt_use_xrender;
 
-K_EXPORT_COMPONENT_FACTORY( libkviewcanvas, KGenericFactory<KImageCanvas> )
+typedef KGenericFactory<KImageCanvas> KImageCanvasFactory;
+K_EXPORT_COMPONENT_FACTORY( libkviewcanvas,
+		KImageCanvasFactory( "kviewcanvas" ) )
 
 KImageCanvas::KImageCanvas( QWidget * parent, const char * name, const QStringList & )
 	: QScrollView( parent, name, WResizeNoErase | WStaticContents )
@@ -48,17 +51,16 @@ KImageCanvas::KImageCanvas( QWidget * parent, const char * name, const QStringLi
 	, m_imageTransformed( 0 )
 	, m_pixmap( 0 )
 	, m_pTimer( new QTimer( this, "KImageCanvas/Timer" ) )
-	, m_maxsize( 0, 0 )
-	, m_minsize( 0, 0 )
+	, m_maxsize( Defaults::maxSize )
+	, m_minsize( Defaults::minSize )
 	, m_currentsize( 0, 0 )
 	, m_zoom( 1.0 )
-	, m_fastscale( false )
-	, m_keepaspectratio( true )
+	, m_fastscale( ! Defaults::smoothScaling )
+	, m_keepaspectratio( Defaults::keepAspectRatio )
 	, m_bImageChanged( false )
 	, m_bSizeChanged( false )
-	, m_iLinesToIgnore( 0 )
 	, m_bNeedNewPixmap( false )
-	, m_bCentered( true )
+	, m_bCentered( Defaults::centerImage )
 	, m_bImageUpdateScheduled( false )
 	, m_bNewImage( false )
 	, m_iBlendTimerId( 0 )
@@ -67,12 +69,16 @@ KImageCanvas::KImageCanvas( QWidget * parent, const char * name, const QStringLi
 	setFrameStyle( QFrame::NoFrame );
 	setResizePolicy( QScrollView::Manual );
 	setMinimumSize( 0, 0 );
+	setBgColor( Defaults::bgColor );
 
 	connect( this, SIGNAL( imageChanged() ), this, SLOT( slotImageChanged() ) );
 	connect( m_pTimer, SIGNAL( timeout() ), this, SLOT( hideCursor() ) );
 
-	setFocusPolicy( QWidget::StrongFocus );
+	KSettings::Dispatcher::self()->registerInstance(
+			KImageCanvasFactory::instance(), this,
+			SLOT( loadSettings() ) );
 
+	viewport()->setFocusProxy( this );
 	clear();
 
 	QWidget::setMouseTracking( true );
@@ -80,6 +86,8 @@ KImageCanvas::KImageCanvas( QWidget * parent, const char * name, const QStringLi
 	m_cursor.setShape( Qt::CrossCursor );
 	viewport()->setCursor( m_cursor );
 	m_pTimer->start( MOUSECURSORHIDETIME, true );
+
+	loadSettings();
 }
 
 KImageCanvas::~KImageCanvas()
@@ -114,7 +122,7 @@ int KImageCanvas::imageDepth() const
 
 QSize KImageCanvas::imageSize() const
 {
-	kdDebug( 4620 ) << k_funcinfo << endl;
+	//kdDebug( 4620 ) << k_funcinfo << endl;
 	if( ! m_image )
 		return QSize( 0, 0 );
 
@@ -156,9 +164,8 @@ void KImageCanvas::setCentered( bool centered )
 	}
 }
 
-void KImageCanvas::setImage( const QImage & newimage, double zoom )
+void KImageCanvas::setImage( const QImage & newimage )
 {
-	kdDebug( 4620 ) << k_funcinfo << zoom << endl;
 	bool emitHasImage = m_image ? false : true;
 	m_matrix.reset();
 	matrixChanged();
@@ -168,11 +175,6 @@ void KImageCanvas::setImage( const QImage & newimage, double zoom )
 	// don't emit the signal here - call the slot directly
 	slotImageChanged();
 
-	if( zoom != m_zoom )
-	{
-		m_zoom = zoom;
-		emit zoomChanged( m_zoom );
-	}
 	sizeFromZoom( m_zoom );
 	updateImage();
 	if( emitHasImage && m_image )
@@ -212,9 +214,17 @@ void KImageCanvas::setZoom( double zoom )
 	}
 }
 
+void KImageCanvas::boundImageTo( const QSize & size )
+{
+	bool keepAspectRatio = m_keepaspectratio;
+	m_keepaspectratio = true;
+	resizeImage( size );
+	m_keepaspectratio = keepAspectRatio;
+}
+
 void KImageCanvas::setMaximumImageSize( const QSize & maxsize )
 {
-	kdDebug( 4620 ) << k_funcinfo << endl;
+	kdDebug( 4620 ) << k_funcinfo << maxsize << endl;
 	if( ( ! m_minsize.isEmpty() ) &&
 			( maxsize.width() < m_minsize.width() || maxsize.height() < m_minsize.height() ) )
 	{
@@ -229,7 +239,7 @@ void KImageCanvas::setMaximumImageSize( const QSize & maxsize )
 
 void KImageCanvas::setMinimumImageSize( const QSize & minsize )
 {
-	kdDebug( 4620 ) << k_funcinfo << endl;
+	kdDebug( 4620 ) << k_funcinfo << minsize << endl;
 	if( ( ! m_maxsize.isEmpty() ) &&
 			( minsize.width() > m_maxsize.width() || minsize.height() > m_maxsize.height() ) )
 	{
@@ -286,6 +296,11 @@ void KImageCanvas::setKeepAspectRatio( bool aspect )
 	m_keepaspectratio = aspect;
 }
 
+unsigned int KImageCanvas::numOfBlendEffects() const
+{
+	return Defaults::numOfBlendEffects;
+}
+
 QString KImageCanvas::blendEffectDescription( unsigned int idx ) const
 {
 	kdDebug( 4620 ) << k_funcinfo << endl;
@@ -293,17 +308,17 @@ QString KImageCanvas::blendEffectDescription( unsigned int idx ) const
 	{
 		case NoBlending:
 			kdWarning( 4620 ) << k_funcinfo << " shouldn't be called with an index of 0 - That's always not really defined\n";
-			return i18n( "No Blending" );
+			return i18n( Defaults::blendEffectDescription[ 0 ] );
 		case AlphaBlend:
-			return i18n( "Alpha Blend" );
+			return i18n( Defaults::blendEffectDescription[ 5 ] );
 		case WipeFromLeft:
-			return i18n( "Wipe From Left" );
+			return i18n( Defaults::blendEffectDescription[ 1 ] );
 		case WipeFromRight:
-			return i18n( "Wipe From Right" );
+			return i18n( Defaults::blendEffectDescription[ 2 ] );
 		case WipeFromTop:
-			return i18n( "Wipe From Top" );
+			return i18n( Defaults::blendEffectDescription[ 3 ] );
 		case WipeFromBottom:
-			return i18n( "Wipe From Bottom" );
+			return i18n( Defaults::blendEffectDescription[ 4 ] );
 	}
 	kdError( 4620 ) << "Effect description for effect with index " << idx << " doesn't exist\n";
 	return QString::null;
@@ -424,8 +439,12 @@ void KImageCanvas::checkBounds( QSize & newsize )
 		if( ( newsize * x1 != origsize ) || ( newsize * x2 != origsize ) )
 		{
 			// not OK
-			kdDebug( 4620 ) << "checkBounds: the aspect ratio wasn't kept" << endl;
-			// TODO
+			kdDebug( 4620 ) << "checkBounds: the aspect ratio wasn't kept changing from " << newsize << endl;
+			// the user want's that the aspect ratio doesn't change. The
+			// question is: make it larger or smaller?
+			// we make it smaller (we depend on that in boundImageTo)
+			newsize = origsize / KMAX( x1, x2 );
+			kdDebug( 4620 ) << "checkBounds: to " << newsize << endl;
 		}
 	}
 	if( ( ! m_maxsize.isEmpty() ) &&
@@ -590,72 +609,68 @@ void KImageCanvas::contentsMousePressEvent( QMouseEvent * ev )
 {
 	if ( ev->button() == RightButton )
 		emit contextPress( ev->globalPos() );
+	QScrollView::contentsMousePressEvent( ev );
 }
 
 void KImageCanvas::contentsWheelEvent( QWheelEvent * ev )
 {
+	//kdDebug( 4620 ) << k_funcinfo << endl;
     // Ctrl+Wheelmouse changes the zoom.
     // Wheelmouse scrolls around
     if ( ev->state() & ControlButton )
     {
-		if( m_iLinesToIgnore != 0 )
-			--m_iLinesToIgnore;
-		else
+		int delta = ev->delta() / 120;
+		double zoom = m_zoom;
+		// make zoom a value of 1/16, 1/15, 1/14, .. , 1/2, 1, 2, 3, .. , 15, 16
+		bool done = false;
+		for( int i = 15; i > 0; --i )
 		{
-			m_iLinesToIgnore = QApplication::wheelScrollLines() - 1;
-			int delta = ev->delta() / 120;
-			double zoom = m_zoom;
-			// make zoom a value of 1/16, 1/15, 1/14, .. , 1/2, 1, 2, 3, .. , 15, 16
-			bool done = false;
-			for( int i = 15; i > 0; --i )
+			if( zoom <= ( 1.0 / i ) )
 			{
-				if( zoom <= ( 1.0 / i ) )
+				if( zoom < ( 1.0 / ( i + 0.5 ) ) )
+					zoom = ( 1.0 / ( i + 1 ) );
+				else
+					zoom = ( 1.0 / i );
+				done = true;
+				// zoom = 1/16, 1/15, .. , 1/2, 1
+				double x = 1.0 / zoom - delta;
+				if( x == 0 )
+					zoom = 2.0;
+				else
+					zoom = 1.0 / x;
+				break;
+			}
+		}
+		if( ! done )
+			for( int i = 2; i < 17; ++i )
+			{
+				if( zoom < (double)i )
 				{
-					if( zoom < ( 1.0 / ( i + 0.5 ) ) )
-						zoom = ( 1.0 / ( i + 1 ) );
+					if( zoom < ( i - 0.5 ) )
+						zoom = i - 1.0;
 					else
-						zoom = ( 1.0 / i );
+						zoom = (double)i;
 					done = true;
-					// zoom = 1/16, 1/15, .. , 1/2, 1
-					double x = 1.0 / zoom - delta;
-					if( x == 0 )
-						zoom = 2.0;
-					else
-						zoom = 1.0 / x;
+					// zoom = 1, 2, .., 15, 16
+					zoom = zoom + delta;
+					if( zoom < 0.9 )
+						zoom = 0.5;
 					break;
 				}
 			}
-			if( ! done )
-				for( int i = 2; i < 17; ++i )
-				{
-					if( zoom < (double)i )
-					{
-						if( zoom < ( i - 0.5 ) )
-							zoom = i - 1.0;
-						else
-							zoom = (double)i;
-						done = true;
-						// zoom = 1, 2, .., 15, 16
-						zoom = zoom + delta;
-						if( zoom < 0.9 )
-							zoom = 0.5;
-						break;
-					}
-				}
-			if( ! done )
-			{
+		if( ! done )
+		{
+			zoom = 16.0;
+			zoom = zoom + delta;
+			if( zoom > 16.0 )
 				zoom = 16.0;
-				zoom = zoom + delta;
-				if( zoom > 16.0 )
-					zoom = 16.0;
-			}
-			kdDebug( 4620 ) << "Mousewheel: oldzoom = " << m_zoom << " newzoom = " << zoom << endl;
-			ev->accept();
-			bool oldscale = fastScale();
-			setFastScale( true );
-			setZoom( zoom );
-			setFastScale( oldscale );
 		}
+		kdDebug( 4620 ) << "Mousewheel: oldzoom = " << m_zoom << " newzoom = " << zoom << endl;
+		ev->accept();
+		bool oldscale = fastScale();
+		setFastScale( true );
+		setZoom( zoom );
+		setFastScale( oldscale );
     }
     else
         QScrollView::contentsWheelEvent( ev );
@@ -663,7 +678,7 @@ void KImageCanvas::contentsWheelEvent( QWheelEvent * ev )
 
 void KImageCanvas::keyPressEvent( QKeyEvent * ev )
 {
-	kdDebug( 4620 ) << k_funcinfo << endl;
+	//kdDebug( 4620 ) << k_funcinfo << endl;
 	switch( ev->key() )
 	{
 		case Key_Down:
@@ -766,29 +781,37 @@ void KImageCanvas::hideCursor()
 const KPixmap KImageCanvas::pixmap()
 {
 	kdDebug( 4620 ) << k_funcinfo << ( m_bNeedNewPixmap ? "convert from Image" : "use old copy" ) << endl;
+	// create a new Pixmap in m_pixmap if needed
 	if( m_bNeedNewPixmap )
 	{
+		// only do it again if requested
 		m_bNeedNewPixmap = false;
+		// ok, the old one may go now
 		delete m_pixmap;
 
+		// if smoothscaling is wanted and the transformation matrix or the image
+		// itself changed...
 		if( ! m_fastscale && ( m_bMatrixChanged || m_bImageChanged ) )
 		{
 			delete m_imageTransformed;
-			m_imageTransformed = new QImage( m_image->xForm( m_matrix ) );
+			// we create a new image transformed by the matrix
+			m_imageTransformed = new QImage( m_matrix.isIdentity() ? *m_image : m_image->xForm( m_matrix ) );
 			kdDebug( 4620 ) << "Size of m_image: " << m_image->size() << endl;
 			kdDebug( 4620 ) << "Size of m_imageTransformed: " << m_imageTransformed->size() << endl;
 		}
+		// smoothScale or fast scaling via m_matrix
 		m_pixmap = new KPixmap();
 		m_pixmap->convertFromImage( m_fastscale ? *m_image : m_imageTransformed->smoothScale( m_currentsize ), KPixmap::ColorOnly );
 	}
 	if( m_fastscale )
 	{
+		// fast scaling is needed so we need to scale now
 		QWMatrix matrix( m_matrix );
 		matrix.scale( m_zoom, m_zoom );
-		return m_pixmap->xForm( matrix );
+		if( ! matrix.isIdentity() )
+			return m_pixmap->xForm( matrix );
 	}
-	else
-		return *m_pixmap;
+	return *m_pixmap;
 }
 
 void KImageCanvas::slotImageChanged()
@@ -798,6 +821,38 @@ void KImageCanvas::slotImageChanged()
 	m_bNeedNewPixmap = true;
 }
 
+void KImageCanvas::loadSettings()
+{
+	KConfigGroup cfgGroup( KImageCanvasFactory::instance()->config(),
+			"Settings" );
+	setFastScale( ! cfgGroup.readBoolEntry( "Smooth Scaling", ! fastScale() ) );
+	setKeepAspectRatio( cfgGroup.readBoolEntry( "Keep Aspect Ratio",
+				keepAspectRatio() ) );
+	setCentered( cfgGroup.readBoolEntry( "Center Image", centered() ) );
+
+	setBgColor( cfgGroup.readColorEntry( "Background Color", &bgColor() ) );
+
+	setMinimumImageSize( QSize( cfgGroup.readNumEntry( "Minimum Width",
+					minimumImageSize().width() ), cfgGroup.readNumEntry(
+					"Minimum Height", minimumImageSize().height() ) ) );
+	setMaximumImageSize( QSize( cfgGroup.readNumEntry( "Maximum Width",
+					maximumImageSize().width() ), cfgGroup.readNumEntry(
+					"Maximum Height", maximumImageSize().height() ) ) );
+
+	KConfigGroup blendConfig( KImageCanvasFactory::instance()->config(),
+			"Blend Effects" );
+	/* TODO
+	m_vEffects.clear();
+	for( unsigned int i = 1; i <= numOfBlendEffects(); ++i )
+	{
+		if( blendConfig.readBoolEntry( QString::number( i ), false ) )
+			m_vEffects.push_back( i );
+	}
+	// and now tell the canvas what blend effect to use
+	switchBlendEffect();
+	*/
+}
+
 void KImageCanvas::selected( const QRect & rect )
 {
 	//kdDebug( 4620 ) << k_funcinfo << rect << endl;
@@ -812,6 +867,14 @@ void KImageCanvas::selected( const QRect & rect )
 	//kdDebug( 4620 ) << "m_selection = " << m_selection << endl;
 	emit selectionChanged( m_selection );
 }
+
+void KImageCanvas::mapCursorPos( const QPoint & pos )
+{
+	QPoint mapped( static_cast<int>( ( pos.x() + 0.5 ) / m_zoom ), static_cast<int>( ( pos.y() + 0.5 ) / m_zoom ) );
+	//kdDebug( 4620 ) << k_funcinfo << pos << " -> " << mapped << endl;
+	emit cursorPos( mapped );
+}
+
 void KImageCanvas::sizeChanged()
 {
 	kdDebug( 4620 ) << k_funcinfo << endl;
@@ -871,9 +934,14 @@ KImageHolder * KImageCanvas::createNewClient()
 	client->setMinimumSize( 0, 0 );
 	client->setMouseTracking( true );
 	client->installEventFilter( this );
+	setFocusProxy( client );
+	client->setFocusPolicy( QWidget::StrongFocus );
+	client->setFocus();
+
 	addChild( client, 0, 0 );
 
 	connect( client, SIGNAL( contextPress( const QPoint& ) ), SIGNAL( contextPress( const QPoint& ) ) );
+	connect( client, SIGNAL( cursorPos( const QPoint & ) ), SLOT( mapCursorPos( const QPoint & ) ) );
 	connect( client, SIGNAL( selected( const QRect & ) ), SLOT( selected( const QRect & ) ) );
 	connect( client, SIGNAL( wannaScroll( int, int ) ), SLOT( scrollBy( int, int ) ) );
 
