@@ -12,10 +12,10 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
-// $Id: kimagecanvas.h,v 1.14.2.1 2003/02/11 13:29:35 mkretz Exp $
+// $Id: kimagecanvas.h,v 1.21 2003/12/12 14:03:43 faure Exp $
 
 #ifndef _KIMAGECANVAS_H
 #define _KIMAGECANVAS_H
@@ -35,7 +35,7 @@ class KPixmap;
 /**
  * @short KViewCanvas
  * @author Matthias Kretz <kretz@kde.org>
- * @version $Id: kimagecanvas.h,v 1.14.2.1 2003/02/11 13:29:35 mkretz Exp $
+ * @version $Id: kimagecanvas.h,v 1.21 2003/12/12 14:03:43 faure Exp $
  */
 class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 {
@@ -124,7 +124,7 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		/**
 		 * @return the number of available blend effects
 		 */
-		unsigned int numOfBlendEffects() const { return 4; }
+		unsigned int numOfBlendEffects() const;
 
 		/**
 		 * @return the description of the blend effect
@@ -141,6 +141,11 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		 */
 		const QSize & minimumImageSize() const { return m_minsize; }
 
+		/**
+		 * @return a pointer to the QWidget interface of this object
+		 */
+		QWidget * widget() { return static_cast<QWidget *>( this ); }
+
 		bool eventFilter( QObject *, QEvent * );
 
 	signals:
@@ -189,6 +194,11 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		 */
 		void imageChanged();
 
+		/**
+		 * The current mouse cursor position on the image.
+		 */
+		void cursorPos( const QPoint & );
+
 	public slots:
 		/**
 		 * Set if the image should always be centered if the canvas is
@@ -197,12 +207,9 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		void setCentered( bool );
 
 		/**
-		 * give the canvas a new image to show
-		 *
-		 * You may set the zoom for the new image from here. It defaults
-		 * to 100%
+		 * give the canvas a new image to show. The zoom level is kept.
 		 */
-		void setImage( const QImage &, double zoom = 1.0 );
+		void setImage( const QImage & );
 
 		/**
 		 * Give the canvas a new image to show.
@@ -217,6 +224,11 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		 */
 		void setZoom( double );
 
+		/**
+		 * Fit the image into the requested width and height.
+		 */
+		void boundImageTo( const QSize & size );
+
 		/**
 		 * Set the maximum size of the image. If this is set the image will
 		 * never exceed this size.
@@ -297,9 +309,11 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		void slotUpdateImage();
 		void hideCursor();
 		void slotImageChanged();
+		void loadSettings();
 
 	private slots:
 		void selected( const QRect & ); // map rect to unzoomed rect
+		void mapCursorPos( const QPoint & );
 
 	private:
 		enum BlendEffect {
@@ -336,7 +350,6 @@ class KImageCanvas : public QScrollView, public KImageViewer::Canvas
 		bool m_bImageChanged;
 		bool m_bSizeChanged;
 		bool m_bMatrixChanged;
-		int m_iLinesToIgnore;
 		bool m_bNeedNewPixmap;
 		bool m_bCentered;
 		bool m_bImageUpdateScheduled;
