@@ -12,11 +12,11 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
-//$Id: kimageholder.cpp,v 1.16.2.1 2003/02/11 13:29:35 mkretz Exp $
+//$Id: kimageholder.cpp,v 1.19 2003/03/07 22:09:24 waba Exp $
 
 #include <assert.h>
 
@@ -37,6 +37,9 @@ KImageHolder::KImageHolder( QWidget * parent, const char * name )
 	: QWidget( parent, name, Qt::WResizeNoErase | Qt::WRepaintNoErase )
 	, m_selected( false )
 	, m_bSelecting( false )
+	, m_scrollTimerId( 0 )
+	, m_xOffset( 0 )
+	, m_yOffset( 0 )
 	, m_pen( new QPen( QColor( 255, 255, 255 ), 0, DashLine ) )
 	, m_pPixmap( 0 )
 	, m_pDoubleBuffer( 0 )
@@ -79,6 +82,9 @@ void KImageHolder::mousePressEvent( QMouseEvent *ev )
 
 void KImageHolder::mouseMoveEvent( QMouseEvent *ev )
 {
+	//FIXME: when scrolling the cursorpos shouldn't change
+	if( this->rect().contains( ev->pos(), false ) )
+		emit cursorPos( ev->pos() );
 	//kdDebug( 4620 ) << k_funcinfo << " ev->state() = " << ev->state() << endl;
 	if( ev->state() & LeftButton || ev->state() & MidButton )
 	{
@@ -90,6 +96,7 @@ void KImageHolder::mouseMoveEvent( QMouseEvent *ev )
 		}
 		else // create a selection
 		{
+			QWidget * parentwidget = ( QWidget* )parent();
 			if( ! m_bSelecting )
 			{
 				m_bSelecting = true;
@@ -111,6 +118,37 @@ void KImageHolder::mouseMoveEvent( QMouseEvent *ev )
 			if( !m_selected )
 				m_selected = true;
 
+			// Autoscrolling:
+			// For every pixel the mouse leaves the image canvas the canvas
+			// should scroll faster. The scrolling should be independent from
+			// the mouse movement. Therefor I need a timer to do the scroll
+			// movement:
+			// e.g.: m_xOffset=-1 => scroll to the left 1 px per 50ms
+			//                 -2 => scroll to the left 2 px per 50ms
+			//                  2 => scroll to the right 2 px per 50ms
+			//                 ...
+			m_xOffset = mapTo( parentwidget, ev->pos() ).x();
+			m_yOffset = mapTo( parentwidget, ev->pos() ).y();
+			if( m_xOffset > 0 )
+			{
+				m_xOffset -= parentwidget->width();
+				if( m_xOffset < 0 )
+					m_xOffset = 0;
+			}
+			if( m_yOffset > 0 )
+			{
+				m_yOffset -= parentwidget->height();
+				if( m_yOffset < 0 )
+					m_yOffset = 0;
+			}
+			if( m_scrollTimerId != 0 && m_xOffset == 0 && m_yOffset == 0 )
+			{
+				killTimer( m_scrollTimerId );
+				m_scrollTimerId = 0;
+			}
+			else if( m_scrollTimerId == 0 && ( m_xOffset != 0 || m_yOffset != 0 ) )
+				m_scrollTimerId = startTimer( 50 );
+
 			int r = ev->x() < width()  ? ( ev->x() < 0 ? 0 : ev->x() ) : width()  - 1;
 			int b = ev->y() < height() ? ( ev->y() < 0 ? 0 : ev->y() ) : height() - 1;
 
@@ -134,6 +172,15 @@ void KImageHolder::mouseMoveEvent( QMouseEvent *ev )
 
 void KImageHolder::mouseReleaseEvent( QMouseEvent * ev )
 {
+	if( m_bSelecting )
+	{
+		m_xOffset = m_yOffset = 0;
+		if( m_scrollTimerId != 0 )
+		{
+			killTimer( m_scrollTimerId );
+			m_scrollTimerId = 0;
+		}
+	}
 	if( ev->state() & LeftButton || ev->state() & MidButton )
 		if( m_bSelecting )
 			m_bSelecting = false;
@@ -254,6 +301,14 @@ void KImageHolder::paintEvent( QPaintEvent *ev )
 		drawSelect( painter );
 }
 
+void KImageHolder::timerEvent( QTimerEvent * ev )
+{
+	if( ev->timerId() != m_scrollTimerId )
+		return;
+
+	emit wannaScroll( m_xOffset, m_yOffset );
+}
+
 void KImageHolder::setPixmap( const KPixmap & pixmap )
 {
 	kdDebug( 4620 ) << k_funcinfo << " " << pixmap.width() << 'x' << pixmap.height() << endl;
