@@ -29,15 +29,15 @@
 #include <kstandarddirs.h>
 #include <kstdaction.h>
 #include <ktempfile.h>
+#include <kinputdialog.h>
 
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <math.h>
 
+#include "centeringScrollview.h"
 #include "marklist.h"
-#include "scrollbox.h"
-#include "gotodialog.h"
 #include "kpagetest.h"
 #include "kviewpart.moc"
 #include "pageSize.h"
@@ -107,26 +107,17 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   connect(watch, SIGNAL(dirty(const QString&)), this, SLOT(fileChanged(const QString&)));
   watch->startScan();
 
-  connect(&previewPaintInitiator, SIGNAL(timeout()), this, SLOT(doRepaintScrollBoxBackground()) );
-
   mainWidget = new QWidget(parentWidget, widgetName);
   mainWidget->setFocusPolicy(QWidget::StrongFocus);
   setWidget(mainWidget);
 
   QHBoxLayout *hbox = new QHBoxLayout(mainWidget, 0, 0);
-  QVBoxLayout *vbox = new QVBoxLayout(hbox);
-
-  scrollBox = new ScrollBox(mainWidget);
-  scrollBox->setFixedWidth(75);
-  scrollBox->setMinimumHeight(75);
-  vbox->addWidget(scrollBox);
-
-  connect(scrollBox, SIGNAL(valueChanged(QPoint)), this, SLOT(scrollBoxChanged(QPoint)));
 
   markList = new MarkList(mainWidget);
+  markList->setMinimumWidth(80);
   markList->setAutoUpdate(true);
-  vbox->addWidget(markList);
-  vbox->setStretchFactor(markList, 1);
+  hbox->addWidget(markList);
+  //  vbox->setStretchFactor(markList, 1);
 
   connect(markList, SIGNAL(selected(int)), this, SLOT(pageSelected(int)));
 
@@ -153,20 +144,19 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   connect(multiPage, SIGNAL(numberOfPages(int)), this, SLOT(numberOfPages(int)));
   connect(multiPage, SIGNAL(pageInfo(int, int)), this, SLOT(pageInfo(int, int)) );
   connect(multiPage, SIGNAL(askingToCheckActions()), this, SLOT(checkActions()) );
-  connect(multiPage->scrollView(), SIGNAL(contentsMoving(int,int)), this, SLOT(contentsMoving(int,int)));
   multiPage->scrollView()->installEventFilter(this);
   connect( multiPage, SIGNAL( started( KIO::Job * ) ), this, SIGNAL( started( KIO::Job * ) ) );
   connect( multiPage, SIGNAL( completed() ), this, SIGNAL( completed() ) );
   connect( multiPage, SIGNAL( canceled( const QString & ) ), this, SIGNAL( canceled( const QString & ) ) );
-  connect( multiPage, SIGNAL( previewChanged(bool)), this, SLOT(updatePreview(bool)));
   connect( multiPage, SIGNAL( setStatusBarText( const QString& ) ), this, SLOT( setStatusBarTextFromMultiPage( const QString& ) ) );
   connect( multiPage, SIGNAL( documentSpecifiedPageSize(const pageSize&) ), this, SLOT( slotPageSizeRequests(const pageSize&) ) );
   connect( multiPage->scrollView(), SIGNAL( wheelEventReceived(QWheelEvent *)), this,   SLOT(wheelEvent(QWheelEvent *)));
   // settings menu
   showmarklist = new KToggleAction (i18n("Show &Sidebar"), 0, this, SLOT(slotShowMarkList()), actionCollection(), "show_page_list");
-  showPreview = new KToggleAction (i18n("Show P&review"), 0, this, SLOT(slotPreview()), actionCollection(), "show_preview");
+  showmarklist->setCheckedState(i18n("Hide &Sidebar"));
   watchAct = new KToggleAction(i18n("&Watch File"), 0, 0, 0, actionCollection(), "watch_file");
   scrollbarHandling = new KToggleAction (i18n("Show Scrollbars"), 0, 0, 0, actionCollection(), "scrollbarHandling");
+  scrollbarHandling->setCheckedState(i18n("Hide Scrollbars"));
   connect(scrollbarHandling, SIGNAL(toggled(bool)),  multiPage, SLOT(slotShowScrollbars(bool)));
 
   // Orientation menu
@@ -182,10 +172,12 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   zoom_action->setComboWidth(80);
   zoom_action->setEditable(true);
   zoom_action->setItems(_zoomVal.zoomNames());
-  _zoomVal.setZoomValue(1.0); // should not be necessary @@@@
   connect (&_zoomVal, SIGNAL(zoomNamesChanged(const QStringList &)), zoom_action, SLOT(setItems(const QStringList &)));
   connect (&_zoomVal, SIGNAL(valNoChanged(int)), zoom_action, SLOT(setCurrentItem(int)));
+  connect (&_zoomVal, SIGNAL(zoomNameChanged(const QString &)), this, SIGNAL(zoomChanged(const QString &)) );
   connect (zoom_action, SIGNAL(activated(const QString &)), this, SLOT(setZoomValue(const QString &)));
+  _zoomVal.setZoomValue(1.0); // should not be necessary @@@@
+  emit(zoomChanged("100%"));
 
   // Paper Size Menu
   media = new KSelectAction (i18n("Preferred Paper &Size"), 0, 0, 0, actionCollection(), "view_media");
@@ -201,10 +193,19 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   // Zoom Actions
   zoomInAct = KStdAction::zoomIn (this, SLOT(zoomIn()), actionCollection());
   zoomOutAct = KStdAction::zoomOut(this, SLOT(zoomOut()), actionCollection());
-  fitAct = KStdAction::actualSize(this, SLOT(fitSize()), actionCollection());
-  fitPageAct = KStdAction::fitToPage(this, SLOT(fitToPage()), actionCollection());
-  fitWidthAct = KStdAction::fitToWidth(this, SLOT(fitToWidth()), actionCollection());
-  fitHeightAct = KStdAction::fitToHeight(this, SLOT(fitToHeight()), actionCollection());
+  fitPageAct = new KToggleAction(i18n("&Fit To Page"), Key_P, actionCollection(), "view_fit_to_page");
+  fitWidthAct = new KToggleAction(i18n("Fit to Page &Width"), Key_W, actionCollection(), "view_fit_to_width");
+  fitHeightAct = new KToggleAction(i18n("Fit to Page &Height"), Key_H, actionCollection(), "view_fit_to_height");
+
+  fitPageAct -> setExclusiveGroup("view_fit");
+  fitWidthAct -> setExclusiveGroup("view_fit");
+  fitHeightAct -> setExclusiveGroup("view_fit");
+    
+  connect(fitPageAct, SIGNAL(toggled(bool)), this, SLOT(enableFitToPage(bool)));
+  connect(fitWidthAct, SIGNAL(toggled(bool)), this, SLOT(enableFitToWidth(bool)));
+  connect(fitHeightAct, SIGNAL(toggled(bool)), this, SLOT(enableFitToHeight(bool)));
+  
+  connect(multiPage, SIGNAL(viewModeChanged()), this, SLOT(updateZoomLevel()));
 
   // go menu
   backAct = KStdAction::prior(this, SLOT(prevPage()), actionCollection());
@@ -213,8 +214,8 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   endAct = KStdAction::lastPage(this, SLOT(lastPage()), actionCollection());
   gotoAct = KStdAction::gotoPage(this, SLOT(goToPage()), actionCollection());
 
-  readUpAct = new KAction(i18n("Read Up Document"), "previous", SHIFT+Key_Space, this, SLOT(readUp() ), actionCollection(), "go_read_up");
-  readDownAct = new KAction(i18n("Read Down Document"), "next", Key_Space, this, SLOT(readDown() ), actionCollection(), "go_read_down");
+  readUpAct = new KAction(i18n("Read Up Document"), "up", SHIFT+Key_Space, this, SLOT(readUp() ), actionCollection(), "go_read_up");
+  readDownAct = new KAction(i18n("Read Down Document"), "down", Key_Space, this, SLOT(readDown() ), actionCollection(), "go_read_down");
 
   printAction = KStdAction::print(this, SLOT(slotPrint()), actionCollection());
   if (multiPage->isReadWrite() )
@@ -239,11 +240,6 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
 
   m_extension = new KViewPartExtension(this);
 
-  // create the goto dialog
-  _gotoDialog = new GotoDialog(mainWidget);
-  _gotoDialog->hide();
-  connect(_gotoDialog, SIGNAL(gotoPage(const QString&)), this, SLOT(slotGotoDialog(const QString&)));
-
   // The page size dialog is constructed on first usage -- saves some
   // memory when not used.
   _pageSizeDialog = 0;
@@ -285,6 +281,11 @@ KViewPart::~KViewPart()
 }
 
 
+void KViewPart::slotStartFitTimer()
+{
+  fitTimer.start(100, true);
+}
+
 QString KViewPart::pageSizeDescription(void)
 {
   // Display the current paper size in the status bar. If the format
@@ -318,13 +319,8 @@ void KViewPart::setPaperSize(float width_in_mm, float height_in_mm)
   float w = width_in_mm/10.0;
   float h = height_in_mm/10.0;
 
-  // reflect in scrollbox
-  scrollBox->setMinimumHeight( int(75.0*h/w+0.5));
-
   // forward
   multiPage->setPaperSize(w, h);
-
-  updateScrollBox();
 }
 
 
@@ -375,12 +371,11 @@ void KViewPart::slotSetFullPage(bool fullpage)
   if (multiPage)
     multiPage->slotSetFullPage(fullpage);
   else
-    kdError() << "KViewPart::slotSetFullPage() called without existing multipage" << endl;
+    kdError(4300) << "KViewPart::slotSetFullPage() called without existing multipage" << endl;
 
-  if (fullpage == true) {
+  if (fullpage == true)
     markList->hide();
-    scrollBox->hide();
-  } else 
+  else 
     slotShowMarkList();
 }
 
@@ -399,13 +394,10 @@ void KViewPart::slotPageSizeRequests(const pageSize &size)
 
 void KViewPart::slotShowMarkList()
 {
-  if (showmarklist->isChecked()) {
+  if (showmarklist->isChecked())
     markList->show();
-    scrollBox->show();
-  } else {
+  else 
     markList->hide();
-    scrollBox->hide();
-  }
 }
 
 
@@ -548,7 +540,6 @@ bool KViewPart::openFile()
   documentHasRequestedPaperSize = false;
   bool r = multiPage->openURL(tmpFileURL);
   
-  updateScrollBox();
   markList->select(0);
   
   if (r) {
@@ -570,8 +561,7 @@ bool KViewPart::openFile()
   }
   
   checkActions();
-  scrollBox->setMinimumHeight( int(75.0*_paperSize.height_in_mm()/_paperSize.width_in_mm()+0.5));
-
+  emit(zoomChanged(QString("%1%").arg((int)(_zoomVal.value()*100.0+0.5))));
   return r;
 }
 
@@ -671,16 +661,6 @@ void KViewPart::slotUseDocumentSpecifiedSize(bool doUse)
 }
 
 
-void KViewPart::doRepaintScrollBoxBackground(void)
-{
-  QPixmap pixmap(scrollBox->width(), scrollBox->height());
-  QPainter p(&pixmap);
-  
-  if (multiPage->preview(&p, scrollBox->width(), scrollBox->height()))
-    scrollBox->setBackgroundPixmap(pixmap);
-}
-
-
 void KViewPart::numberOfPages(int nr)
 {
   _numberOfPages = nr;
@@ -688,9 +668,6 @@ void KViewPart::numberOfPages(int nr)
   markList->clear();
 
   if (nr == 0) {
-    // clear scroll box
-    scrollBox->setPageSize(QSize(0,0));
-    scrollBox->setViewSize(QSize(0,0));
     _currentPage = 0;
     return;
   }
@@ -704,33 +681,32 @@ void KViewPart::numberOfPages(int nr)
 
 void KViewPart::pageInfo(int numpages, int currentpage)
 {
-  _numberOfPages = numpages;
-
-  markList->clear();
-
-  if (numpages == 0) {
-    // clear scroll box
-    scrollBox->setPageSize(QSize(0,0));
-    scrollBox->setViewSize(QSize(0,0));
-    emit pageChanged(QString::null);
-    _currentPage = 0;
-    return;
+  bool markListChanged = false;
+  if (_numberOfPages != numpages) {
+    markListChanged = true;
+    _numberOfPages = numpages;
+    markList->clear();
+    if (numpages == 0) {
+      emit pageChanged(QString::null);
+      _currentPage = 0;
+      return;
+    }
+    for (int i=0; i<numpages; i++)
+      markList->insertItem(QString("%1").arg(i+1), i);
+  }
+  
+  if ((_currentPage != currentpage) || (markListChanged == true)) {
+    _currentPage = currentpage;
+    // ATTN: The string here must be the same as in setPage() below
+    QString pageString = i18n("Page %1 of %2").arg(_currentPage+1).arg(_numberOfPages);
+    if (pageChangeIsConnected)
+      emit pageChanged(pageString);
+    else
+      emit setStatusBarText(pageString);
+    
+    markList->select(currentpage);
+    checkActions();
   }
-
-  for (int i=0; i<numpages; i++)
-    markList->insertItem(QString("%1").arg(i+1), i);
-
-  _currentPage = currentpage;
-  // ATTN: The string here must be the same as in setPage() below
-  QString pageString = i18n("Page %1 of %2").arg(_currentPage+1).arg(_numberOfPages);
-  if (pageChangeIsConnected)
-    emit pageChanged(pageString);
-  else
-    emit setStatusBarText(pageString);
-
-  markList->select(currentpage);
-  checkActions();
-  updateScrollBox();
 }
 
 
@@ -755,21 +731,27 @@ void KViewPart::setPage(int page)
   markList->select(page);
 
   checkActions();
-  updateScrollBox();
 }
 
 
 void KViewPart::prevPage()
 {
-  if (page() > 0)
-    setPage(page()-1);
+  Q_UINT8 cols = multiPage->scrollView()->numCols();
+  
+  Q_UINT16 np = 0;
+  if (cols < page())
+    np = page()-cols;
+  if (np != page())
+    setPage(np);
 }
 
 
 void KViewPart::nextPage()
 {
-  if (page()+1 < pages())
-    setPage(page()+1);
+  Q_UINT8 cols = multiPage->scrollView()->numCols();
+  Q_UINT16 np = QMIN(page()+cols, pages()-1);
+  if (np != page())
+    setPage(np);
 }
 
 
@@ -787,16 +769,11 @@ void KViewPart::lastPage()
 
 void KViewPart::goToPage()
 {
-  _gotoDialog->show();
-}
-
-
-void KViewPart::slotGotoDialog(const QString &page)
-{
-  bool ok;
-  int p = page.toInt(&ok)-1;
-  if (ok && p >= 0 && p < pages())
-    setPage(p);
+  bool ok = false;
+  int p = KInputDialog::getInteger(i18n("Go to Page"), i18n("Enter page number:"), 
+                        page()+1, 1, pages(), 1 /*step*/, &ok, mainWidget, "gotoDialog"); 
+  if (ok)
+    setPage(p-1);
 }
 
 
@@ -807,43 +784,143 @@ void KViewPart::pageSelected(int nr)
   // and then we don't want the mulipage to render twice.
   if (_currentPage == nr)
     return;
-
+  
   if ((nr >= 0) && (nr<pages()))
     setPage(nr);
 }
 
+void KViewPart::disableZoomFit()
+{
+  if (fitPageAct -> isChecked())
+  {
+    fitPageAct -> setChecked(false);
+    enableFitToPage(false);
+  }
+  else if(fitWidthAct -> isChecked())
+  {  
+    fitWidthAct -> setChecked(false);
+    enableFitToWidth(false);
+  }
+  else if (fitHeightAct -> isChecked())
+  {
+    fitHeightAct -> setChecked(false);
+    enableFitToHeight(false);
+  }
+}
 
 void KViewPart::zoomIn()
 {
+  disableZoomFit();
+
   float oldVal = _zoomVal.value();
   float newVal = _zoomVal.zoomIn();
 
-  if (oldVal != newVal) {
+  if (oldVal != newVal)
     _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.zoomIn()));
-    updateScrollBox();
-  }
 }
 
 
 void KViewPart::zoomOut()
 {
+  disableZoomFit();
+
   float oldVal = _zoomVal.value();
   float newVal = _zoomVal.zoomOut();
 
-  if (oldVal != newVal) {
+  if (oldVal != newVal)
     _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.zoomOut()));
-    updateScrollBox();
+}
+
+void KViewPart::updateZoomLevel()
+{
+  if (fitPageAct->isChecked())
+  {
+    fitToPage();
+  }
+  else if (fitWidthAct->isChecked())
+  {
+    fitToWidth();
+  }
+  else if (fitHeightAct->isChecked())
+  {
+    fitToHeight();
+  }
+  else
+  {
+    // Manuell Zoom
   }
 }
 
+void KViewPart::enableFitToPage(bool enable)
+{
+  if (enable)
+  {
+    fitToPage();
+    connect(multiPage->scrollView(), SIGNAL(viewSizeChanged(QSize)), 
+            this, SLOT(slotStartFitTimer()));
+    connect(&fitTimer, SIGNAL(timeout()), SLOT(fitToPage()));
+  }
+  else
+  {
+    disconnect(multiPage->scrollView(), SIGNAL(viewSizeChanged(QSize)),
+               this, SLOT(slotStartFitTimer()));
+    disconnect(&fitTimer, SIGNAL(timeout()), this, SLOT(fitToPage()));
+  }
+}
+
+void KViewPart::enableFitToWidth(bool enable)
+{
+  if (enable)
+  {
+    fitToWidth();
+    connect(multiPage->scrollView(), SIGNAL(viewSizeChanged(QSize)), 
+            this, SLOT(slotStartFitTimer()));
+    connect(&fitTimer, SIGNAL(timeout()), SLOT(fitToWidth()));
+  }
+  else
+  {
+    disconnect(multiPage->scrollView(), SIGNAL(viewSizeChanged(QSize)),
+               this, SLOT(slotStartFitTimer()));
+    disconnect(&fitTimer, SIGNAL(timeout()), this, SLOT(fitToWidth()));
+  }
+}
+
+void KViewPart::enableFitToHeight(bool enable)
+{
+  if (enable)
+  {
+    fitToHeight();
+    connect(multiPage->scrollView(), SIGNAL(viewSizeChanged(QSize)), 
+            this, SLOT(slotStartFitTimer()));
+    connect(&fitTimer, SIGNAL(timeout()), SLOT(fitToHeight()));
+  }
+  else
+  {
+    disconnect(multiPage->scrollView(), SIGNAL(viewSizeChanged(QSize)),
+               this, SLOT(slotStartFitTimer()));
+    disconnect(&fitTimer, SIGNAL(timeout()), this, SLOT(fitToHeight()));
+  }
+}
 
 void KViewPart::fitToPage()
 {
   double w, h;
+  int targetHeight, targetWidth;
+  int pageHeight, pageWidth;
   float _zoom;
 
-  w = multiPage->zoomForWidth(sizeOfPage().width());
-  h = multiPage->zoomForHeight(sizeOfPage().height());
+  // See below, in the documentation of the method "fitToWidth" for an
+  // explanation of the complicated calculation we are doing here.
+  targetHeight = multiPage->scrollView()->viewportSize(0,0).height()-1;
+  targetWidth  = (int)(targetHeight * _paperSize.width_in_mm()/_paperSize.height_in_mm() +0.5);
+  pageHeight   = multiPage->scrollView()->viewportSize(targetWidth, targetHeight).height()-1;
+
+  targetWidth  = multiPage->scrollView()->viewportSize(0,0).width()-1;
+  targetHeight = (int)(targetWidth * _paperSize.height_in_mm()/_paperSize.width_in_mm() +0.5);
+  pageWidth    = multiPage->scrollView()->viewportSize(targetWidth, targetHeight).width()-1;
+
+  w = multiPage->zoomForWidth(pageWidth);
+  h = multiPage->zoomForHeight(pageHeight);
 
   if (w < h)
     _zoom = w;
@@ -854,23 +931,18 @@ void KViewPart::fitToPage()
   if (_zoom > ZoomLimits::MaxZoom/1000.0)
     _zoom = ZoomLimits::MaxZoom/1000.0;
   _zoomVal.setZoomValue(multiPage->setZoom(_zoom));
-  updateScrollBox();
-}
-
-
-void KViewPart::fitSize()
-{
-  _zoomVal.setZoomValue(multiPage->setZoom(1.0));
-  updateScrollBox();
 }
 
 
 void KViewPart::fitToHeight()
 {
+  double paperWidthInMillimeters = _paperSize.width_in_mm() * multiPage->scrollView()->getNrColumns();
   // See below, in the documentation of the method "fitToWidth" for an
   // explanation of the complicated calculation we are doing here.
   int targetHeight = multiPage->scrollView()->viewportSize(0,0).height()-1;
-  int targetWidth  = (int)(targetHeight * _paperSize.width_in_mm()/_paperSize.height_in_mm() +0.5);
+  int targetWidth  = (int)(targetHeight * paperWidthInMillimeters/_paperSize.height_in_mm() + 0.5);
+  // We have to account for the gaps between pages and the pagedecorations.
+  targetWidth += (multiPage->scrollView()->getNrColumns()-1) * CenteringScrollview::distanceBetweenWidgets+28;
   targetHeight = multiPage->scrollView()->viewportSize(targetWidth, targetHeight).height()-1;
 
   float _zoom = multiPage->zoomForHeight(targetHeight);
@@ -879,7 +951,6 @@ void KViewPart::fitToHeight()
   if (_zoom > ZoomLimits::MaxZoom/1000.0)
     _zoom = ZoomLimits::MaxZoom/1000.0;
   _zoomVal.setZoomValue(multiPage->setZoom(_zoom));
-  updateScrollBox();
 }
 
 
@@ -910,17 +981,17 @@ void KViewPart::fitToWidth()
   if (_zoom > ZoomLimits::MaxZoom/1000.0)
     _zoom = ZoomLimits::MaxZoom/1000.0;
   _zoomVal.setZoomValue(multiPage->setZoom(_zoom));
-  updateScrollBox();
 }
 
 
 void KViewPart::setZoomValue(const QString &sval)
 {
+  disableZoomFit();
+  
   float fval = _zoomVal.value();
   _zoomVal.setZoomValue(sval);
   if (fval != _zoomVal.value())
     _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.value()));
-  updateScrollBox();
 }
 
 
@@ -931,16 +1002,6 @@ QSize KViewPart::sizeOfPage()
 }
 
 
-void KViewPart::updateScrollBox()
-{
-  QScrollView *sv = multiPage->scrollView();
-  scrollBox->setPageSize(QSize(sv->contentsWidth(), sv->contentsHeight()));
-  scrollBox->setViewSize(QSize(sv->visibleWidth(), sv->visibleHeight()));
-  scrollBox->setViewPos(QPoint(sv->contentsX(), sv->contentsY()));
-  emit zoomChanged(QString("%1%").arg((int)(_zoomVal.value()*100.0+0.5)));
-}
-
-
 void KViewPart::checkActions()
 {
   bool doc = !url().isEmpty();
@@ -951,11 +1012,11 @@ void KViewPart::checkActions()
   endAct->setEnabled(doc && page()+1 < pages());
   gotoAct->setEnabled(doc && pages()>1);
   readDownAct->setEnabled(doc);
+  readUpAct->setEnabled(doc);
 
   zoomInAct->setEnabled(doc);
   zoomOutAct->setEnabled(doc);
 
-  fitAct->setEnabled(doc);
   fitPageAct->setEnabled(doc);
   fitHeightAct->setEnabled(doc);
   fitWidthAct->setEnabled(doc);
@@ -984,26 +1045,8 @@ void KViewPart::checkActions()
 }
 
 
-void KViewPart::contentsMoving(int x, int y)
-{
-  QScrollView *sv = multiPage->scrollView();
-  scrollBox->setPageSize(QSize(sv->contentsWidth(), sv->contentsHeight()));
-  scrollBox->setViewSize(QSize(sv->visibleWidth(), sv->visibleHeight()));
-  scrollBox->setViewPos(QPoint(x,y));
-}
-
-
-void KViewPart::scrollBoxChanged(QPoint np)
-{
-  multiPage->scrollView()->setContentsPos(np.x(), np.y());
-}
-
-
 bool KViewPart::eventFilter(QObject *obj, QEvent *ev)
 {
-  if (obj == this && ev->type() == QEvent::Resize)
-    QTimer::singleShot(0, this, SLOT(updateScrollBox()));
-
   if (obj != this) {
     if (ev->type() == QEvent::MouseButtonPress) {
       mousePos = ((QMouseEvent*)ev)->globalPos();
@@ -1032,13 +1075,27 @@ void KViewPart::wheelEvent(QWheelEvent *e)
   QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
   if (sb == 0)
     return;
+
+  // Zoom in/out if 'control' is pressed
+  if (e->state() & ControlButton) {
+    if (e->delta() < 0)
+      zoomOut();
+    else
+      zoomIn();
+    return;
+  }
   
   Q_INT32 pxl = -(e->delta()*sb->lineStep())/60;
-  if (pxl == 0) 
+  if (pxl == 0)
     if (e->delta() > 0)
       pxl = -1;
     else
       pxl = 1;
+  
+  // Faster scrolling when 'shift' is pressed
+  if (e->state() & ShiftButton)
+    pxl *= 10;
+  
   scroll(pxl);
 }
 
@@ -1064,7 +1121,8 @@ void KViewPart::scrollDown()
 
 void KViewPart::scroll(Q_INT32 deltaInPixel)
 {
-  QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
+  QScrollView *sv = multiPage->scrollView();
+  QScrollBar *sb = sv->verticalScrollBar();
   if (sb == 0) {
     kdError(4300) << "KViewpart::scroll called without scrollBar" << endl;
     return;
@@ -1075,6 +1133,10 @@ void KViewPart::scroll(Q_INT32 deltaInPixel)
       if ( (page() == 0) || (changePageDelayTimer.isActive()) )
 	return;
       
+      const QObjectList *ol = sv->queryList("documentWidget");
+      if ((ol == 0) || (ol->getFirst() != ol->getLast()))
+	return;
+      
       changePageDelayTimer.stop();
       prevPage();
       QScrollView *sv = multiPage->scrollView();
@@ -1088,6 +1150,10 @@ void KViewPart::scroll(Q_INT32 deltaInPixel)
       if ( (page()+1 == pages()) || (changePageDelayTimer.isActive()) )
 	return;
       
+      const QObjectList *ol = sv->queryList("documentWidget");
+      if ((ol == 0) || (ol->getFirst() != ol->getLast()))
+	return;
+      
       changePageDelayTimer.stop();
       nextPage();
       QScrollView *sv = multiPage->scrollView();
@@ -1097,20 +1163,19 @@ void KViewPart::scroll(Q_INT32 deltaInPixel)
   }
   
   sb->setValue(sb->value() + deltaInPixel);
-
+  
   if ( (sb->value() == sb->maxValue()) || (sb->value() == sb->minValue()) )
     changePageDelayTimer.start(200,true);
   else
     changePageDelayTimer.stop();
 }
-
-
+  
+  
 void KViewPart::scrollLeft()
 {
   QScrollBar *sb = multiPage->scrollView()->horizontalScrollBar();
   if (sb)
     sb->subtractLine();
-  updateScrollBox();
 }
 
 
@@ -1119,7 +1184,6 @@ void KViewPart::scrollRight()
   QScrollBar *sb = multiPage->scrollView()->horizontalScrollBar();
   if (sb)
     sb->addLine();
-  updateScrollBox();
 }
 
 
@@ -1128,7 +1192,6 @@ void KViewPart::scrollUpPage()
   QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
   if (sb)
     sb->subtractPage();
-  updateScrollBox();
 }
 
 
@@ -1137,7 +1200,6 @@ void KViewPart::scrollDownPage()
   QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
   if (sb)
     sb->addPage();
-  updateScrollBox();
 }
 
 
@@ -1146,7 +1208,6 @@ void KViewPart::scrollLeftPage()
   QScrollBar *sb = multiPage->scrollView()->horizontalScrollBar();
   if (sb)
     sb->subtractPage();
-  updateScrollBox();
 }
 
 
@@ -1155,7 +1216,6 @@ void KViewPart::scrollRightPage()
   QScrollBar *sb = multiPage->scrollView()->horizontalScrollBar();
   if (sb)
     sb->addPage();
-  updateScrollBox();
 }
 
 
@@ -1163,35 +1223,6 @@ void KViewPart::scrollTo(int x, int y)
 {
   QScrollView *sv = multiPage->scrollView();
   sv->setContentsPos(x, y);
-  updateScrollBox();
-}
-
-
-void KViewPart::updatePreview(bool avail)
-{
-  if (showPreview->isChecked() && showmarklist->isChecked() && avail)
-    // Rather than calling the method doRepaintScrollBoxBackground()
-    // here (which turns to the multipage to have a pixmap for the
-    // preview drawn), we use a timer here whose timeout() signal is
-    // connected to doRepaintScrollBoxBackground(). Giving the
-    // interval is 0ms will, according to the manual of QT, make the
-    // timer time out only after all the events in the window system's
-    // event queue have been processed.
-
-    // The reason: shrinking the pixmap to create the preview often
-    // takes a while. Doing that after all other events are processed
-    // makes sure that the user interface is not blocked. That is
-    // particularly important when the user uses repeat keystrokes to
-    // read through the document.
-    previewPaintInitiator.start(0, true);
-  else
-    scrollBox->setBackgroundMode(QFrame::PaletteMid);
-}
-
-
-void KViewPart::slotPreview()
-{
-  updatePreview(true);
 }
 
 
@@ -1218,12 +1249,36 @@ void KViewPart::readSettings()
   slotShowMarkList();
 
   watchAct->setChecked(config->readBoolEntry("WatchFile", true));
-  showPreview->setChecked(config->readBoolEntry("ShowPreview", false));
+
+  // The value 'fitToPage' has several meanings: 1 is 'fit to page
+  // width', 2 is 'fit to page height', 3 is 'fit to page'. Other
+  // values indicate 'no fit to page'. Note: at the time this code is
+  // executed, the methods fitToWidth(), etc., do not work well at all
+  // (perhaps some data is not initialized yet)? For that reason, we
+  // do not call these methods, and load the last zoom-value from the
+  // configuration file below. The hope is that this value is not
+  // terribly wrong. If the user doesn't like it, it suffices to
+  // resize the window just a bit...
+  int _fitToPage = config->readNumEntry("FitToPage", 0);
+  if (_fitToPage == 1) {
+    fitWidthAct->setChecked(true);
+    enableFitToWidth(true);
+  }
+  if (_fitToPage == 2) {
+    fitHeightAct->setChecked(true);
+    enableFitToHeight(true);
+  }
+  if (_fitToPage == 3) {
+    fitPageAct->setChecked(true);
+    enableFitToPage(true);
+  }
+  
+  // Load the zoom value, and rectify the mess that is created above.
   float _zoom = config->readDoubleNumEntry("Zoom", 1.0);
   if ( (_zoom < ZoomLimits::MinZoom/1000.0) || (_zoom > ZoomLimits::MaxZoom/1000.0))
     _zoom = 1.0;
   _zoomVal.setZoomValue(multiPage->setZoom(_zoom));
-
+  
   // Read Paper Size. and orientation. The GUI is updated
   // automatically by the signals/slots mechanism whenever
   // userRequestedPaperSize is changed.
@@ -1248,9 +1303,20 @@ void KViewPart::writeSettings()
 
   config->setGroup("GUI");
 
+  // The value 'FitToPage' has several meanings: -1 is 'fit to page
+  // width', -2 is 'fit to page height', -3 is 'fit to page'. Other
+  // values mean: none of this
+  if (fitPageAct->isChecked())
+    config->writeEntry("FitToPage", 3);
+  else if(fitWidthAct->isChecked())
+    config->writeEntry("FitToPage", 1);
+  else if (fitHeightAct->isChecked())
+    config->writeEntry("FitToPage", 2);
+  else
+    config->writeEntry("FitToPage", 0);
+
   config->writeEntry("PageMarks", showmarklist->isChecked());
   config->writeEntry("WatchFile", watchAct->isChecked());
-  config->writeEntry("ShowPreview", showPreview->isChecked());
   config->writeEntry("Zoom", _zoomVal.value());
   config->writeEntry("PaperFormat", userRequestedPaperSize.serialize() );
   config->writeEntry("Scrollbars", scrollbarHandling->isChecked() );
@@ -1261,35 +1327,45 @@ void KViewPart::writeSettings()
 
 void KViewPart::readDown()
 {
-  QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
-  if (sb) {
-    if (sb->value() == sb->maxValue()) {
-      if (page()+1 == pages())
-	return;
+  QScrollView *sv = multiPage->scrollView();
+  QScrollBar *sb = sv->verticalScrollBar();
+  if (sb == 0)
+    return;
+  
+  if (sb->value() == sb->maxValue()) {
+    const QObjectList *ol = sv->queryList("documentWidget");
+    if ((ol == 0) || (ol->getFirst() != ol->getLast()))
+      return;
 
-      nextPage();
-      QScrollView *sv = multiPage->scrollView();
-      scrollTo(sv->contentsX(), 0);
-    } else
-      sb->addPage();
-  }
+    if (page()+1 == pages())
+      return;
+    
+    nextPage();
+    scrollTo(sv->contentsX(), 0);
+  } else
+    sb->addPage();
 }
 
 
 void KViewPart::readUp()
 {
-  QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
-  if (sb) {
-    if (sb->value() == sb->minValue()) {
-      if (page() == 0)
-	return;
+  QScrollView *sv = multiPage->scrollView();
+  QScrollBar *sb = sv->verticalScrollBar();
+  if (sb == 0)
+    return;
+  
+  if (sb->value() == sb->minValue()) {
+    const QObjectList *ol = sv->queryList("documentWidget");
+    if ((ol == 0) || (ol->getFirst() != ol->getLast()))
+      return;
 
-      prevPage();
-      QScrollView *sv = multiPage->scrollView();
-      scrollTo(sv->contentsX(), sb->maxValue());
-    } else
-      sb->subtractPage();
-  }
+    if (page() == 0)
+      return;
+    
+    prevPage();
+    scrollTo(sv->contentsX(), sb->maxValue());
+  } else
+    sb->subtractPage();
 }
 
 
@@ -1312,14 +1388,16 @@ void KViewPart::setStatusBarTextFromMultiPage( const QString &msg )
     emit setStatusBarText(msg);
 }
 
+
 void KViewPart::partActivateEvent( KParts::PartActivateEvent *ev )
 {
-    QApplication::sendEvent( multiPage, ev );
+  QApplication::sendEvent( multiPage, ev );
 }
 
+
 void KViewPart::guiActivateEvent( KParts::GUIActivateEvent *ev )
 {
-    QApplication::sendEvent( multiPage, ev );
+  QApplication::sendEvent( multiPage, ev );
 }
 
 
