@@ -16,12 +16,11 @@
 class QDoubleValidator;
 
 class pageSizeWidget : public pageSizeWidget_base
-{ 
+{
   Q_OBJECT
 
 public:
   pageSizeWidget( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
-  ~pageSizeWidget();
 
   // Sets the page size. If the dialog is already shown, updates all
   // the GUI. Accepts the same strings as input as the setPageSize()
@@ -29,8 +28,8 @@ public:
   void setPageSize(QString);
 
   void setOrientation(int ori);
-  
-  QString serialize(void) {return chosenSize->serialize();};
+
+  const pageSize &pageSizeData(void) const {return chosenSize;};
 
 protected slots:
   void paperSize(int);
@@ -38,13 +37,13 @@ protected slots:
 
   // Dummy function, for convenience. Ignores the argument and calls
   // the fillTextFields(void) slot.
-  void unitsChanged(int); 
+  void unitsChanged(int);
   void orientationChanged(int = 0);
-  void input(const QString &); 
+  void input(const QString &);
 
 private:
-  pageSize          *chosenSize;
-  
+  pageSize           chosenSize;
+
   QDoubleValidator  *widthValidator;
   QDoubleValidator  *heightValidator;
 };
