@@ -1,5 +1,5 @@
 /* This file is part of the KDE Project
-   Copyright (C) 1999 Klaas Freitag <freitag@suse.de>  
+   Copyright (C) 1999 Klaas Freitag <freitag@suse.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -16,7 +16,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   $Id: kscandevice.cpp,v 1.37 2002/09/26 06:54:45 freitag Exp $
+   $Id: kscandevice.cpp,v 1.44 2004/04/08 14:32:00 freitag Exp $
 */
 
 #include <stdlib.h>
@@ -45,6 +45,7 @@
 #include "kscanoption.h"
 #include "kscanoptset.h"
 #include "devselector.h"
+#include <ksimpleconfig.h>
 
 #define MIN_PREVIEW_DPI 75
 #define UNDEF_SCANNERNAME I18N_NOOP( "undefined" )
@@ -110,7 +111,7 @@ KScanOption *KScanDevice::getGuiElement( const QCString& name, QWidget *parent,
    {
       /** store new gui-elem in list of all gui-elements */
       gui_elements.append( so );
-	 	
+
       w = so->createWidget( parent, desc, tooltip );
       if( w )
       {
@@ -134,7 +135,7 @@ KScanOption *KScanDevice::getGuiElement( const QCString& name, QWidget *parent,
       delete so;
       so = 0;
    }
-	
+
    return( so );
 }
 // ---------------------------------------------------------------------------
@@ -203,7 +204,8 @@ KScanDevice::KScanDevice( QObject *parent )
 KScanDevice::~KScanDevice()
 {
   if( storeOptions )  delete (storeOptions );
- 
+  kdDebug(29000) << "Calling sane_exit to finish sane!" << endl;
+  sane_exit();
 }
 
 
@@ -232,7 +234,7 @@ KScanStat KScanDevice::openDevice( const QCString& backend )
 	     // fill description dic with names and numbers
 	stat = find_options();
 	scanner_name = backend;
-	
+
       } else {
 	stat = KSCAN_ERR_OPEN_DEV;
 	scanner_name = UNDEF_SCANNERNAME;
@@ -250,6 +252,9 @@ void KScanDevice::slCloseDevice( )
    /* First of all, send a signal to close down the scanner dev. */
    emit( sigCloseDevice( ));
 
+   kdDebug(29000) << "Saving scan settings" << endl;
+   slSaveScanConfigSet( DEFAULT_OPTIONSET, i18n("the default startup setup"));
+
    /* After return, delete all related stuff */
    scanner_name = UNDEF_SCANNERNAME;
    if( scanner_handle )
@@ -289,7 +294,7 @@ QString KScanDevice::getScannerName(const QCString& name) const
   if( scanner ) {
      // ret.sprintf( "%s %s %s", scanner->vendor, scanner->model, scanner->type );
      ret.sprintf( "%s %s", scanner->vendor, scanner->model );
-  }		
+  }
 
   kdDebug(29000) << "getScannerName returns <" << ret << ">" << endl;
   return ( ret );
@@ -361,7 +366,7 @@ KScanStat KScanDevice::find_options()
 		 const QString qq = newOpt->get();
 		 qDebug( "INIT: <%s> = <%s>", d->name, qq );
 		 allOptionSet->insert( d->name, newOpt );
-#endif		
+#endif
 	      }
 	      else if( d->type == SANE_TYPE_GROUP )
 	      {
@@ -435,7 +440,7 @@ KScanStat KScanDevice::apply( KScanOption *opt, bool isGammaTable )
    if( ! opt->initialised() || opt->getBuffer() == 0 )
    {
       kdDebug(29000) << "Attempt to set Zero buffer of " << oname << " -> skipping !" << endl;
-   	
+
       if( opt->autoSetable() )
       {
 	 kdDebug(29000) << "Setting option automatic !" << endl;
@@ -450,7 +455,7 @@ KScanStat KScanDevice::apply( KScanOption *opt, bool isGammaTable )
       stat = KSCAN_ERR_PARAM;
    }
    else
-   {	
+   {
       if( ! opt->active() )
       {
 	 kdDebug(29000) << "Option " << oname << " is not active now!" << endl;
@@ -463,13 +468,13 @@ KScanStat KScanDevice::apply( KScanOption *opt, bool isGammaTable )
       }
       else
       {
-	
+
 	 sane_stat = sane_control_option( scanner_handle, *num,
 					  SANE_ACTION_SET_VALUE,
 					  opt->getBuffer(),
 					  &sane_result );
       }
-   }		
+   }
 
    if( stat == KSCAN_OK )
    {
@@ -569,7 +574,7 @@ QCString KScanDevice::aliasName( const QCString& name )
     {
 	if((*option_dic)["gamma-correction"])
 	    ret = "gamma-correction";
-	
+
     }
 
     if( ret != name )
@@ -594,13 +599,13 @@ void KScanDevice::slReloadAllBut( KScanOption *not_opt )
         {
             kdDebug(29000) << "ReloadAllBut called with invalid argument" << endl;
             return;
-        }        	
+        }
 
         /* Make sure its applied */
 	apply( not_opt );
 
 	kdDebug(29000) << "*** Reload of all except <" << not_opt->getName() << "> forced ! ***" << endl;
-	
+
 	for( KScanOption *so = gui_elements.first(); so; so = gui_elements.next())
 	{
 	    if( so != not_opt )
@@ -611,7 +616,7 @@ void KScanDevice::slReloadAllBut( KScanOption *not_opt )
 	    }
 	}
 	kdDebug(29000) << "*** Reload of all finished ! ***" << endl;
-		
+
 }
 
 
@@ -619,7 +624,7 @@ void KScanDevice::slReloadAllBut( KScanOption *not_opt )
 void KScanDevice::slReloadAll( )
 {
 	kdDebug(29000) << "*** Reload of all forced ! ***" << endl;
-	
+
 	for( KScanOption *so = gui_elements.first(); so; so = gui_elements.next())
 	{
 	 	so->slReload();
@@ -707,12 +712,17 @@ KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
 
    if( dpi == 0 )
    {
-      /* No resolution argument */
-      if( ! res.getRange( &min, &max, &q ) )
-      {
-	 kdDebug(29000) << "Could not retrieve resolution range!" << endl;
-	 min = 75.0; // Hope that every scanner can 75 
-      }      
+       /* No resolution argument */
+       if( ! res.getRange( &min, &max, &q ) )
+       {
+	   if( ! res.getRangeFromList ( &min, &max, &q ) )
+	   {
+	       kdDebug(29000) << "Could not retrieve resolution range!" << endl;
+	       min = 75.0; // Hope that every scanner can 75
+	   }
+       }
+       kdDebug(29000) << "Minimum Range: " << min << ", Maximum Range: " << max << endl;
+
       if( min > MIN_PREVIEW_DPI )
 	 set_dpi = (int) min;
       else
@@ -759,7 +769,19 @@ KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
  * prepareScan tries to set as much as parameters as possible.
  *
  **/
-#define NOTIFIER(X) ( X == 0  ? ' ' : 'X' )
+#define NOTIFIER(X) optionNotifyString(X)
+
+QString KScanDevice::optionNotifyString( int i ) const
+{
+    const QString sOff = "        |";
+    const QString sOn  = "   X    |";   
+    if( i > 0 )
+    {
+	return sOn;
+    }
+    return sOff;
+}
+
 
 void KScanDevice::prepareScan( void )
 {
@@ -768,9 +790,9 @@ void KScanDevice::prepareScan( void )
     kdDebug(29000) << "########################################################################################################" << endl;
     kdDebug(29000) << "Scanner: " << scanner_name << endl;
     kdDebug(29000) << "         " << getScannerName() << endl;
-    kdDebug(29000) << "----------------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+" << endl;
-    kdDebug(29000) << " Option-Name          | SOFT_SEL  | HARD_SEL  | SOFT_DET  | EMULATED  | AUTOMATIC | INACTIVE  | ADVANCED  |" << endl;
-    kdDebug(29000) << "----------------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+" << endl;
+    kdDebug(29000) << "----------------------------------+--------+--------+--------+--------+--------+--------+--------+" << endl;
+    kdDebug(29000) << " Option-Name                      |SOFT_SEL|HARD_SEL|SOFT_DET|EMULATED|AUTOMATI|INACTIVE|ADVANCED|" << endl;
+    kdDebug(29000) << "----------------------------------+--------+--------+--------+--------+--------+--------+--------+" << endl;
 
     while ( it.current() )
     {
@@ -782,20 +804,21 @@ void KScanDevice::prepareScan( void )
        if( d )
        {
 	  int cap = d->cap;
-
-	  kdDebug(29000) << " " << it.currentKey() << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_SOFT_SELECT) ) << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_HARD_SELECT) ) << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_SOFT_DETECT) ) << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_EMULATED) ) << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_AUTOMATIC) ) << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_INACTIVE) ) << " |     " <<
-		 NOTIFIER( ((cap) & SANE_CAP_ADVANCED ) ) << " |" << endl;
+	  
+	  QString s = QString(it.currentKey()).leftJustify(32, ' ');
+	  kdDebug(29000) << " " << s << " |" <<
+		 NOTIFIER( ((cap) & SANE_CAP_SOFT_SELECT)) << 
+		 NOTIFIER( ((cap) & SANE_CAP_HARD_SELECT)) << 
+		 NOTIFIER( ((cap) & SANE_CAP_SOFT_DETECT)) << 
+		 NOTIFIER( ((cap) & SANE_CAP_EMULATED)   ) << 
+		 NOTIFIER( ((cap) & SANE_CAP_AUTOMATIC)  ) << 
+		 NOTIFIER( ((cap) & SANE_CAP_INACTIVE)   ) << 
+		 NOTIFIER( ((cap) & SANE_CAP_ADVANCED )  )    << endl;
 
        }
        ++it;
     }
-    kdDebug(29000) << "----------------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+" << endl;
+    kdDebug(29000) << "----------------------------------+--------+--------+--------+--------+--------+--------+--------+" << endl;
 
     KScanOption pso( SANE_NAME_PREVIEW );
     const QString q = pso.get();
@@ -812,12 +835,12 @@ void KScanDevice::prepareScan( void )
 KScanStat KScanDevice::acquire( const QString& filename )
 {
     KScanOption *so = 0;
- 	
+
     if( filename.isEmpty() )
     {
  	/* *real* scanning - apply all Options and go for it */
  	prepareScan();
- 	
+
  	for( so = gui_elements.first(); so; so = gui_elements.next() )
  	{
  	    if( so->active() )
@@ -830,7 +853,7 @@ KScanStat KScanDevice::acquire( const QString& filename )
  	        kdDebug(29000) << "Option <" << so->getName() << "> is not active !" << endl;
  	    }
  	}
-	return( acquire_data( false ));	
+	return( acquire_data( false ));
     }
     else
     {
@@ -872,7 +895,7 @@ KScanStat KScanDevice::createNewImage( SANE_Parameters *p )
       img->setColor( 0, qRgb( 0,0,0));
       img->setColor( 1, qRgb( 255,255,255) );
     }
-  }	
+  }
   else if( p->depth == 8 )
   {
     // 8 bit rgb-Picture
@@ -883,7 +906,7 @@ KScanStat KScanDevice::createNewImage( SANE_Parameters *p )
       if( img )
       {
 	img->setNumColors( 256 );
-	
+
 	for(int i = 0; i<256; i++)
 	  img->setColor(i, qRgb(i,i,i));
       }
@@ -915,7 +938,7 @@ KScanStat KScanDevice::acquire_data( bool isPreview )
    scanningPreview = isPreview;
 
    emit sigScanStart();
-   
+
    sane_stat = sane_start( scanner_handle );
    if( sane_stat == SANE_STATUS_GOOD )
    {
@@ -967,12 +990,12 @@ KScanStat KScanDevice::acquire_data( bool isPreview )
    /* Signal for a progress dialog */
    emit( sigScanProgress( 0 ) );
    emit( sigAcquireStart() );
-   
+
    if( stat == KSCAN_OK )
-   {	
+   {
       /* initiates Redraw of the Progress-Window */
       qApp->processEvents();
-   }	
+   }
 
    if( stat == KSCAN_OK )
    {
@@ -987,14 +1010,14 @@ KScanStat KScanDevice::acquire_data( bool isPreview )
        *  this status might be changed by pressing Stop on a GUI-Dialog displayed during scan */
       if( sane_set_io_mode( scanner_handle, SANE_TRUE ) == SANE_STATUS_GOOD )
       {
-	
+
 	 int fd = 0;
 	 if( sane_get_select_fd( scanner_handle, &fd ) == SANE_STATUS_GOOD )
 	 {
 	    sn = new QSocketNotifier( fd, QSocketNotifier::Read, this );
 	    QObject::connect( sn, SIGNAL(activated(int)),
 			      this, SLOT( doProcessABlock() ) );
-	
+
 	 }
       }
       else
@@ -1042,14 +1065,14 @@ void KScanDevice::loadOptionSet( KScanOptSet *optSet )
       KScanOption *so = it.current();
       if( ! so->initialised() )
 	 kdDebug(29000) << so->getName() << " is not initialised" << endl;
-      
+
       if( ! so->active() )
 	 kdDebug(29000) << so->getName() << " is not active" << endl;
 
       if( so && so->active() && so->initialised())
       {
-	 const QString qq = so->get();	
-	
+	 const QString qq = so->get();
+
 	 kdDebug(29000) << "Post-Scan-apply <" << it.currentKey() << ">: " << qq << endl;
 	 apply( so );
       }
@@ -1096,7 +1119,7 @@ void KScanDevice::slScanFinished( KScanStat status )
       {
 	 emit( sigNewImage( img ));
       }
-   }	
+   }
 
 
    sane_cancel(scanner_handle);
@@ -1172,7 +1195,7 @@ void KScanDevice::doProcessABlock( void )
 	      if( sane_scan_param.lines < 1 ) break;
 	      bytes_written += rest_bytes; // die bergebliebenen Bytes dazu
 	      rest_bytes = bytes_written % 3;
-	
+
 	      for( val = 0; val < ((bytes_written-rest_bytes) / 3); val++ )
 	      {
 		 red   = *rptr++;
@@ -1192,7 +1215,7 @@ void KScanDevice::doProcessABlock( void )
 	      {
 		 *(data+val) = *rptr++;
 	      }
-	
+
 	      break;
 	   case SANE_FRAME_GRAY:
 	      for( val = 0; val < bytes_written ; val++ )
@@ -1225,8 +1248,8 @@ void KScanDevice::doProcessABlock( void )
 		       }
 		    }
 		 }
-	      }	
-	
+	      }
+
 	      break;
 	   case SANE_FRAME_RED:
 	   case SANE_FRAME_GREEN:
@@ -1238,17 +1261,17 @@ void KScanDevice::doProcessABlock( void )
 		 {
 		    pixel_y++; pixel_x = 0;
 		 }
-	
+
 		 if( pixel_y < sane_scan_param.lines )
 		 {
-		
+
 		    col = img->pixel( pixel_x, pixel_y );
-		
+
 		    red   = qRed( col );
 		    green = qGreen( col );
 		    blue  = qBlue( col );
 		    chan  = *rptr++;
-		
+
 		    switch( sane_scan_param.format )
 		    {
 		       case SANE_FRAME_RED :
@@ -1275,7 +1298,7 @@ void KScanDevice::doProcessABlock( void )
 	      break;
 	} /* switch */
 
-	
+
 	if( (sane_scan_param.lines > 0) && (sane_scan_param.lines * pixel_y> 0) )
 	{
 	   int progress =  (int)((double)MAX_PROGRESS / (double) sane_scan_param.lines *
@@ -1315,10 +1338,10 @@ void KScanDevice::doProcessABlock( void )
      if ( sane_scan_param.last_frame)
      {
 	/** Everythings okay, the picture is ready **/
-	kdDebug(29000) << "last frame reached - scan successfull" << endl;
+	kdDebug(29000) << "last frame reached - scan successful" << endl;
 	scanStatus = SSTAT_SILENT;
 	emit( sigScanFinished( KSCAN_OK ));
-     }	
+     }
      else
      {
 	/** EOF und nicht letzter Frame -> Parameter neu belegen und neu starten **/
@@ -1329,7 +1352,7 @@ void KScanDevice::doProcessABlock( void )
   }
 
   if( sane_stat == SANE_STATUS_CANCELLED )
-  {	
+  {
      scanStatus = SSTAT_STOP_NOW;
      kdDebug(29000) << "Scan was cancelled" << endl;
 
@@ -1346,7 +1369,7 @@ void KScanDevice::slSaveScanConfigSet( const QString& setName, const QString& de
    if( setName.isEmpty() || setName.isNull()) return;
 
    kdDebug(29000) << "Saving Scan Configuration" << setName << endl;
-   
+
    KScanOptSet optSet( DEFAULT_OPTIONSET );
    getCurrentOptions( &optSet );
 
@@ -1358,7 +1381,7 @@ void KScanDevice::slSaveScanConfigSet( const QString& setName, const QString& de
 void  KScanDevice::getCurrentOptions( KScanOptSet *optSet )
 {
    if( ! optSet ) return;
-   
+
    for( KScanOption *so = gui_elements.first(); so; so = gui_elements.next())
    {
       kdDebug(29000) << "Storing <" << so->getName() << ">" << endl;
@@ -1380,8 +1403,36 @@ void  KScanDevice::getCurrentOptions( KScanOptSet *optSet )
       ++it;
    }
 }
-   
 
+QString KScanDevice::getConfig( const QString& key, const QString& def ) const
+{
+    QString confFile = SCANNER_DB_FILE;
+
+    KSimpleConfig scanConfig( confFile, true );
+    scanConfig.setGroup( shortScannerName() );
+
+    return scanConfig.readEntry( key, def );
+}
+
+void KScanDevice::slStoreConfig( const QString& key, const QString& val )
+{
+    QString confFile = SCANNER_DB_FILE;
+    QString scannerName = shortScannerName();
+
+    if( scannerName.isEmpty() || scannerName == UNDEF_SCANNERNAME )
+    {
+        kdDebug(29000) << "Skipping config write, scanner name is empty!" << endl;
+    }
+    else
+    {
+        kdDebug(29000) << "Storing config " << key << " in Group " << scannerName << endl;
+
+        KSimpleConfig scanConfig( confFile );
+        scanConfig.setGroup( scannerName );
+        scanConfig.writeEntry( key, val );
+        scanConfig.sync();
+    }
+}
 
 
 bool KScanDevice::scanner_initialised = false;
