@@ -16,7 +16,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   $Id: kscanoption.cpp,v 1.20 2003/07/01 16:22:08 mueller Exp $
+   $Id: kscanoption.cpp,v 1.22 2004/04/08 14:32:00 freitag Exp $
 */
 
 #include <stdlib.h>
@@ -181,7 +181,7 @@ KScanOption::KScanOption( const KScanOption &so ) :
    
    if( ! ( desc && name ) )
    {
-      kdWarning( 29000) << "Trying to copy a not healty option" << endl;
+      kdWarning( 29000) << "Trying to copy a not healthy option (no name nor desc)" << endl;
       return;
    }
    
@@ -996,12 +996,53 @@ QStrList KScanOption::getList( ) const
 }
 
 
+
+bool KScanOption::getRangeFromList( double *min, double *max, double *q ) const
+{
+   if( !desc ) return( false );
+   bool ret = true;
+
+   if ( desc->constraint_type == SANE_CONSTRAINT_WORD_LIST ) {
+	    // Try to read resolutions from word list
+	    kdDebug(29000) << "Resolutions are in a word list" << endl;
+            const SANE_Int *sint =  desc->constraint.word_list;
+            int amount_vals = *sint; sint ++;
+	    double value;
+	    *min = 0;
+	    *max = 0;
+	    *q = -1; // What is q?
+
+            for( int i=0; i < amount_vals; i++ ) {
+		if( desc->type == SANE_TYPE_FIXED ) {
+		    value = (double) SANE_UNFIX( *sint );
+		} else {
+		    value = *sint;
+		}
+		if ((*min > value) || (*min == 0)) *min = value;
+		if ((*max < value) || (*max == 0)) *max = value;
+		
+		if( min != 0 && max != 0 && max > min ) {
+		    double newq = max - min;
+		    *q = newq;
+		}
+		sint++;
+            }
+   } else {
+      kdDebug(29000) << "getRangeFromList: No list type " << desc->name << endl;
+      ret = false;
+    }
+    return( ret );
+}
+
+
+
 bool KScanOption::getRange( double *min, double *max, double *q ) const
 {
    if( !desc ) return( false );
    bool ret = true;
 
-   if( desc->constraint_type == SANE_CONSTRAINT_RANGE) {
+   if( desc->constraint_type == SANE_CONSTRAINT_RANGE ||
+	   desc->constraint_type == SANE_CONSTRAINT_WORD_LIST ) {
       const SANE_Range *r = desc->constraint.range;
 
       if( desc->type == SANE_TYPE_FIXED ) {
