@@ -16,7 +16,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   $Id: scanparams.cpp,v 1.45 2003/08/25 14:06:30 freitag Exp $
+   $Id: scanparams.cpp,v 1.46 2004/06/26 10:13:01 binner Exp $
 */
 
 #include <sane/saneopts.h>
@@ -587,7 +587,7 @@ void ScanParams::slFileSelect( void )
    }
    else
    {
-      filter.append( i18n( "*.pnm|PNM image files (*.pnm)") );
+      filter.append( i18n( "*.pnm|PNM Image Files (*.pnm)") );
    }
 
 
